#ifndef GUI_OBJUTILS___NONASN_TRACK_DATA__HPP
#define GUI_OBJUTILS___NONASN_TRACK_DATA__HPP

/*  $Id: nonasn_track_data.hpp 45881 2021-01-05 03:10:02Z rudnev $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data, the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties, express or implied, including
 *  warranties of performance, merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 * Authors:  Vladislav Evgeniev
 *
 * File Description:
 *    Interface for non-ASN1 tracks.
 *
 */

#include <corelib/ncbistd.hpp>
#include <string>
#include <vector>
#include <gui/objutils/annot_info.hpp>
#include <objects/gbproj/ProjectItem.hpp>
#include <gui/gui.hpp>

 /** @addtogroup GUI_OBJUTILS
  *
  * @{
  */


BEGIN_NCBI_SCOPE

#define EXT_POINT__NONASN_TRACK_DATA_FACTORY "nonasn_track_data_factory"

class NCBI_GUIOBJUTILS_EXPORT INonAsnTrackData
{
public:
    using TDataBlob = vector<char>;
    using TAnnots = vector<string>;

    virtual const string& GetKey() const = 0;
    virtual const string& GetSubkey() const = 0;
    virtual const string& GetDisplayName() const = 0;  
    virtual bool GetShown() const = 0;
    virtual const string& GetCategory() const = 0;
    virtual const string& GetSubcategory() const = 0;
    virtual const TAnnots& GetAnnots() const = 0;

    // Returns the actual data
    virtual const TDataBlob& GetBlob() const = 0;

    virtual const string GetDataSourceTypeName() const = 0;

	// performs some sort of a search in data and place results into a list of objects
	virtual void DoSearch(const objects::CSeq_id* seq_id, const string& sSearchTerm, list<CRef<CObject>>& results) const = 0;

    virtual ~INonAsnTrackData() {};
};

class NCBI_GUIOBJUTILS_EXPORT INonAsnTrackDataFactory
{
public:
    virtual INonAsnTrackData* CreateTrackData(const objects::CSeq_id &seq_id, const objects::CProjectItem &item) const = 0;

    virtual ~INonAsnTrackDataFactory() {};
};

class NCBI_GUIOBJUTILS_EXPORT INonAsnDataRegistry
{
public:
    using TDataBlob = INonAsnTrackData::TDataBlob;

    virtual void AddNonAsnData(const INonAsnTrackData &data) = 0;
    
    virtual void RemoveNonAsnData(const INonAsnTrackData &data) = 0;
    
    virtual bool HasNonAsnData(const string &annot_name) const = 0;
    
    virtual const TDataBlob& GetNonAsnDataBlob(const string &annot_name) const = 0;

    virtual ~INonAsnDataRegistry() {};
};

class NCBI_GUIOBJUTILS_EXPORT INonAsnDataSource
{
public:
    using TDataBlob = INonAsnTrackData::TDataBlob;

    virtual void SetNonAsnDataRegistry(const INonAsnDataRegistry *registry) = 0;

    virtual bool HasNonAsnData(const string &annot_name) const = 0;

    virtual const TDataBlob& GetNonAsnDataBlob(const string &annot_name) const = 0;

	// performs some sort of a search in data and place results into a list of objects
	virtual void DoSearch(const string &annot_name, const string& sSearchTerm, list<CRef<CObject>>& results) const = 0;

	virtual ~INonAsnDataSource() {};
};

END_NCBI_SCOPE

/* @} */

#endif  /// GUI_OBJUTILS___NONASN_TRACK_DATA__HPP

