#ifndef GUI_PACKAGES___TABLE_DATA_VCF__HPP
#define GUI_PACKAGES___TABLE_DATA_VCF__HPP

/*  $Id: table_data_vcf.hpp 45881 2021-01-05 03:10:02Z rudnev $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data, the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties, express or implied, including
 *  warranties of performance, merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 * Authors: Dmitry Rudnev
 *
 * File Description:
 *
 */
#include <gui/utils/event_handler.hpp>
#include <gui/utils/app_job_dispatcher.hpp>
#include <gui/objutils/table_data_base.hpp>
#include <gui/objutils/snp_gui.hpp>
#include <objects/seqalign/Seq_align.hpp>
#include <objects/seqloc/Seq_loc.hpp>
#include <objmgr/scope.hpp>
#include <objmgr/bioseq_handle.hpp>
#include <gui/objutils/nonasn_track_data.hpp>

BEGIN_NCBI_SCOPE

class CVcfTableEvent : public CEvent
{
public:
    enum EEventType {
        eDataChanged,
    };

    CVcfTableEvent(ITableData& tableData)
        : CEvent(eDataChanged), m_TableData(&tableData) {}
        
    CIRef<ITableData> GetTableData() { return m_TableData; }

private:
    CIRef<ITableData> m_TableData;
};

class CTableDataVcf 
    : public CObjectEx,
      public CEventHandler,
      public CTableDataBase
{
public:
    // number of entries and order must correspond with s_ColNames
    enum EColContents {
		ePosition = 0,
		eId,
		eVariationType,
		eAlleles,
        eMaxFixedColNum = eAlleles
    };

    // data load status
    enum EDataLoadStatus {
        eNotReady,
        eFailed,
        eReady
    };

    // maximal amount of VCF rows we should realistically see in a VCF table
	//!! until dynamic loading is implemented
    static const size_t c_MaxRows;

	CTableDataVcf(objects::CScope& scope,
		objects::CBioseq_Handle handle, 
		const TSeqRange& range,
		const INonAsnTrackData::TDataBlob& vcf_blob);
    ~CTableDataVcf();

    void LoadData();

    /// @name ITableData interface implementation
    /// @{
    virtual ColumnType GetColumnType(size_t col) const;
    virtual bool       IsHtmlColumn(size_t col) const;
    virtual string     GetColumnLabel(size_t col) const;

    virtual size_t GetRowsCount() const;
    virtual size_t GetColsCount() const;

    virtual void GetStringValue(size_t row, size_t col, string& value) const;
    virtual string GetHtmlValue(size_t /*row*/, size_t /*col*/) const;
    virtual long   GetIntValue(size_t row, size_t col) const;
    virtual double GetRealValue(size_t row, size_t col) const;
    
	// the object will be represented as a CVariation
	virtual SConstScopedObject GetObjectValue(size_t row, size_t col) const;
    virtual SConstScopedObject GetRowObject(size_t row) const;

    CRef<objects::CScope> GetScope() { return m_Scope; }
    /// @}

    typedef struct {
		vector<string> m_InfoFields;
		vector<string> m_SampleNames;
		vector<CRef<objects::CVcfVariant>> m_VcfVariants;
	} TVcfDataList;

    void OnJobNotification(CEvent* evt);

    DECLARE_EVENT_MAP();

private:
	vector<string> m_ColNames;
	size_t m_FirstInfoCol{0};
	size_t m_FirstSampleCol{0};
	CConstRef<objects::CVcfVariant> x_GetRowData(size_t row) const;
    CRef<objects::CScope> m_Scope;
	objects::CBioseq_Handle m_Handle;           ///< target sequence
	TSeqRange               m_Range;            ///< target range
	const INonAsnTrackData::TDataBlob& m_VcfBlob;
	CRef<CObjectFor<TVcfDataList>> m_listVcfs;

	int m_hJob{0};     // Job ID
    EDataLoadStatus m_DataLoadStatus;
};


END_NCBI_SCOPE

#endif  // GUI_PACKAGES___TABLE_DATA_VCF__HPP
