#ifndef GUI_WIDGETS_LOADERS__COLUMNAR_VCF_EX__HPP
#define GUI_WIDGETS_LOADERS__COLUMNAR_VCF_EX__HPP

/*  $Id: columnar_vcf_ex.hpp 45881 2021-01-05 03:10:02Z rudnev $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data, the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties, express or implied, including
 *  warranties of performance, merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 * Authors: Andrea Asztalos
 *
 * File Description: Exception class for processing VCF file
 *
 */

#include <corelib/ncbiexpt.hpp>
#include <serial/serialbase.hpp>
#include <gui/gui_export.h>


BEGIN_NCBI_SCOPE


class NCBI_GUIWIDGETS_LOADERS_EXPORT CColumnarVCFException : EXCEPTION_VIRTUAL_BASE public CException {
public:
    // Error codes for parsing the VCF file
    //
    enum EErrCode {
        eParseTabExpected,
        eMismatchChromosome,
        eParsePosFailed

    };

    // Convertion of error code to user domain message
    // 
    virtual const char* GetErrCodeString(void) const override {
        switch (GetErrCode()) {
        case eParseTabExpected:              return "eParseTabExpected";
        case eMismatchChromosome:            return "eMismatchChromosome";
        case eParsePosFailed:                return "eParsePosFailed";
        
        default: return CException::GetErrCodeString();
        }
    }

    NCBI_EXCEPTION_DEFAULT(CColumnarVCFException, CException);
};

// Errors related to serialization and deserialization of the VCF data
class NCBI_GUIWIDGETS_LOADERS_EXPORT CVCFDataException : EXCEPTION_VIRTUAL_BASE public CException
{
public:
    enum EErrCode {
        eSerializationFailed,
        eDeserializationFailed,
        eFileCorrupted,

    };

    // Convertion of error code to user domain message
    // 
    virtual const char* GetErrCodeString(void) const override {
        switch (GetErrCode()) {
        case eSerializationFailed:              return "eSerializationFailed";
        case eDeserializationFailed:            return "eDeserializationFailed";
        case eFileCorrupted:                    return "eFileCorrupted";

        default: return CException::GetErrCodeString();
        }
    }

    NCBI_EXCEPTION_DEFAULT(CVCFDataException, CException);
};


END_NCBI_SCOPE


#endif  // GUI_WIDGETS_LOADERS__COLUMNAR_VCF_EX__HPP

