#ifndef GUI_WIDGETS___LOADERS___COLUMNAR_VCF_ITERATOR__HPP_
#define GUI_WIDGETS___LOADERS___COLUMNAR_VCF_ITERATOR__HPP_
/*  $Id: columnar_vcf_iterator.hpp 45881 2021-01-05 03:10:02Z rudnev $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data, the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties, express or implied, including
 *  warranties of performance, merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 * Authors:  Andrea Asztalos
 *
 * File Description:
 *   
 *
 */

#include <corelib/ncbistd.hpp>
#include <gui/gui_export.h>
#include <util/range.hpp>
#include <gui/widgets/loaders/columnar_vcf_variants_base.hpp>
#include <unordered_map>


BEGIN_NCBI_SCOPE


class NCBI_GUIWIDGETS_LOADERS_EXPORT CVCFVariant_CI
{
public:
    using TSparseStrVector = CVCFVariantsBase::TSparseStrVector;
    using TIndexVector = CVCFVariantsBase::TIndexVector;
    using TSparseOptVector = CVariantDescriptors::TSparseOptVector;

    CVCFVariant_CI(const CVCFVariantsBase& var_list, const TSeqRange& range);
    
    ~CVCFVariant_CI() {}

    CVCFVariant_CI& begin();
    CVCFVariant_CI& operator++();
    bool IsValid() const;

    DECLARE_OPERATOR_BOOL(IsValid());

    bool operator==(const CVCFVariant_CI& other) const;
    bool operator!=(const CVCFVariant_CI& other) const;

    size_t GetSize() const;
    
    unsigned GetPosition() const;
    string GetVariantID() const { return x_GetData(CVCFVariantsBase::sm_ID); }
    string GetRef() const { return x_GetData(CVCFVariantsBase::sm_REF); }
    string GetAlt() const { return x_GetData(CVCFVariantsBase::sm_ALT); }
    string GetQual() const { return x_GetData(CVCFVariantsBase::sm_QUAL); }
    string GetFilter() const { return x_GetData(CVCFVariantsBase::sm_FILTER); }

    bool IsSetFormat() const { return x_IsSetData(CVCFVariantsBase::sm_FORMAT); }
    string GetFormat() const { return x_GetData(CVCFVariantsBase::sm_FORMAT); }
    
    bool   IsSetInfo() const;
    string GetInfo() const;

    bool   IsSetInfoField(const string& field_name) const;
    string GetInfoField(const string& field_name) const;

    bool IsSetSample(const string& name) const;
    string GetSample(const string& name) const;

    CVCFVariant_CI(const CVCFVariant_CI& other) = delete;
    CVCFVariant_CI& operator=(const CVCFVariant_CI& other) = delete;

private:
    bool x_IsSetData(const string& col_name) const;
    string x_GetData(const string& col_name) const;

    CConstRef<CVCFVariantsBase> m_Variants;

    using TSparseCI = TSparseStrVector::const_iterator;
    unordered_map<string, unique_ptr<TSparseCI>> m_Iterators;

    using TInfoCI = TSparseOptVector::const_iterator;
    unordered_map<string, unique_ptr<TInfoCI>> m_InfoIterators;
    unordered_map<string, unique_ptr<TInfoCI>> m_SampleIterators;

    // denotes a semi-open interval [m_Start, m_Stop) 
    size_t m_Start{ 0 };  // line index 
    size_t m_Stop{ 0 };   // line index
    size_t m_Current{ 0 };
};

END_NCBI_SCOPE


#endif 
    //GUI_WIDGETS___LOADERS___COLUMNAR_VCF_ITERATOR__HPP_
