#ifndef GUI_WIDGETS___LOADERS___VCF_ASSEMBLY_SEL_PANEL__HPP
#define GUI_WIDGETS___LOADERS___VCF_ASSEMBLY_SEL_PANEL__HPP
/*  $Id: vcf_assembly_sel_panel.hpp 45881 2021-01-05 03:10:02Z rudnev $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data,  the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties,  express or implied,  including
 *  warranties of performance,  merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 * Authors: Vladislav Evgeniev 
 */

#include <corelib/ncbistd.hpp>
#include <corelib/ncbiobj.hpp>
#include <gui/gui_export.h>
#include <gui/objutils/reg_settings.hpp>
#include <gui/utils/job_future.hpp>
#include <wx/panel.h>
#include <set>

class wxStaticText;

/*!
 * Control identifiers
 */

////@begin control identifiers
#define SYMBOL_CVCFASSEMBLYSELECTIONPANEL_STYLE wxTAB_TRAVERSAL
#define SYMBOL_CVCFASSEMBLYSELECTIONPANEL_TITLE _("Assembly Options")
#define SYMBOL_CVCFASSEMBLYSELECTIONPANEL_IDNAME ID_VCFASSEMBLYSELPANEL
#define SYMBOL_CVCFASSEMBLYSELECTIONPANEL_SIZE wxSize(400, 300)
#define SYMBOL_CVCFASSEMBLYSELECTIONPANEL_POSITION wxDefaultPosition
////@end control identifiers


BEGIN_NCBI_SCOPE

class CAssemblySelPanel;
class CColumnarVCFReader;

/*!
 * CVcfAssemblySelectionPanel class declaration
 */

class NCBI_GUIWIDGETS_LOADERS_EXPORT CVcfAssemblySelectionPanel: public wxPanel
{    
    DECLARE_DYNAMIC_CLASS( CVcfAssemblySelectionPanel )
    DECLARE_EVENT_TABLE()

public:
    /// Constructors
    CVcfAssemblySelectionPanel();
    CVcfAssemblySelectionPanel( wxWindow* parent, CColumnarVCFReader* reader,
        const wxString &input_vcf,
        wxWindowID id = SYMBOL_CVCFASSEMBLYSELECTIONPANEL_IDNAME, 
        const wxPoint& pos = SYMBOL_CVCFASSEMBLYSELECTIONPANEL_POSITION, 
        const wxSize& size = SYMBOL_CVCFASSEMBLYSELECTIONPANEL_SIZE, 
        long style = SYMBOL_CVCFASSEMBLYSELECTIONPANEL_STYLE );

    /// Creation
    bool Create( wxWindow* parent,
        wxWindowID id = SYMBOL_CVCFASSEMBLYSELECTIONPANEL_IDNAME, 
        const wxPoint& pos = SYMBOL_CVCFASSEMBLYSELECTIONPANEL_POSITION, 
        const wxSize& size = SYMBOL_CVCFASSEMBLYSELECTIONPANEL_SIZE, 
        long style = SYMBOL_CVCFASSEMBLYSELECTIONPANEL_STYLE );

    /// Destructor
    ~CVcfAssemblySelectionPanel();

    /// Initialises member variables
    void Init();

    /// Creates the controls and sizers
    void CreateControls();

    bool IsInputValid();

    /// @name IRegSettings interface implementation
    /// @{
    virtual void    SetRegistryPath(const string& path) { m_RegPath = path; }
    virtual void    LoadSettings();
    virtual void    SaveSettings() const;
    /// @}

////@begin CVcfAssemblySelectionPanel event handler declarations

    /// wxEVT_IDLE event handler for ID_VCFASSEMBLYSELPANEL
    void OnIdle( wxIdleEvent& event );

////@end CVcfAssemblySelectionPanel event handler declarations

////@begin CVcfAssemblySelectionPanel member function declarations
    void SetNewFile(const wxString &input_vcf);

    /// Retrieves bitmap resources
    wxBitmap GetBitmapResource( const wxString& name );

    /// Retrieves icon resources
    wxIcon GetIconResource( const wxString& name );
////@end CVcfAssemblySelectionPanel member function declarations

    /// Should we show tooltips?
    static bool ShowToolTips();
    
    /// Control identifiers
    enum {
        ID_VCFASSEMBLYSELPANEL = 10074,
        ID_PANEL9 = 10014
    };

    
////@begin CVcfAssemblySelectionPanel member variables
    wxStaticText* m_VcfAssembly;
    CAssemblySelPanel* m_AssemblyPanel;

private:
    void x_ReadAssembly();

    CColumnarVCFReader* m_VCFReader{ nullptr };
    wxString m_InputVcf;
    string m_RegPath;
    job_future<string> m_VcfAssemblyFuture;
////@end CVcfAssemblySelectionPanel member variables
};


END_NCBI_SCOPE

#endif // GUI_WIDGETS___LOADERS___VCF_ASSEMBLY_SEL_PANEL__HPP

