#ifndef GUI_WIDGETS___LOADERS___VCF_COLUMN_SEL_PANEL__HPP
#define GUI_WIDGETS___LOADERS___VCF_COLUMN_SEL_PANEL__HPP
/*  $Id: vcf_column_sel_panel.hpp 45881 2021-01-05 03:10:02Z rudnev $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data,  the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties,  express or implied,  including
 *  warranties of performance,  merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 * Authors: Andrea Asztalos
 */

#include <corelib/ncbistd.hpp>
#include <gui/gui_export.h>
#include <wx/panel.h>


BEGIN_NCBI_SCOPE

class wxCheckedListCtrl;
struct SVcfFieldData;
/*!
 * Control identifiers
 */

////@begin control identifiers
#define SYMBOL_CVCFCOLUMNSELECTIONPANEL_STYLE wxTAB_TRAVERSAL
#define SYMBOL_CVCFCOLUMNSELECTIONPANEL_TITLE _("VCF Column Selection")
#define SYMBOL_CVCFCOLUMNSELECTIONPANEL_IDNAME ID_CVCFCOLUMNSELECTIONPANEL
#define SYMBOL_CVCFCOLUMNSELECTIONPANEL_SIZE wxSize(400, 300)
#define SYMBOL_CVCFCOLUMNSELECTIONPANEL_POSITION wxDefaultPosition
////@end control identifiers


/*!
 * CVcfColumnSelectionPanel class declaration
 */

class NCBI_GUIWIDGETS_LOADERS_EXPORT CVcfColumnSelectionPanel: public wxPanel
{    
    DECLARE_DYNAMIC_CLASS( CVcfColumnSelectionPanel )
    DECLARE_EVENT_TABLE()

public:
    /// Constructors
    CVcfColumnSelectionPanel();
    CVcfColumnSelectionPanel( wxWindow* parent,
        wxWindowID id = SYMBOL_CVCFCOLUMNSELECTIONPANEL_IDNAME, 
        const wxPoint& pos = SYMBOL_CVCFCOLUMNSELECTIONPANEL_POSITION, 
        const wxSize& size = SYMBOL_CVCFCOLUMNSELECTIONPANEL_SIZE, 
        long style = SYMBOL_CVCFCOLUMNSELECTIONPANEL_STYLE );

    /// Creation
    bool Create( wxWindow* parent, 
        wxWindowID id = SYMBOL_CVCFCOLUMNSELECTIONPANEL_IDNAME, 
        const wxPoint& pos = SYMBOL_CVCFCOLUMNSELECTIONPANEL_POSITION, 
        const wxSize& size = SYMBOL_CVCFCOLUMNSELECTIONPANEL_SIZE, 
        long style = SYMBOL_CVCFCOLUMNSELECTIONPANEL_STYLE );

    /// Destructor
    ~CVcfColumnSelectionPanel();

    /// Initialises member variables
    void Init();

    /// Creates the controls and sizers
    void CreateControls();

////@begin CVcfColumnSelectionPanel event handler declarations

    /// wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_SelectAllINFO
    void OnSelectAllClick( wxCommandEvent& event );

    /// wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_DeselectAllINFO
    void OnDeselectAllClick( wxCommandEvent& event );

////@end CVcfColumnSelectionPanel event handler declarations

////@begin CVcfColumnSelectionPanel member function declarations
    void SetInfoFields(const set<CConstRef<SVcfFieldData>>& fields);
    void GetSelectedInfoFields(set<string>& selected);
    bool AllAreChecked() const;

    /// Retrieves bitmap resources
    wxBitmap GetBitmapResource( const wxString& name );

    /// Retrieves icon resources
    wxIcon GetIconResource( const wxString& name );
////@end CVcfColumnSelectionPanel member function declarations

    /// Should we show tooltips?
    static bool ShowToolTips();

    /// Control identifiers
    enum {
        ID_CVCFCOLUMNSELECTIONPANEL = 10345,
        ID_INFOFIELDSLIST = 10346,
        ID_SelectAllINFO = 10347,
        ID_DeselectAllINFO = 10348
    };

private:
    wxCheckedListCtrl* m_InfoSubCols;
    set<CConstRef<SVcfFieldData>> m_FieldData;
};

END_NCBI_SCOPE

#endif
    // GUI_WIDGETS___LOADERS___VCF_COLUMN_SEL_PANEL__HPP
