#ifndef GUI_WIDGETS___LOADERS___VCF_REF_SEQUENCES_PANEL__HPP
#define GUI_WIDGETS___LOADERS___VCF_REF_SEQUENCES_PANEL__HPP

/*  $Id: vcf_ref_sequences_panel.hpp 46175 2021-01-28 15:47:23Z evgeniev $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data,  the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties,  express or implied,  including
 *  warranties of performance,  merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 * Authors: Vladislav Evgeniev 
 */

#include <corelib/ncbistd.hpp>
#include <corelib/ncbiobj.hpp>

#include <wx/panel.h>
#include <wx/checklst.h>

#include <gui/gui_export.h>

#include <objects/seqloc/Seq_id.hpp>
#include <objmgr/scope.hpp>
#include <objtools/readers/iidmapper.hpp>

/*!
 * Includes
 */

////@begin includes
#include "wx/srchctrl.h"
////@end includes

/*!
 * Forward declarations
 */

////@begin forward declarations
class wxSearchCtrl;
class wxRadioBox;
class wxButton;
////@end forward declarations

/*!
 * Control identifiers
 */

////@begin control identifiers
#define SYMBOL_CVCFREFSEQUENCESPANEL_STYLE wxTAB_TRAVERSAL
#define SYMBOL_CVCFREFSEQUENCESPANEL_TITLE _("VCF Referenced Sequences")
#define SYMBOL_CVCFREFSEQUENCESPANEL_IDNAME ID_CVCFREFSEQUENCESPANEL
#define SYMBOL_CVCFREFSEQUENCESPANEL_SIZE wxSize(400, 300)
#define SYMBOL_CVCFREFSEQUENCESPANEL_POSITION wxDefaultPosition
////@end control identifiers

BEGIN_NCBI_SCOPE

/*!
 * CVcfRefSequencesPanel class declaration
 */

class NCBI_GUIWIDGETS_LOADERS_EXPORT CVcfRefSequencesPanel: public wxPanel
{    
    DECLARE_DYNAMIC_CLASS( CVcfRefSequencesPanel )
    DECLARE_EVENT_TABLE()

public:

    /// Constructors
    CVcfRefSequencesPanel();
    CVcfRefSequencesPanel( wxWindow* parent,
        wxWindowID id = SYMBOL_CVCFREFSEQUENCESPANEL_IDNAME, 
        const wxPoint& pos = SYMBOL_CVCFREFSEQUENCESPANEL_POSITION, 
        const wxSize& size = SYMBOL_CVCFREFSEQUENCESPANEL_SIZE, 
        long style = SYMBOL_CVCFREFSEQUENCESPANEL_STYLE );

    /// Creation
    bool Create( wxWindow* parent, 
        wxWindowID id = SYMBOL_CVCFREFSEQUENCESPANEL_IDNAME, 
        const wxPoint& pos = SYMBOL_CVCFREFSEQUENCESPANEL_POSITION, 
        const wxSize& size = SYMBOL_CVCFREFSEQUENCESPANEL_SIZE, 
        long style = SYMBOL_CVCFREFSEQUENCESPANEL_STYLE );

    /// Destructor
    ~CVcfRefSequencesPanel();

    /// Initialises member variables
    void Init();

    /// Creates the controls and sizers
    void CreateControls();

////@begin CVcfRefSequencesPanel event handler declarations

    /// wxEVT_IDLE event handler for ID_CVCFREFSEQUENCESPANEL
    void OnIdle( wxIdleEvent& event );

    /// wxEVT_COMMAND_RADIOBOX_SELECTED event handler for ID_RADIOBOX
    void OnRadioboxSelected( wxCommandEvent& event );

    /// wxEVT_COMMAND_CHECKLISTBOX_TOGGLED event handler for ID_CHECKLISTBOX
    void OnChecklistboxToggled( wxCommandEvent& event );

    /// wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_SELECT_ALL_BTN
    void OnSelectAllBtnClick( wxCommandEvent& event );

    /// wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_DESELECT_ALL_BTN
    void OnDeselectAllBtnClick( wxCommandEvent& event );

    /// wxEVT_COMMAND_SEARCHCTRL_SEARCH_BTN event handler for ID_SEARCHCTRL
    void OnSearchButtonClick( wxCommandEvent& event );

    /// wxEVT_COMMAND_SEARCHCTRL_CANCEL_BTN event handler for ID_SEARCHCTRL
    void OnSearchCancelClick( wxCommandEvent& event );

    /// wxEVT_COMMAND_TEXT_ENTER event handler for ID_SEARCHCTRL
    void OnSearchEnter( wxCommandEvent& event );

    /// wxEVT_UPDATE_UI event handler for ID_SEARCHCTRL
    void OnSearchCtrlUpdate( wxUpdateUIEvent& event );

////@end CVcfRefSequencesPanel event handler declarations

////@begin CVcfRefSequencesPanel member function declarations

    /// Retrieves bitmap resources
    wxBitmap GetBitmapResource( const wxString& name );

    /// Retrieves icon resources
    wxIcon GetIconResource( const wxString& name );
////@end CVcfRefSequencesPanel member function declarations

    void SetReferenceSeqIds(const vector<CConstRef<objects::CSeq_id> >& refSeqIds);
    void GetSelectedReferenceSeqIds(vector<CConstRef<objects::CSeq_id> >& selSeqIds) const;
    bool ImportAll() const;
    void SetGenCollAssembly(const CRef<objects::CGC_Assembly>& gc_assm);
    void SetTopLevelSeqCount(size_t count);
    /// Should we show tooltips?
    static bool ShowToolTips();

////@begin CVcfRefSequencesPanel member variables
    wxRadioBox* m_LoadSequencesRadio;
    wxCheckListBox* m_RefSeqList;
    wxButton* m_SelectAllBtn;
    wxButton* m_DeselctAllBtn;
    wxSearchCtrl* m_SearchCtrl;
    wxStaticText* m_SeqSelectText;
    /// Control identifiers
    enum {
        ID_CVCFREFSEQUENCESPANEL = 10000,
        ID_CHECKLISTBOX = 10001,
        ID_SELECT_ALL_BTN = 10002,
        ID_DESELECT_ALL_BTN = 10003,
        ID_RADIOBOX = 10004,
        ID_SEQ_SELECT_TEXT = 10005, 
        ID_SEARCHCTRL = 10024
    };
////@end CVcfRefSequencesPanel member variables

private:
    vector<CConstRef<objects::CSeq_id> > m_RefSeqIds;
    set<const objects::CSeq_id*> m_SelectedSeqIds;
    list<size_t> m_IdsWithNoDescriptions;
    CRef<objects::CScope> m_Scope;
    string m_CurrentFilter;
    unique_ptr<objects::IIdMapper> m_IdMapper;
    CRef<objects::CGC_Assembly>    m_GenCollAssm;

    void x_FillList(const string& filter);
};

END_NCBI_SCOPE

#endif // GUI_WIDGETS___LOADERS___VCF_REF_SEQUENCES_PANEL__HPP
