#ifndef GUI_WIDGETS___LOADERS___VCF_SAMPLE_SEL_PANEL__HPP
#define GUI_WIDGETS___LOADERS___VCF_SAMPLE_SEL_PANEL__HPP
/*  $Id: vcf_sample_sel_panel.hpp 45881 2021-01-05 03:10:02Z rudnev $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data,  the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties,  express or implied,  including
 *  warranties of performance,  merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 * Authors: Andrea Asztalos
 */

#include <corelib/ncbistd.hpp>
#include <gui/gui_export.h>
#include <wx/panel.h>

class wxCheckListBox;

BEGIN_NCBI_SCOPE
/*!
 * Control identifiers
 */

////@begin control identifiers
#define SYMBOL_CVCFSAMPLESELECTIONPANEL_STYLE wxTAB_TRAVERSAL
#define SYMBOL_CVCFSAMPLESELECTIONPANEL_TITLE _("VCF SAMPLES Selection")
#define SYMBOL_CVCFSAMPLESELECTIONPANEL_IDNAME ID_CVCFSAMPLESELECTIONPANEL
#define SYMBOL_CVCFSAMPLESELECTIONPANEL_SIZE wxSize(400, 300)
#define SYMBOL_CVCFSAMPLESELECTIONPANEL_POSITION wxDefaultPosition
////@end control identifiers


/*!
 * CVcfSampleSelectionPanel class declaration
 */

class NCBI_GUIWIDGETS_LOADERS_EXPORT CVcfSampleSelectionPanel: public wxPanel
{    
    DECLARE_DYNAMIC_CLASS( CVcfSampleSelectionPanel )
    DECLARE_EVENT_TABLE()

public:
    /// Constructors
    CVcfSampleSelectionPanel();
    CVcfSampleSelectionPanel( wxWindow* parent, 
        wxWindowID id = SYMBOL_CVCFSAMPLESELECTIONPANEL_IDNAME, 
        const wxPoint& pos = SYMBOL_CVCFSAMPLESELECTIONPANEL_POSITION, 
        const wxSize& size = SYMBOL_CVCFSAMPLESELECTIONPANEL_SIZE, 
        long style = SYMBOL_CVCFSAMPLESELECTIONPANEL_STYLE );

    /// Creation
    bool Create( wxWindow* parent, 
        wxWindowID id = SYMBOL_CVCFSAMPLESELECTIONPANEL_IDNAME, 
        const wxPoint& pos = SYMBOL_CVCFSAMPLESELECTIONPANEL_POSITION, 
        const wxSize& size = SYMBOL_CVCFSAMPLESELECTIONPANEL_SIZE, 
        long style = SYMBOL_CVCFSAMPLESELECTIONPANEL_STYLE );

    /// Destructor
    ~CVcfSampleSelectionPanel();

    /// Initialises member variables
    void Init();

    /// Creates the controls and sizers
    void CreateControls();

////@begin CVcfSampleSelectionPanel event handler declarations

    /// wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_SELECTALLSAMPLES
    void OnSelectallSamplesClick( wxCommandEvent& event );

    /// wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_DESELECTALLSAMPLES
    void OnDeselectallSamplesClick( wxCommandEvent& event );

////@end CVcfSampleSelectionPanel event handler declarations

////@begin CVcfSampleSelectionPanel member function declarations
    void SetSamples(const map<unsigned, string>& samples);
    void GetSelectedSamples(map<unsigned, string>& selected);
    bool AllAreChecked() const;

    /// Retrieves bitmap resources
    wxBitmap GetBitmapResource( const wxString& name );

    /// Retrieves icon resources
    wxIcon GetIconResource( const wxString& name );
////@end CVcfSampleSelectionPanel member function declarations

    /// Should we show tooltips?
    static bool ShowToolTips();

    /// Control identifiers
    enum {
        ID_CVCFSAMPLESELECTIONPANEL = 10000,
        ID_SAMPLECHKLIST = 10001,
        ID_SELECTALLSAMPLES = 10002,
        ID_DESELECTALLSAMPLES = 10003
    };
private:
////@begin CVcfSampleSelectionPanel member variables
    wxCheckListBox* m_SampleList;
    
////@end CVcfSampleSelectionPanel member variables
};

END_NCBI_SCOPE

#endif
    // GUI_WIDGETS___LOADERS___VCF_SAMPLE_SEL_PANEL__HPP
