#ifndef GUI_WIDGETS_SEQ_GRAPHIC___INTRON_GLYPH__HPP
#define GUI_WIDGETS_SEQ_GRAPHIC___INTRON_GLYPH__HPP

/*  
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data, the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties, express or implied, including
 *  warranties of performance, merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 * Authors:  Andrei Shkeda
 *
 * File Description:
 *    CIntronGlyph -- utility class representing intron feature
 * 
 */

#include <gui/gui.hpp>
#include <gui/widgets/seq_graphic/feature_glyph.hpp>


BEGIN_NCBI_SCOPE

BEGIN_objects_SCOPE
class CUser_object;
END_objects_SCOPE


class NCBI_GUIWIDGETS_SEQGRAPHIC_EXPORT CIntronGlyph :
    public CFeatGlyph
{
public:
    /// @name ctors.
    /// @{
    CIntronGlyph(const objects::CMappedFeat& feat, ELinkedFeatDisplay LinkedFeat = ELinkedFeatDisplay::eLFD_Default);
    CIntronGlyph(const objects::CMappedFeat& feat, const objects::CSeq_loc& loc, ELinkedFeatDisplay LinkedFeat = ELinkedFeatDisplay::eLFD_Default);
    /// @}

    /// @name CFeatGlyph virtual method implementation.
    /// @{
    virtual bool IsConsensus() const;
    virtual void GetAccessoryLabel(string& accessory_label) const;
    /// @}

    int GetReadCount() const {
        return m_ReadCount;
    }

private:
    void x_Init();

    /// Number of reads supprtoing this intron
    /// Initialized from feature's user object
    int m_ReadCount = 0;

}; 


END_NCBI_SCOPE


#endif  // GUI_WIDGETS_SEQ_GRAPHIC___INTRON_GLYPH__HPP
