#ifndef GUI_WIDGETS_SEQ_GRAPHIC___INTRON_TRACK__HPP
#define GUI_WIDGETS_SEQ_GRAPHIC___INTRON_TRACK__HPP

/*  
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data, the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties, express or implied, including
 *  warranties of performance, merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 * Authors:  Andrei Shkeda
 *
 * File Description:
 *
 */

#include <corelib/ncbiobj.hpp>
#include <gui/utils/extension.hpp>
#include <gui/objutils/reg_settings.hpp>
#include <gui/widgets/seq_graphic/feature_track.hpp>
#include <gui/widgets/seq_graphic/feature_ds.hpp>
#include <gui/widgets/seq_graphic/feature_enums.hpp>
#include <gui/widgets/seq_graphic/column_layout_policy.hpp>
#include <gui/widgets/seq_graphic/compact_layout_policy.hpp>


BEGIN_NCBI_SCOPE


///////////////////////////////////////////////////////////////////////////////
/// CIntronTrack
///
class NCBI_GUIWIDGETS_SEQGRAPHIC_EXPORT CIntronTrack 
    : public CFeatureTrack
{
    friend class CFeatureTrackFactory;

public:
    CIntronTrack(CSGFeatureDS* ds, CRenderingContext* r_cntx);
    virtual ~CIntronTrack() {}
    virtual void GetHTMLActiveAreas (TAreaVector *p_areas) const;
protected:
    /// @name CFeatureTrack virtual methods overloading.
    /// @{
    virtual void x_ParseSettings(const TKeyValuePairs& settings);
    virtual void x_LoadFeatures(const TSeqRange& range, int feat_limit);
    virtual void x_OnIconClicked(TIconID id);
    virtual void x_OnLayoutPolicyChanged();
    /// @}

    /// @name CGlyphContainer virtual methods overloading
    /// @{
    virtual void x_UpdateBoundingBox();
    /// @}

private:
    /// @name prohibited copy constructor and assignment operator.
    /// @{
    CIntronTrack(const CIntronTrack&);
    CIntronTrack& operator=(const CIntronTrack&);
    /// @}

    void x_OnSettingsIconClicked();

protected:
    bool  m_SortReads = false;
    bool  m_SortStrand = false;
    bool  m_HideIntrons = false;
    bool  m_NoReadFilter = true;
    int   m_ReadsCountMin = -1;
    int   m_ReadsCountMax = -1;
};

END_NCBI_SCOPE

#endif  // GUI_WIDGETS_SEQ_GRAPHIC___INTRON_TRACK__HPP
