/*  $Id: test_doi_lookup.cpp 45799 2020-12-02 19:18:48Z asztalos $
* ===========================================================================
*
*                            PUBLIC DOMAIN NOTICE
*               National Center for Biotechnology Information
*
*  This software/database is a "United States Government Work" under the
*  terms of the United States Copyright Act.  It was written as part of
*  the author's official duties as a United States Government employee and
*  thus cannot be copyrighted.  This software/database is freely available
*  to the public for use. The National Library of Medicine and the U.S.
*  Government have not placed any restriction on its use or reproduction.
*
*  Although all reasonable efforts have been taken to ensure the accuracy
*  and reliability of the software and data, the NLM and the U.S.
*  Government do not and cannot warrant the performance or results that
*  may be obtained by using this software or data. The NLM and the U.S.
*  Government disclaim all warranties, express or implied, including
*  warranties of performance, merchantability or fitness for any particular
*  purpose.
*
*  Please cite the author in any work or product based on this material.
*
* ===========================================================================
*
* Author:  Igor Filippov
*
* File Description:
*   DOI lookup unit test.
*
* ===========================================================================
*/

#include <ncbi_pch.hpp>

#include <corelib/ncbi_system.hpp>
#include <corelib/ncbiapp.hpp>
#include <corelib/ncbifile.hpp>

#include <gui/objutils/doi_lookup.hpp>
#include <cstdio>

// This header must be included before all Boost.Test headers if there are any
#include <corelib/test_boost.hpp>

#include <objtools/unit_test_util/unit_test_util.hpp>

USING_NCBI_SCOPE;
USING_SCOPE(objects);

NCBITEST_AUTO_INIT()
{
}


NCBITEST_INIT_CMDLINE(arg_descrs)
{   
}

NCBITEST_AUTO_FINI()
{
}

namespace {
    CRef<CPubdesc> GetGroundTruth(const string& filename)
    {
        CNcbiIfstream ifstr(filename.c_str());
        CRef <CPubdesc> ground(new CPubdesc);
        ifstr >> MSerial_AsnText >> *ground;
        ifstr.close();
        return ground;
    }

    void TestDoiLookup(const string& filename, const string& doi)
    {
        cout << "Testing " << filename << endl;
        CRef<CPubdesc> ground = GetGroundTruth(filename);
        BOOST_REQUIRE(ground);

        pair<CRef<CPubdesc>, string> pub = CDoiLookup::GetPubFromCrossRef(doi);
        if (!pub.second.empty()) {
            if (NStr::StartsWith(pub.second, "Server api.crossref.org returned error code")) {
                LOG_POST(Info << "Ignore test as publication could not be retrieved");
                return;
            }
            BOOST_CHECK(pub.second.empty()); // let it fail
        }
        else if (!pub.first.IsNull()) {
            BOOST_CHECK(ground->Equals(*pub.first));
        }
    }
}

BOOST_AUTO_TEST_CASE(RunTests)
{
    TestDoiLookup("test_doi_lookup-1.asn", "10.1007/s11557-011-0743-4");
    TestDoiLookup("test_doi_lookup-2.asn", "10.1016/j.protis.2017.07.005");
    TestDoiLookup("test_doi_lookup-3.asn", "10.3391/bir.2018.7.1.11");
    TestDoiLookup("test_doi_lookup-4.asn", "10.1039/C7NP00047B");
    TestDoiLookup("test_doi_lookup-5.asn", "10.1515/ap-2017-0051");
    TestDoiLookup("test_doi_lookup-6.asn", "10.1101/616904");
}
