/*  $Id: clustalw_tool_panel.cpp 43795 2019-08-30 19:22:56Z katargir $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data, the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties, express or implied, including
 *  warranties of performance, merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 * Authors: Roman Katargin
 *
 * File Description:
 *
 */

#include <ncbi_pch.hpp>

#include <gui/widgets/wx/exe_file_validator.hpp>

#include "clustalw_tool_panel.hpp"

#include <wx/sizer.h>
#include <wx/stattext.h>
#include <wx/choice.h>
#include <wx/bmpbuttn.h>
#include <wx/checkbox.h>
#include <wx/valgen.h>
#include <wx/valtext.h>
#include <wx/msgdlg.h>
#include <wx/filedlg.h>
#include <wx/filename.h>
#include <wx/artprov.h>
#include <wx/bitmap.h>
#include <wx/icon.h>

////@begin includes
////@end includes

BEGIN_NCBI_SCOPE
USING_SCOPE(ncbi::objects);

IMPLEMENT_DYNAMIC_CLASS( CClustalwToolPanel, CAlgoToolManagerParamsPanel )

BEGIN_EVENT_TABLE( CClustalwToolPanel, CAlgoToolManagerParamsPanel )

////@begin CClustalwToolPanel event table entries
    EVT_BUTTON( ID_BITMAPBUTTON1, CClustalwToolPanel::OnClustalWPathClick )
////@end CClustalwToolPanel event table entries

END_EVENT_TABLE()

CClustalwToolPanel::CClustalwToolPanel()
{
    Init();
}

CClustalwToolPanel::CClustalwToolPanel( wxWindow* parent, wxWindowID id, const wxPoint& pos, const wxSize& size, long style, bool visible )
{
    Init();
    Create(parent, id, pos, size, style, visible);
}

bool CClustalwToolPanel::Create( wxWindow* parent, wxWindowID id, const wxPoint& pos, const wxSize& size, long style, bool visible )
{
    SetExtraStyle(wxWS_EX_VALIDATE_RECURSIVELY);

    CAlgoToolManagerParamsPanel::Create( parent, id, pos, size, style );

    if (!visible) {
        Hide();
    }

    CreateControls();
    if (GetSizer())
    {
        GetSizer()->SetSizeHints(this);
    }
    Centre();
    return true;
}

CClustalwToolPanel::~CClustalwToolPanel()
{
////@begin CClustalwToolPanel destruction
////@end CClustalwToolPanel destruction
}

void CClustalwToolPanel::Init()
{
////@begin CClustalwToolPanel member initialisation
    m_LocationSel = NULL;
////@end CClustalwToolPanel member initialisation
}

void CClustalwToolPanel::CreateControls()
{    
////@begin CClustalwToolPanel content construction
    // Generated by DialogBlocks, 30/08/2019 15:17:57 (unregistered)

    CClustalwToolPanel* itemCAlgoToolManagerParamsPanel1 = this;

    wxBoxSizer* itemBoxSizer2 = new wxBoxSizer(wxVERTICAL);
    itemCAlgoToolManagerParamsPanel1->SetSizer(itemBoxSizer2);

    m_LocationSel = new CObjectListWidgetSel( itemCAlgoToolManagerParamsPanel1, ID_PANEL, wxDefaultPosition, wxDefaultSize, wxNO_BORDER|wxTAB_TRAVERSAL );
    itemBoxSizer2->Add(m_LocationSel, 1, wxGROW|wxALL, 0);

    wxCheckBox* itemCheckBox4 = new wxCheckBox( itemCAlgoToolManagerParamsPanel1, ID_CHECKBOX12, _("Return guide tree, used for alignment"), wxDefaultPosition, wxDefaultSize, 0 );
    itemCheckBox4->SetValue(false);
    itemBoxSizer2->Add(itemCheckBox4, 0, wxALIGN_LEFT|wxALL, 5);

    wxBoxSizer* itemBoxSizer5 = new wxBoxSizer(wxHORIZONTAL);
    itemBoxSizer2->Add(itemBoxSizer5, 0, wxGROW|wxALL, 0);

    wxStaticText* itemStaticText6 = new wxStaticText( itemCAlgoToolManagerParamsPanel1, wxID_STATIC, _("Command line options"), wxDefaultPosition, wxDefaultSize, 0 );
    itemBoxSizer5->Add(itemStaticText6, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxTextCtrl* itemTextCtrl7 = new wxTextCtrl( itemCAlgoToolManagerParamsPanel1, ID_TEXTCTRL11, wxEmptyString, wxDefaultPosition, wxDefaultSize, 0 );
    itemBoxSizer5->Add(itemTextCtrl7, 1, wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxBoxSizer* itemBoxSizer8 = new wxBoxSizer(wxHORIZONTAL);
    itemBoxSizer2->Add(itemBoxSizer8, 0, wxGROW|wxALL, 0);

    wxStaticText* itemStaticText9 = new wxStaticText( itemCAlgoToolManagerParamsPanel1, wxID_STATIC, _("Path to executable"), wxDefaultPosition, wxDefaultSize, 0 );
    itemBoxSizer8->Add(itemStaticText9, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxTextCtrl* itemTextCtrl10 = new wxTextCtrl( itemCAlgoToolManagerParamsPanel1, ID_TEXTCTRL12, wxEmptyString, wxDefaultPosition, wxDefaultSize, 0 );
    itemBoxSizer8->Add(itemTextCtrl10, 1, wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxBitmapButton* itemBitmapButton11 = new wxBitmapButton( itemCAlgoToolManagerParamsPanel1, ID_BITMAPBUTTON1, itemCAlgoToolManagerParamsPanel1->GetBitmapResource(wxT("menu::open")), wxDefaultPosition, wxDefaultSize, wxBU_AUTODRAW );
    itemBitmapButton11->SetHelpText(_("Select ClustalW executable"));
    if (CClustalwToolPanel::ShowToolTips())
        itemBitmapButton11->SetToolTip(_("Select ClustalW executable"));
    itemBoxSizer8->Add(itemBitmapButton11, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);

    // Set validators
    itemCheckBox4->SetValidator( wxGenericValidator(& GetData().m_GenerateTree) );
    itemTextCtrl7->SetValidator( wxTextValidator(wxFILTER_NONE, & GetData().m_CommandLine) );
    itemTextCtrl10->SetValidator( CExeFileValidator(& GetData().m_ClustalWPath, wxT("ClustalW")) );
////@end CClustalwToolPanel content construction

    m_LocationSel->SetTitle(wxT("Select Locations"));
    m_LocationSel->SetDoSelectAll(true);
    m_LocationSel->ShowNuclProtButtons();
    m_LocationSel->EnableGroups(false);
    m_LocationSel->SetDefaultSelection("Genomic Location (total range)");
}

void CClustalwToolPanel::SetObjects(map<string, TConstScopedObjects>* objects)
{
    m_LocationSel->SetObjects(objects);
}

bool CClustalwToolPanel::ShowToolTips()
{
    return true;
}
wxBitmap CClustalwToolPanel::GetBitmapResource( const wxString& name )
{
    return wxArtProvider::GetBitmap(name);
}
wxIcon CClustalwToolPanel::GetIconResource( const wxString& name )
{
    // Icon retrieval
////@begin CClustalwToolPanel icon retrieval
    wxUnusedVar(name);
    return wxNullIcon;
////@end CClustalwToolPanel icon retrieval
}

bool CClustalwToolPanel::TransferDataFromWindow()
{
    if (!CAlgoToolManagerParamsPanel::TransferDataFromWindow())
        return false;

    GetData().SetObjects() = m_LocationSel->GetSelection();

    return true;
}

static const char* kLocationSel = ".LocationSel";

void CClustalwToolPanel::SetRegistryPath(const string& path)
{
    m_RegPath = path;
    m_LocationSel->SetRegistryPath(m_RegPath + kLocationSel);
}

void CClustalwToolPanel::SaveSettings() const
{
    m_LocationSel->SaveSettings();
}

void CClustalwToolPanel::LoadSettings()
{
    m_LocationSel->LoadSettings();
}

void CClustalwToolPanel::OnClustalWPathClick( wxCommandEvent& WXUNUSED(event) )
{
    wxTextCtrl* textCtrl = dynamic_cast<wxTextCtrl*>(FindWindow(ID_TEXTCTRL12));
    _ASSERT(nullptr != textCtrl);

    wxString path = textCtrl->GetValue();

    wxFileDialog dlg(this, wxT("Select a file"), wxT(""), wxT(""), wxT("*"),
        wxFD_OPEN|wxFD_FILE_MUST_EXIST);

    dlg.SetPath(path);

    if (dlg.ShowModal() != wxID_OK)
        return;

    path = dlg.GetPath();
    textCtrl->SetValue(path);
}

void CClustalwToolPanel::RestoreDefaults()
{
    GetData().Init();
    TransferDataToWindow();
}

END_NCBI_SCOPE
