#ifndef GUI_SERVICES___VCF_SEARCH_TOOL_HPP
#define GUI_SERVICES___VCF_SEARCH_TOOL_HPP

/*  $Id: vcf_search_tool.hpp 45881 2021-01-05 03:10:02Z rudnev $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data, the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties, express or implied, including
 *  warranties of performance, merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 * Authors:  Andrey Yazhuk
 *
 * File Description:
 *
 */

#include <gui/core/seqloc_search_context.hpp>
#include <gui/core/search_form_base.hpp>


class wxTextCtrl;
class wxComboBox;
class wxChoice;

BEGIN_NCBI_SCOPE


/** @addtogroup GUI_PKG_SEQUENCE
 *
 * @{
 */

///////////////////////////////////////////////////////////////////////////////
/// CVcfSearchTool
class CVcfSearchTool : public CSearchToolBase
{
public:
    CVcfSearchTool();

    /// @name IUITool implementation
    /// @{
    virtual IUITool* Clone() const;
    virtual string  GetName() const;
    virtual string  GetDescription() const;
    /// @}

    /// @name IDMSearchTool implementation
    /// @{
    virtual CIRef<IDMSearchForm>  CreateSearchForm();
    virtual bool    IsCompatible(IDataMiningContext* context);
    /// @}

    /// @name IExtension implementation
    /// @{
    virtual string  GetExtensionIdentifier() const;
    virtual string  GetExtensionLabel() const;
    /// @}

protected:
    /// implementing CSearchToolBase pure virtual function
    virtual CRef<CSearchJobBase> x_CreateJob(IDMSearchQuery& query);
};


///////////////////////////////////////////////////////////////////////////////
/// CVcfSearchQuery
class CVcfSearchQuery :
	public CObject,
	public IDMSearchQuery
{
public:
	CVcfSearchQuery(vector<ISeqLocSearchContext*> VcfContexts, const string& ids);
	const vector<ISeqLocSearchContext*>& GetVcfContexts() const { return m_VcfContexts; }

    string      GetIds() const {   return m_Ids;   }

    virtual string ToString() const { return GetIds(); }

protected:
	vector<ISeqLocSearchContext*> m_VcfContexts;
    string      m_Ids;
};


///////////////////////////////////////////////////////////////////////////////
/// CVcfSearchForm
class CComboBox;

class CVcfSearchForm : public CSearchFormBase
{
public:
    CVcfSearchForm(CVcfSearchTool& tool);
    
    /// @name IDMSearchForm implementation
    /// @{
    virtual void    Create();
    virtual void    Init();
    virtual void    Update();
    /// @}

    virtual CIRef<IDMSearchQuery>   ConstructQuery();
    virtual wxSizer *  GetWidget(wxWindow * parent);
    virtual void UpdateContexts();
protected:
    /// @name CSearchFormBase overridables
    /// @{
    //virtual string  x_GetRunningText();
    virtual void    x_CreateWidgets();

//    virtual void    x_UpdateData(EDDXDirection dir);

    virtual IDMSearchTool*  x_GetTool();
    /// @}


protected:
    virtual void    x_LoadSettings(const CRegistryReadView& /*view*/) {}
    virtual void    x_SaveSettings(CRegistryWriteView /*view*/) const {}

    CRef<CVcfSearchTool>  m_Tool;
    wxChoice*                 m_DbCombo;
};


///////////////////////////////////////////////////////////////////////////////
/// CVcfSearchJob
///
class CVcfSearchJob : public CSearchJobBase
{
public:
    CVcfSearchJob(CVcfSearchQuery& query);

protected:

	/// @name CSearchJobBase overridables
    /// @{
    virtual bool    x_ValidateParams();
    virtual EJobState    x_DoSearch();
    virtual void    x_SetupColumns(CObjectList& obj_list);
    /// @}

protected:
    CRef<CVcfSearchQuery>   m_Query;
};


/* @} */

END_NCBI_SCOPE

#endif  // GUI_SERVICES___VCF_SEARCH_TOOL_HPP
