/*  $Id: test_table_reader_macro.cpp 45654 2020-10-13 15:36:09Z asztalos $
* ===========================================================================
*
*                            PUBLIC DOMAIN NOTICE
*               National Center for Biotechnology Information
*
*  This software/database is a "United States Government Work" under the
*  terms of the United States Copyright Act.  It was written as part of
*  the author's official duties as a United States Government employee and
*  thus cannot be copyrighted.  This software/database is freely available
*  to the public for use. The National Library of Medicine and the U.S.
*  Government have not placed any restriction on its use or reproduction.
*
*  Although all reasonable efforts have been taken to ensure the accuracy
*  and reliability of the software and data, the NLM and the U.S.
*  Government do not and cannot warrant the performance or results that
*  may be obtained by using this software or data. The NLM and the U.S.
*  Government disclaim all warranties, express or implied, including
*  warranties of performance, merchantability or fitness for any particular
*  purpose.
*
*  Please cite the author in any work or product based on this material.
*
* ===========================================================================
*
* Author:  Andrea Asztalos
*
* File Description:
*   Tests to generate macros in the background by the table reader
*/

#include <ncbi_pch.hpp>

// This header must be included before all Boost.Test headers if there are any
#include <corelib/test_boost.hpp>
#include <gui/packages/pkg_sequence_edit/table_reader_macro.hpp>


USING_NCBI_SCOPE;
USING_SCOPE(objects);

extern const char* sc_Test_GiraffetblReplaceAll;
extern const char* sc_Test_GiraffetblMatchTaxname;
extern const char* sc_Test_GB1932;
extern const char* sc_Test_GB6650;
extern const char* sc_Test_GB3565;
extern const char* sc_Test_GB3533;
extern const char* sc_Test_GB3533Multi;
extern const char* sc_Test_GB3533Delimiter;
extern const char* sc_Test_GB3533MergeDels;
extern const char* sc_Test_BsrcMolinfoMatch;
extern const char* sc_Test_BsrcInstMatch;
extern const char* sc_Test_BsrcDBLinkMatch;
extern const char* sc_Test_BsrcPubMatch;
extern const char* sc_Test_BsrcFeatQualMatch;
extern const char* sc_Test_BsrcCdsGeneProtMatch;
extern const char* sc_Test_BsrcRnaGeneSynMatch;
extern const char* sc_Test_BsrcDoubleCultColl;
extern const char* sc_Test_BsrcSubmissionID;
extern const char* sc_Test_BsrcMultipleSpsOnly;
extern const char* sc_Test_ApplyGenome;

extern const char* sc_Test_DBLinkMulti;
extern const char* sc_Test_DBLinkBsrcMatch;
extern const char* sc_Test_DBLinkStrCommFieldMatch;
extern const char* sc_Test_DBLinkMolinfoMatch;
extern const char* sc_Test_DBLinkMolinfoInstMatch;


extern const char* sc_TestStrCommFieldBsrcMatch;
extern const char* sc_TestStrCommFieldDbLinkMatch;
extern const char* sc_TestStrCommDBSeqidMatch;
extern const char* sc_TestStrCommFieldnameDblinkMatch;

extern const char* sc_TestMolinfoBsrcMatch;
extern const char* sc_TestMolinfoSeqIDMatch;
extern const char* sc_TestMolinfoDBLinkMatch;
extern const char* sc_TestMolinfoMolinfoMatch;

extern const char* sc_TestMiscBsrcMatch;
extern const char* sc_TestMiscSeqIdMatch;
extern const char* sc_TestMiscDBLinkMatch;
extern const char* sc_TestMiscPubMatch;
extern const char* sc_TestMiscGeneMatch;
extern const char* sc_TestMiscCdsGeneProtMatch;

extern const char* sc_TestPubSeqIDMatch;
extern const char* sc_TestPubBsrcMatch;
extern const char* sc_TestPubDBLinkMatch;
extern const char* sc_TestPubPubMatch;
extern const char* sc_TestPubStrCommMatch;

extern const char* sc_TestFeatGene_GeneMatch;
extern const char* sc_TestFeatGene1_GeneMatch;
extern const char* sc_TestFeatProtein_GeneMatch;
extern const char* sc_TestFeatCDS_ProteinMatch;
extern const char* sc_TestFeatGene_IDMatch;
extern const char* sc_TestFeatProtein_BsrcMatch;
extern const char* sc_TestFeatProtein_DBLinkMatch;
extern const char* sc_TestFeat_GB8268;
extern const char* sc_TestFeatGene_CdsGeneProtMatch;
extern const char* sc_TestGeneDbxref_GeneMatch;

extern const char* sc_TestCGProt_GeneMatch;
extern const char* sc_TestCGProt1_GeneMatch;
extern const char* sc_TestCGPProtein_GeneMatch;
extern const char* sc_TestCGPCDS_ProteinMatch;
extern const char* sc_TestCGPGene_IDMatch;
extern const char* sc_TestCGPProtein_BsrcMatch;
extern const char* sc_TestCGPProtein_DBLinkMatch;
extern const char* sc_TestCGPmRNA_CDSMatch;
extern const char* sc_TestCGPMatPeptide_GeneMatch;
extern const char* sc_TestCGP_GeneDbxrefMatch;

extern const char* sc_TestRNA_SeqIDMatch;
extern const char* sc_TestmRNA_GeneMatch;
extern const char* sc_TestncRNA_SeqIDMatch;
extern const char* sc_TestrRNA_BsrcMatch;
extern const char* sc_TestrRNA_GeneLocusTagMatch;

extern const char* sc_TestBsrc_StructuredComment;
extern const char* sc_TestBsrc_rRNAComment;
extern const char* sc_TestMany_PmidMatch;


NCBITEST_AUTO_INIT()
{
}

NCBITEST_AUTO_FINI()
{
}

CRef<CUser_object> ReadUserObj(const string& name)
{
    CRef<CUser_object> column_prop(new CUser_object);
    CNcbiIstrstream istr(name.c_str());
    istr >> MSerial_AsnText >> column_prop;
    return column_prop;
}

/* Target = BioSource
Match_field:
 SeqID - Test_Giraffe_ReplaceAll, Test_Bsrc_DoubleCultColl
 BioSource - Test_Giraffe_MatchTaxname
 Molinfo - Test_BsrcMolinfoMatch
 Molinfo (inst) - Test_BsrcInstMatch
 FeatQual - Test_Bsrc_GeneMatch
 CDS-Gene-Prot - Test_Bsrc_GeneMatch
 RNA - Test_Bsrc_RNAGeneSynMatch
 Misc
 Pub - Test_Bsrc_PubMatch
 StrComm
 DBlink - Test_BsrcDBLinkMatch
*/

BOOST_AUTO_TEST_CASE(Test_Giraffe_ReplaceAll)
{
    CRef<CUser_object> column_prop = ReadUserObj(sc_Test_GiraffetblReplaceAll);
    CTableReaderMacro tbl_macro;
    CTableReaderMacro::SColumnDelimiters del_options("\t");
    auto macros = tbl_macro.CreateMacros("giraffe.tbl", column_prop, del_options, nullptr, true);
    BOOST_CHECK(macros.size() == 1);

    string check_macro =
        "MACRO TableReaderMacro \"Description\"\n"
        "VAR\n"
        "table = \"giraffe.tbl\"\n"
        "merge_del = false\n"
        "split_first = false\n"
        "convert_multi = false\n"
        "merge_first = false\n"
        "multiple_spaces_only = false\n"
        "FOR EACH BioSource\n"
        "WHERE INTABLE(SEQID(), table, 1, \"\\t\", merge_del, split_first, convert_multi, merge_first, false, multiple_spaces_only)\n"
        "DO\n"
        "value = ValueFromTable(table, 2);\n"
        "SetStringQual(\"org.taxname\", value, \"eReplace\", false);\n"
        "value = ValueFromTable(table, 3);\n"
        "obj = Resolve(\"org.orgname.mod\") WHERE obj.subtype = \"acronym\";\n"
        "SetModifier(obj, \"org.orgname.mod\", \"acronym\", value, \"eReplace\", false);\n"
        "value = ValueFromTable(table, 4);\n"
        "obj = Resolve(\"org.orgname.mod\") WHERE obj.subtype = \"isolate\";\n"
        "SetModifier(obj, \"org.orgname.mod\", \"isolate\", value, \"eReplace\", false);\n"
        "value = ValueFromTable(table, 5);\n"
        "obj = Resolve(\"org.orgname.mod\") WHERE obj.subtype = \"serotype\";\n"
        "SetModifier(obj, \"org.orgname.mod\", \"serotype\", value, \"eReplace\", false);\n"
        "DONE\n"
        "---------------------------------------------------\n";
    BOOST_CHECK(macros.front() == check_macro);
}

const char* sc_Test_GiraffetblReplaceAll = " \
User-object ::= { \
  type str \"Column Meta Info\",\
  data { \
    { \
      label str \"Column.1\", \
      data str \"&skipped=false&match=true&name=match Seq-ID&qualifier=SeqId&qual_type=SeqId&blanks=ignore\" \
    }, \
    { \
      label str \"Column.2\", \
      data str \"&skipped=false&match=false&name=taxname&qualifier=taxname&qual_type=Taxname&blanks=ignore&text_separator=;&update_rule=replace\" \
    }, \
    { \
      label str \"Column.3\", \
      data str \"&skipped=false&match=false&name=acronym&qualifier=acronym&qual_type=Source Qualifier&blanks=ignore&text_separator=;&update_rule=replace\" \
    }, \
    { \
      label str \"Column.4\", \
      data str \"&skipped=false&match=false&name=isolate&qualifier=isolate&qual_type=Source Qualifier&blanks=ignore&text_separator=;&update_rule=replace\" \
    }, \
    { \
      label str \"Column.5\", \
      data str \"&skipped=false&match=false&name=serotype&qualifier=serotype&qual_type=Source Qualifier&blanks=ignore&text_separator=;&update_rule=replace\" \
    } \
  } \
}";

BOOST_AUTO_TEST_CASE(Test_Giraffe_MatchTaxname)
{
    CRef<CUser_object> column_prop = ReadUserObj(sc_Test_GiraffetblMatchTaxname);
    CTableReaderMacro tbl_macro;
    CTableReaderMacro::SColumnDelimiters del_options("\t");
    auto macros = tbl_macro.CreateMacros("giraffe.tbl", column_prop, del_options, nullptr, true);
    BOOST_CHECK(macros.size() == 1);

    string check_macro =
        "MACRO TableReaderMacro \"Description\"\n"
        "VAR\n"
        "table = \"giraffe.tbl\"\n"
        "merge_del = false\n"
        "split_first = false\n"
        "convert_multi = false\n"
        "merge_first = false\n"
        "multiple_spaces_only = false\n"
        "FOR EACH BioSource\n"
        "WHERE INTABLE(\"org.taxname\", table, 2, \"\\t\", merge_del, split_first, convert_multi, merge_first, true, multiple_spaces_only)\n"
        "DO\n"
        "value = ValueFromTable(table, 3);\n"
        "obj = Resolve(\"org.orgname.mod\") WHERE obj.subtype = \"acronym\";\n"
        "SetModifier(obj, \"org.orgname.mod\", \"acronym\", value, \"eAppend\", \":\", false);\n"
        "value = ValueFromTable(table, 4);\n"
        "obj = Resolve(\"org.orgname.mod\") WHERE obj.subtype = \"isolate\";\n"
        "SetModifier(obj, \"org.orgname.mod\", \"isolate\", value, \"ePrepend\", \",\", true);\n"
        "value = ValueFromTable(table, 5);\n"
        "obj = Resolve(\"org.orgname.mod\") WHERE obj.subtype = \"serotype\";\n"
        "SetModifier(obj, \"org.orgname.mod\", \"serotype\", value, \"eReplace\", false);\n"
        "DONE\n"
        "---------------------------------------------------\n";
    BOOST_CHECK(macros.front() == check_macro);
}

const char* sc_Test_GiraffetblMatchTaxname = " \
User-object ::= { \
  type str \"Column Meta Info\",\
  data { \
    { \
      label str \"Column.1\", \
      data str \"&skipped=true&match=false&name=Accession&qualifier=SeqId&qual_type = SeqId&blanks=ignore&text_separator=;&update_rule=replace\" \
    }, \
    { \
      label str \"Column.2\", \
      data str \"&skipped=false&match=true&name=match taxname&qualifier=taxname&qual_type=Taxname&blanks=ignore\" \
    }, \
    { \
      label str \"Column.3\", \
      data str \"&skipped=false&match=false&name=acronym&qualifier=acronym&qual_type=Source Qualifier&blanks=ignore&text_separator=:&update_rule=append\" \
    }, \
    { \
      label str \"Column.4\", \
      data str \"&skipped=false&match=false&name=isolate&qualifier=isolate&qual_type=Source Qualifier&blanks=erase&text_separator=,&update_rule=prefix\" \
    }, \
    { \
      label str \"Column.5\", \
      data str \"&skipped=false&match=false&name=serotype&qualifier=serotype&qual_type=Source Qualifier&blanks=ignore&text_separator=;&update_rule=replace\" \
    } \
  } \
}";

BOOST_AUTO_TEST_CASE(Test_GB_1932)
{
    CRef<CUser_object> column_prop = ReadUserObj(sc_Test_GB1932);
    CTableReaderMacro tbl_macro;
    CTableReaderMacro::SColumnDelimiters del_options("\t");
    auto macros = tbl_macro.CreateMacros("CloneIsosrcTable.txt", column_prop, del_options, nullptr, true);
    BOOST_CHECK(macros.size() == 1);

    string check_macro = 
        "MACRO TableReaderMacro \"Description\"\n"
        "VAR\n"
        "table = \"CloneIsosrcTable.txt\"\n"
        "merge_del = false\n"
        "split_first = false\n"
        "convert_multi = false\n"
        "merge_first = false\n"
        "multiple_spaces_only = false\n"
        "FOR EACH BioSource\n"
        "WHERE INTABLE(\"subtype\", \"clone\", table, 1, \"\\t\", merge_del, split_first, convert_multi, merge_first, true, multiple_spaces_only)\n"
        "DO\n"
        "value = ValueFromTable(table, 2);\n"
        "obj = Resolve(\"subtype\") WHERE obj.subtype = \"isolation-source\";\n"
        "SetModifier(obj, \"subtype\", \"isolation-source\", value, \"eReplace\", false);\n"
        "DONE\n"
        "---------------------------------------------------\n";
        BOOST_CHECK(macros.front() == check_macro);
        
}

const char* sc_Test_GB1932 = " \
User-object ::= { \
  type str \"Column Meta Info\",\
  data { \
    { \
      label str \"Column.1\", \
      data str \"&skipped=false&match=true&name=match clone&qualifier=clone&qual_type=Source Qualifier&blanks=ignore\" \
    }, \
    { \
      label str \"Column.2\", \
      data str \"&skipped=false&match=false&name=isolation-source&qualifier=isolation-source&qual_type=Source Qualifier&blanks=ignore&text_separator=;&update_rule=replace\" \
    } \
  } \
}";

BOOST_AUTO_TEST_CASE(Test_GB_6650)
{
    CRef<CUser_object> column_prop = ReadUserObj(sc_Test_GB6650);
    CTableReaderMacro tbl_macro;
    CTableReaderMacro::SColumnDelimiters del_options("\t");
    auto macros = tbl_macro.CreateMacros("nullpointertable.txt", column_prop, del_options, nullptr, true);
    BOOST_CHECK(macros.size() == 1);

    string check_macro =
        "MACRO TableReaderMacro \"Description\"\n"
        "VAR\n"
        "table = \"nullpointertable.txt\"\n"
        "merge_del = false\n"
        "split_first = false\n"
        "convert_multi = false\n"
        "merge_first = false\n"
        "multiple_spaces_only = false\n"
        "FOR EACH BioSource\n"
        "WHERE INTABLE(SEQID(), table, 1, \"\\t\", merge_del, split_first, convert_multi, merge_first, false, multiple_spaces_only)\n"
        "DO\n"
        "value = ValueFromTable(table, 2);\n"
        "ApplyStructVoucherPart(\"specimen-voucher\", \"specid\", value, \"eReplace\", false);\n"
        "value = ValueFromTable(table, 3);\n"
        "ApplyStructVoucherPart(\"specimen-voucher\", \"inst\", value, \"eReplace\", false);\n"
        "DONE\n"
        "---------------------------------------------------\n";
    BOOST_CHECK(macros.front() == check_macro);

}

const char* sc_Test_GB6650 = " \
User-object ::= { \
  type str \"Column Meta Info\",\
  data { \
    { \
      label str \"Column.1\", \
      data str \"&skipped=false&match=true&name=match Seq-ID&qualifier=SeqId&qual_type=SeqId&blanks=ignore\" \
    }, \
    { \
      label str \"Column.2\", \
      data str \"&skipped=false&match=false&name=specimen-voucher-specid&qualifier=specimen-voucher-specid&qual_type=Source Qualifier&blanks=ignore&text_separator=;&update_rule=replace\" \
    }, \
    { \
      label str \"Column.3\", \
      data str \"&skipped=false&match=false&name=specimen-voucher-inst&qualifier=specimen-voucher-inst&qual_type=Source Qualifier&blanks=ignore&text_separator=;&update_rule=replace\" \
    } \
  } \
}";


BOOST_AUTO_TEST_CASE(Test_GB_3565_MergeFirstCol)
{
    CRef<CUser_object> column_prop = ReadUserObj(sc_Test_GB3565);
    CTableReaderMacro tbl_macro;
    CTableReaderMacro::SColumnDelimiters del_options("\t");
    del_options.m_Merge_first_two_cols = true;
    auto macros = tbl_macro.CreateMacros("table.txt", column_prop, del_options, nullptr, true);
    BOOST_CHECK(macros.size() == 1);

    string check_macro =
        "MACRO TableReaderMacro \"Description\"\n"
        "VAR\n"
        "table = \"table.txt\"\n"
        "merge_del = false\n"
        "split_first = false\n"
        "convert_multi = false\n"
        "merge_first = true\n"
        "multiple_spaces_only = false\n"
        "FOR EACH BioSource\n"
        "WHERE INTABLE(SEQID(), table, 1, \"\\t\", merge_del, split_first, convert_multi, merge_first, false, multiple_spaces_only)\n"
        "DO\n"
        "value = ValueFromTable(table, 2);\n"
        "SetStringQual(\"org.taxname\", value, \"eReplace\", false);\n"
        "value = ValueFromTable(table, 3);\n"
        "obj = Resolve(\"org.orgname.mod\") WHERE obj.subtype = \"strain\";\n"
        "SetModifier(obj, \"org.orgname.mod\", \"strain\", value, \"eReplace\", false);\n"
        "DONE\n"
        "---------------------------------------------------\n";
    BOOST_CHECK(macros.front() == check_macro);

}

const char* sc_Test_GB3565 = " \
User-object ::= { \
  type str \"Column Meta Info\",\
  data { \
    { \
      label str \"Column.1\", \
      data str \"&skipped=false&match=true&name=match Seq-ID&qualifier=SeqId&qual_type=SeqId&blanks=ignore\" \
    }, \
    { \
      label str \"Column.2\", \
      data str \"&skipped=false&match=false&name=taxname&qualifier=taxname&qual_type=Source Qualifier&blanks=ignore&text_separator=;&update_rule=replace\" \
    }, \
    { \
      label str \"Column.3\", \
      data str \"&skipped=false&match=false&name=strain&qualifier=strain&qual_type=Source Qualifier&blanks=ignore&text_separator=;&update_rule=replace\" \
    } \
  } \
}";

BOOST_AUTO_TEST_CASE(Test_GB_3533_SplitFirstCol)
{
    CRef<CUser_object> column_prop = ReadUserObj(sc_Test_GB3533);
    CTableReaderMacro tbl_macro;
    CTableReaderMacro::SColumnDelimiters del_options("\t");
    del_options.m_Split_first_col = true;
    auto macros = tbl_macro.CreateMacros("splitcoltable.txt", column_prop, del_options, nullptr, true);
    BOOST_CHECK(macros.size() == 1);

    string check_macro =
        "MACRO TableReaderMacro \"Description\"\n"
        "VAR\n"
        "table = \"splitcoltable.txt\"\n"
        "merge_del = false\n"
        "split_first = true\n"
        "convert_multi = false\n"
        "merge_first = false\n"
        "multiple_spaces_only = false\n"
        "FOR EACH BioSource\n"
        "WHERE INTABLE(SEQID(), table, 1, \"\\t\", merge_del, split_first, convert_multi, merge_first, false, multiple_spaces_only)\n"
        "DO\n"
        "value = ValueFromTable(table, 2);\n"
        "SetStringQual(\"org.taxname\", value, \"eReplace\", false);\n"
        "value = ValueFromTable(table, 3);\n"
        "obj = Resolve(\"org.orgname.mod\") WHERE obj.subtype = \"isolate\";\n"
        "SetModifier(obj, \"org.orgname.mod\", \"isolate\", value, \"eReplace\", false);\n"
        "value = ValueFromTable(table, 4);\n"
        "obj = Resolve(\"org.orgname.mod\") WHERE obj.subtype = \"other\";\n"
        "SetModifier(obj, \"org.orgname.mod\", \"other\", value, \"eReplace\", false);\n"
        "DONE\n"
        "---------------------------------------------------\n";
    BOOST_CHECK(macros.front() == check_macro);
}

const char* sc_Test_GB3533 = " \
User-object ::= { \
  type str \"Column Meta Info\",\
  data { \
    { \
      label str \"Column.1\", \
      data str \"&skipped=false&match=true&name=match Seq-ID&qualifier=SeqId&qual_type=SeqId&blanks=ignore\" \
    }, \
    { \
      label str \"Column.2\", \
      data str \"&skipped=false&match=false&name=taxname&qualifier=taxname&qual_type=Source Qualifier&blanks=ignore&text_separator=;&update_rule=replace\" \
    }, \
    { \
      label str \"Column.3\", \
      data str \"&skipped=false&match=false&name=isolate&qualifier=isolate&qual_type=Source Qualifier&blanks=ignore&text_separator=;&update_rule=replace\" \
    }, \
    { \
      label str \"Column.4\", \
      data str \"&skipped=false&match=false&name=note-orgmod&qualifier=note-orgmod&qual_type=Source Qualifier&blanks=ignore&text_separator=;&update_rule=replace\" \
    } \
  } \
}";

BOOST_AUTO_TEST_CASE(Test_GB_3533_MultiSpaceToTab)
{
    CRef<CUser_object> column_prop = ReadUserObj(sc_Test_GB3533Multi);
    CTableReaderMacro tbl_macro;
    CTableReaderMacro::SColumnDelimiters del_options("\t");
    del_options.m_Multispace_to_tab = true;
    auto macros = tbl_macro.CreateMacros("convertmultitbl.txt", column_prop, del_options, nullptr, true);
    BOOST_CHECK(macros.size() == 1);

    string check_macro =
        "MACRO TableReaderMacro \"Description\"\n"
        "VAR\n"
        "table = \"convertmultitbl.txt\"\n"
        "merge_del = false\n"
        "split_first = false\n"
        "convert_multi = true\n"
        "merge_first = false\n"
        "multiple_spaces_only = false\n"
        "FOR EACH BioSource\n"
        "WHERE INTABLE(SEQID(), table, 3, \"\\t\", merge_del, split_first, convert_multi, merge_first, false, multiple_spaces_only)\n"
        "DO\n"
        "value = ValueFromTable(table, 1);\n"
        "SetStringQual(\"org.taxname\", value, \"eReplace\", false);\n"
        "value = ValueFromTable(table, 2);\n"
        "obj = Resolve(\"subtype\") WHERE obj.subtype = \"isolation-source\";\n"
        "SetModifier(obj, \"subtype\", \"isolation-source\", value, \"eReplace\", false);\n"
        "DONE\n"
        "---------------------------------------------------\n";
    BOOST_CHECK(macros.front() == check_macro);

}

const char* sc_Test_GB3533Multi = " \
User-object ::= { \
  type str \"Column Meta Info\",\
  data { \
    { \
      label str \"Column.1\", \
      data str \"&skipped=false&match=false&name=taxname&qualifier=taxname&qual_type=Taxname&blanks=ignore&text_separator=;&update_rule=replace\" \
    }, \
    { \
      label str \"Column.2\", \
      data str \"&skipped=false&match=false&name=isolation-source&qualifier=isolation-source&qual_type=Source Qualifier&blanks=ignore&text_separator=;&update_rule=replace\" \
    }, \
    { \
      label str \"Column.3\", \
      data str \"&skipped=false&match=true&name=match Seq-ID&qualifier=SeqId&qual_type=SeqId&blanks=ignore\" \
    } \
  } \
}";

BOOST_AUTO_TEST_CASE(Test_GB_3533_Delimiters)
{
    CRef<CUser_object> column_prop = ReadUserObj(sc_Test_GB3533Delimiter);
    CTableReaderMacro tbl_macro;
    CTableReaderMacro::SColumnDelimiters del_options(",");
    del_options.m_Split_first_col = true;
    auto macros = tbl_macro.CreateMacros("first_table_modCS.txt", column_prop, del_options, nullptr, true);
    BOOST_CHECK(macros.size() == 1);

    string check_macro =
        "MACRO TableReaderMacro \"Description\"\n"
        "VAR\n"
        "table = \"first_table_modCS.txt\"\n"
        "merge_del = false\n"
        "split_first = true\n"
        "convert_multi = false\n"
        "merge_first = false\n"
        "multiple_spaces_only = false\n"
        "FOR EACH BioSource\n"
        "WHERE INTABLE(SEQID(), table, 1, \",\", merge_del, split_first, convert_multi, merge_first, false, multiple_spaces_only)\n"
        "DO\n"
        "value = ValueFromTable(table, 2);\n"
        "SetStringQual(\"org.taxname\", value, \"eReplace\", false);\n"
        "value = ValueFromTable(table, 3);\n"
        "obj = Resolve(\"subtype\") WHERE obj.subtype = \"isolation-source\";\n"
        "SetModifier(obj, \"subtype\", \"isolation-source\", value, \"eReplace\", false);\n"
        "DONE\n"
        "---------------------------------------------------\n";
    BOOST_CHECK(macros.front() == check_macro);
}

const char* sc_Test_GB3533Delimiter = " \
User-object ::= { \
  type str \"Column Meta Info\",\
  data { \
    { \
      label str \"Column.1\", \
      data str \"&skipped=false&match=true&name=match Seq-ID&qualifier=SeqId&qual_type=SeqId&blanks=ignore\" \
    }, \
    { \
      label str \"Column.2\", \
      data str \"&skipped=false&match=false&name=taxname&qualifier=taxname&qual_type=Source Qualifier&blanks=ignore&text_separator=;&update_rule=replace\" \
    }, \
    { \
      label str \"Column.3\", \
      data str \"&skipped=false&match=false&name=isolation-source&qualifier=isolation-source&qual_type=Source Qualifier&blanks=ignore&text_separator=;&update_rule=replace\" \
    }, \
    { \
      label str \"Column.4\", \
      data str \"&skipped=true&match=false&name=Col 3&qualifier=&qual_type=\" \
    } \
  } \
}";

BOOST_AUTO_TEST_CASE(Test_GB_3533_MergeDelimiters)
{
    CRef<CUser_object> column_prop = ReadUserObj(sc_Test_GB3533MergeDels);
    CTableReaderMacro tbl_macro;
    CTableReaderMacro::SColumnDelimiters del_options(" ");
    del_options.m_Merge_delimiters = true;
    auto macros = tbl_macro.CreateMacros("first_table_mergedel.txt", column_prop, del_options, nullptr, true);
    BOOST_CHECK(macros.size() == 1);

    string check_macro =
        "MACRO TableReaderMacro \"Description\"\n"
        "VAR\n"
        "table = \"first_table_mergedel.txt\"\n"
        "merge_del = true\n"
        "split_first = false\n"
        "convert_multi = false\n"
        "merge_first = false\n"
        "multiple_spaces_only = false\n"
        "FOR EACH BioSource\n"
        "WHERE INTABLE(SEQID(), table, 1, \" \", merge_del, split_first, convert_multi, merge_first, false, multiple_spaces_only)\n"
        "DO\n"
        "value = ValueFromTable(table, 2);\n"
        "SetStringQual(\"org.taxname\", value, \"eReplace\", false);\n"
        "value = ValueFromTable(table, 3);\n"
        "obj = Resolve(\"subtype\") WHERE obj.subtype = \"isolation-source\";\n"
        "SetModifier(obj, \"subtype\", \"isolation-source\", value, \"eReplace\", false);\n"
        "DONE\n"
        "---------------------------------------------------\n";
    BOOST_CHECK(macros.front() == check_macro);
}

const char* sc_Test_GB3533MergeDels = " \
User-object ::= { \
  type str \"Column Meta Info\",\
  data { \
    { \
      label str \"Column.1\", \
      data str \"&skipped=false&match=true&name=match Seq-ID&qualifier=SeqId&qual_type=SeqId&blanks=ignore\" \
    }, \
    { \
      label str \"Column.2\", \
      data str \"&skipped=false&match=false&name=taxname&qualifier=taxname&qual_type=Source Qualifier&blanks=ignore&text_separator=;&update_rule=replace\" \
    }, \
    { \
      label str \"Column.3\", \
      data str \"&skipped=false&match=false&name=isolation-source&qualifier=isolation-source&qual_type=Source Qualifier&blanks=ignore&text_separator=;&update_rule=replace\" \
    }, \
    { \
      label str \"Column.4\", \
      data str \"&skipped=true&match=false&name=Col 3&qualifier=&qual_type=\" \
    } \
  } \
}";

BOOST_AUTO_TEST_CASE(Test_Bsrc_PubMatch)
{
    CRef<CUser_object> column_prop = ReadUserObj(sc_Test_BsrcPubMatch);
    CTableReaderMacro tbl_macro;
    CTableReaderMacro::SColumnDelimiters del_options("\t");
    auto macros = tbl_macro.CreateMacros("pmid_names.txt", column_prop, del_options, nullptr, true);
    BOOST_CHECK(macros.size() == 1);

    string check_macro =
        "MACRO TableReaderMacro \"Description\"\n"
        "VAR\n"
        "table = \"pmid_names.txt\"\n"
        "merge_del = false\n"
        "split_first = false\n"
        "convert_multi = false\n"
        "merge_first = false\n"
        "multiple_spaces_only = false\n"
        "FOR EACH BioSource\n"
        "WHERE INTABLE(PUB_PMID(), table, 1, \"\\t\", merge_del, split_first, convert_multi, merge_first, true, multiple_spaces_only)\n"
        "DO\n"
        "value = ValueFromTable(table, 2);\n"
        "SetStringQual(\"org.taxname\", value, \"eReplace\", false);\n"
        "DONE\n"
        "---------------------------------------------------\n";
    BOOST_CHECK(macros.front() == check_macro);
}

const char* sc_Test_BsrcPubMatch = " \
User-object ::= { \
  type str \"Column Meta Info\",\
  data { \
    { \
      label str \"Column.1\", \
      data str \"&skipped=false&match=true&name=match pmid&qualifier=pmid&qual_type=Pub Qualifier&blanks=ignore\" \
    }, \
    { \
      label str \"Column.2\", \
      data str \"&skipped=false&match=false&name=taxname&qualifier=taxname&qual_type=Taxname&blanks=ignore&text_separator=;&update_rule=replace\" \
    } \
  } \
}";

BOOST_AUTO_TEST_CASE(Test_Bsrc_GeneMatch)
{
    CRef<CUser_object> column_prop = ReadUserObj(sc_Test_BsrcFeatQualMatch);
    CTableReaderMacro tbl_macro;
    CTableReaderMacro::SColumnDelimiters del_options("\t");
    auto macros = tbl_macro.CreateMacros("applygenetable.txt", column_prop, del_options, nullptr, true);
    BOOST_CHECK(macros.size() == 1);

    string check_macro =
        "MACRO TableReaderMacro \"Description\"\n"
        "VAR\n"
        "table = \"applygenetable.txt\"\n"
        "merge_del = false\n"
        "split_first = false\n"
        "convert_multi = false\n"
        "merge_first = false\n"
        "multiple_spaces_only = false\n"
        "FOR EACH BioSource\n"
        "WHERE INTABLE(FEATURES(\"Gene\",\"data.gene.syn\"), table, 2, \"\\t\", merge_del, split_first, convert_multi, merge_first, true, multiple_spaces_only)\n"
        "DO\n"
        "value = ValueFromTable(table, 3);\n"
        "obj = Resolve(\"org.orgname.mod\") WHERE obj.subtype = \"strain\";\n"
        "SetModifier(obj, \"org.orgname.mod\", \"strain\", value, \"eAppend\", \";\", false);\n"
        "DONE\n"
        "---------------------------------------------------\n";
    BOOST_CHECK(macros.front() == check_macro);

    column_prop = ReadUserObj(sc_Test_BsrcCdsGeneProtMatch);
    macros = tbl_macro.CreateMacros("applygenetable.txt", column_prop, del_options, nullptr, true);
    BOOST_CHECK(macros.size() == 1);
    BOOST_CHECK(NStr::EqualNocase(macros.front(), check_macro) == true);
}

const char* sc_Test_BsrcFeatQualMatch = " \
User-object ::= { \
  type str \"Column Meta Info\",\
  data { \
    { \
      label str \"Column.1\", \
      data str \"&skipped=true&match=false&name=Col 1&qualifier=SeqId&qual_type=SeqId&blanks=ignore&text_separator=;&update_rule=replac\" \
    }, \
    { \
      label str \"Column.2\", \
      data str \"&skipped=false&match=true&name=match Gene gene_synonym&qualifier=Gene gene_synonym&qual_type=Feature Qualifier&blanks=ignore\" \
    }, \
    { \
      label str \"Column.3\", \
      data str \"&skipped=false&match=false&name=strain&qualifier=strain&qual_type=Taxname&blanks=ignore&text_separator=;&update_rule=append\" \
    } \
  } \
}";

const char* sc_Test_BsrcCdsGeneProtMatch = " \
User-object ::= { \
  type str \"Column Meta Info\",\
  data { \
    { \
      label str \"Column.1\", \
      data str \"&skipped=true&match=false&name=Col 1&qualifier=SeqId&qual_type=SeqId&blanks=ignore&text_separator=;&update_rule=replac\" \
    }, \
    { \
      label str \"Column.2\", \
      data str \"&skipped=false&match=true&name=match gene synonym&qualifier=gene synonym&qual_type=CDS-Gene-Prot Qualifier&blanks=ignore\" \
    }, \
    { \
      label str \"Column.3\", \
      data str \"&skipped=false&match=false&name=strain&qualifier=strain&qual_type=Taxname&blanks=ignore&text_separator=;&update_rule=append\" \
    } \
  } \
}";


BOOST_AUTO_TEST_CASE(Test_Bsrc_RNAGeneSynMatch)
{
    CRef<CUser_object> column_prop = ReadUserObj(sc_Test_BsrcRnaGeneSynMatch);
    CTableReaderMacro tbl_macro;
    CTableReaderMacro::SColumnDelimiters del_options("\t");
    auto macros = tbl_macro.CreateMacros("featqual_table.txt", column_prop, del_options, nullptr, true);
    BOOST_CHECK(macros.size() == 1);

    string check_macro =
        "MACRO TableReaderMacro \"Description\"\n"
        "VAR\n"
        "table = \"featqual_table.txt\"\n"
        "merge_del = false\n"
        "split_first = false\n"
        "convert_multi = false\n"
        "merge_first = false\n"
        "multiple_spaces_only = false\n"
        "FOR EACH BioSource\n"
        "WHERE INTABLE(FEATURES(\"gene\",\"data.gene.syn\"), table, 2, \"\\t\", merge_del, split_first, convert_multi, merge_first, true, multiple_spaces_only)\n"
        "DO\n"
        "value = ValueFromTable(table, 3);\n"
        "obj = Resolve(\"org.orgname.mod\") WHERE obj.subtype = \"strain\";\n"
        "SetModifier(obj, \"org.orgname.mod\", \"strain\", value, \"eAppend\", \":\", false);\n"
        "DONE\n"
        "---------------------------------------------------\n";
    BOOST_CHECK(macros.front() == check_macro);
}

const char* sc_Test_BsrcRnaGeneSynMatch = " \
User-object ::= { \
  type str \"Column Meta Info\",\
  data { \
    { \
      label str \"Column.1\", \
      data str \"&skipped=true&match=false&name=accession&qualifier=SeqId&qual_type=SeqId&blanks=ignore&text_separator=;&update_rule=replace\" \
    }, \
    { \
      label str \"Column.2\", \
      data str \"&skipped=false&match=true&name=match mRNA gene synonym&qualifier=mRNA gene synonym&qual_type=RNA Qualifier&blanks=ignore\" \
    }, \
    { \
      label str \"Column.3\", \
      data str \"&skipped=false&match=false&name=strain&qualifier=strain&qual_type=Source Qualifier&blanks=ignore&text_separator=:&update_rule=append\" \
    } \
  } \
}";

BOOST_AUTO_TEST_CASE(Test_Bsrc_DoubleCultColl)
{
    CRef<CUser_object> column_prop = ReadUserObj(sc_Test_BsrcDoubleCultColl);
    CTableReaderMacro tbl_macro;
    CTableReaderMacro::SColumnDelimiters del_options("\t");
    auto macros = tbl_macro.CreateMacros("\double_cultcoll.txt", column_prop, del_options, nullptr, true);
    BOOST_CHECK(macros.size() == 1);

    string check_macro =
        "MACRO TableReaderMacro \"Description\"\n"
        "VAR\n"
        "table = \"\double_cultcoll.txt\"\n"
        "merge_del = false\n"
        "split_first = false\n"
        "convert_multi = false\n"
        "merge_first = false\n"
        "multiple_spaces_only = false\n"
        "FOR EACH BioSource\n"
        "WHERE INTABLE(SEQID(), table, 1, \"\\t\", merge_del, split_first, convert_multi, merge_first, false, multiple_spaces_only)\n"
        "DO\n"
        "value = ValueFromTable(table, 2);\n"
        "SetStringQual(\"org.taxname\", value, \"eReplace\", false);\n"
        "value = ValueFromTable(table, 3);\n"
        "obj = Resolve(\"org.orgname.mod\") WHERE obj.subtype = \"culture-collection\";\n"
        "SetModifier(obj, \"org.orgname.mod\", \"culture-collection\", value, \"eReplace\", false);\n"
        "value = ValueFromTable(table, 4);\n"
        "obj = Resolve(\"org.orgname.mod\") WHERE obj.subtype = \"culture-collection\";\n"
        "SetModifier(obj, \"org.orgname.mod\", \"culture-collection\", value, \"eAppend\", \";\", false);\n"
        "DONE\n"
        "---------------------------------------------------\n";
    BOOST_CHECK(macros.front() == check_macro);
}

const char* sc_Test_BsrcDoubleCultColl = " \
User-object ::= { \
  type str \"Column Meta Info\",\
  data { \
    { \
      label str \"Column.1\", \
      data str \"&skipped=false&match=true&name=match Seq-ID&qualifier=SeqId&qual_type=SeqId&blanks=ignore\" \
    }, \
    { \
      label str \"Column.2\", \
      data str \"&skipped=false&match=false&name=taxname&qualifier=taxname&qual_type=Taxname&blanks=ignore&text_separator=;&update_rule=replace\" \
    }, \
    { \
      label str \"Column.3\", \
      data str \"&skipped=false&match=false&name=culture-collection&qualifier=culture-collection&qual_type=Source Qualifier&blanks=ignore&text_separator=;&update_rule=replace\" \
    }, \
    { \
      label str \"Column.4\", \
      data str \"&skipped=false&match=false&name=culture-collection&qualifier=culture-collection&qual_type=Source Qualifier&blanks=ignore&text_separator=;&update_rule=append\" \
    } \
  } \
}";

BOOST_AUTO_TEST_CASE(Test_Bsrc_MatchSUBID)
{
    CRef<CUser_object> column_prop = ReadUserObj(sc_Test_BsrcSubmissionID);
    CTableReaderMacro tbl_macro;
    CTableReaderMacro::SColumnDelimiters del_options("\t");
    del_options.m_Merge_first_two_cols = true;
    auto macros = tbl_macro.CreateMacros("\subid_table.txt", column_prop, del_options, nullptr, true);
    BOOST_CHECK(macros.size() == 1);

    string check_macro =
        "MACRO TableReaderMacro \"Description\"\n"
        "VAR\n"
        "table = \"\subid_table.txt\"\n"
        "merge_del = false\n"
        "split_first = false\n"
        "convert_multi = false\n"
        "merge_first = true\n"
        "multiple_spaces_only = false\n"
        "FOR EACH BioSource\n"
        "WHERE INTABLE(SEQID(), table, 1, \"\\t\", merge_del, split_first, convert_multi, merge_first, false, multiple_spaces_only)\n"
        "DO\n"
        "value = ValueFromTable(table, 2);\n"
        "SetStringQual(\"org.taxname\", value, \"eReplace\", false);\n"
        "DONE\n"
        "---------------------------------------------------\n";
    BOOST_CHECK(macros.front() == check_macro);
}

const char* sc_Test_BsrcSubmissionID = " \
User-object ::= { \
  type str \"Column Meta Info\",\
  data { \
    { \
      label str \"Column.1\", \
      data str \"&skipped=false&match=true&name=match Seq-ID&qualifier=SeqId&qual_type=SeqId&blanks=ignore\" \
    }, \
    { \
      label str \"Column.2\", \
      data str \"&skipped=false&match=false&name=taxname&qualifier=taxname&qual_type=Taxname&blanks=ignore&text_separator=;&update_rule=replace\" \
    } \
  } \
}";

BOOST_AUTO_TEST_CASE(Test_Bsrc_MultipleSpacesOnly)
{
    CRef<CUser_object> column_prop = ReadUserObj(sc_Test_BsrcMultipleSpsOnly);
    CTableReaderMacro tbl_macro;
    CTableReaderMacro::SColumnDelimiters del_options(" ");
    del_options.m_Merge_delimiters = true;
    del_options.m_Multiple_spaces_only = true;
    auto macros = tbl_macro.CreateMacros("\multiplespaces_tbl.txt", column_prop, del_options, nullptr, true);
    BOOST_CHECK(macros.size() == 1);

    string check_macro =
        "MACRO TableReaderMacro \"Description\"\n"
        "VAR\n"
        "table = \"\multiplespaces_tbl.txt\"\n"
        "merge_del = true\n"
        "split_first = false\n"
        "convert_multi = false\n"
        "merge_first = false\n"
        "multiple_spaces_only = true\n"
        "FOR EACH BioSource\n"
        "WHERE INTABLE(SEQID(), table, 1, \" \", merge_del, split_first, convert_multi, merge_first, false, multiple_spaces_only)\n"
        "DO\n"
        "value = ValueFromTable(table, 2);\n"
        "SetStringQual(\"org.taxname\", value, \"eReplace\", false);\n"
        "value = ValueFromTable(table, 3);\n"
        "obj = Resolve(\"org.orgname.mod\") WHERE obj.subtype = \"other\";\n"
        "SetModifier(obj, \"org.orgname.mod\", \"other\", value, \"eReplace\", false);\n"
        "DONE\n"
        "---------------------------------------------------\n";
    BOOST_CHECK(macros.front() == check_macro);
}

const char* sc_Test_BsrcMultipleSpsOnly = " \
User-object ::= { \
  type str \"Column Meta Info\",\
  data { \
    { \
      label str \"Column.1\", \
      data str \"&skipped=false&match=true&name=match Seq-ID&qualifier=SeqId&qual_type=SeqId&blanks=ignore\" \
    }, \
    { \
      label str \"Column.2\", \
      data str \"&skipped=false&match=false&name=taxname&qualifier=taxname&qual_type=Source Qualifier&blanks=ignore&text_separator=;&update_rule=replace\" \
    }, \
    { \
      label str \"Column.3\", \
      data str \"&skipped=false&match=false&name=note-orgmod&qualifier=note-orgmod&qual_type=Source Qualifier&blanks=ignore&text_separator=;&update_rule=replace\" \
    } \
  } \
}";

BOOST_AUTO_TEST_CASE(Test_ApplyGenome)
{
    CRef<CUser_object> column_prop = ReadUserObj(sc_Test_ApplyGenome);
    CTableReaderMacro tbl_macro;
    CTableReaderMacro::SColumnDelimiters del_options("\t");
    auto macros = tbl_macro.CreateMacros("giraffe.tbl", column_prop, del_options, nullptr, true);
    BOOST_CHECK(macros.size() == 1);

    string check_macro =
        "MACRO TableReaderMacro \"Description\"\n"
        "VAR\n"
        "table = \"giraffe.tbl\"\n"
        "merge_del = false\n"
        "split_first = false\n"
        "convert_multi = false\n"
        "merge_first = false\n"
        "multiple_spaces_only = false\n"
        "FOR EACH BioSource\n"
        "WHERE INTABLE(SEQID(), table, 1, \"\\t\", merge_del, split_first, convert_multi, merge_first, false, multiple_spaces_only)\n"
        "DO\n"
        "value = ValueFromTable(table, 2);\n"
        "SetStringQual(\"genome\", value, \"eReplace\", false);\n"
        "DONE\n"
        "---------------------------------------------------\n";
    BOOST_CHECK(macros.front() == check_macro);
}

const char* sc_Test_ApplyGenome = " \
User-object ::= { \
  type str \"Column Meta Info\",\
  data { \
    { \
      label str \"Column.1\", \
      data str \"&skipped=false&match=true&name=match Seq-ID&qualifier=SeqId&qual_type=SeqId&blanks=ignore\" \
    }, \
    { \
      label str \"Column.2\", \
      data str \"&skipped=false&match=false&name=genome&qualifier=genome&qual_type=Source Qualifier&blanks=ignore&text_separator=;&update_rule=replace\" \
    } \
  } \
}";


/* Target = DBLINk
Match_field:
 SeqID - tested indirectly through other targets
 BioSource - Test_DBLinkBsrcMatch
 Molinfo - Test_DBLink_MolinfoMatch
 Molinfo (inst) - sc_Test_DBLinkMolinfoInstMatch
 FeatQual
 CDS-Gene-Prot
 RNA
 Misc
 Pub
 StrComm - Test_DBLink_StrCommFieldMatch
 DBlink - Test_DBLinkMulti
*/

BOOST_AUTO_TEST_CASE(Test_DBLinkMulti)
{
    CRef<CUser_object> column_prop = ReadUserObj(sc_Test_DBLinkMulti);
    CTableReaderMacro tbl_macro;
    CTableReaderMacro::SColumnDelimiters del_options("\t");
    auto macros = tbl_macro.CreateMacros("dblink_multitbl.tbl", column_prop, del_options, nullptr, true);
    BOOST_CHECK(macros.size() == 1);

    string check_macro =
        "MACRO TableReaderMacro \"Description\"\n"
        "VAR\n"
        "table = \"dblink_multitbl.tbl\"\n"
        "merge_del = false\n"
        "split_first = false\n"
        "convert_multi = false\n"
        "merge_first = false\n"
        "multiple_spaces_only = false\n"
        "FOR EACH SeqNA\n"
        "WHERE INTABLE(DBLINKTYPE(\"BioSample\"), table, 1, \"\\t\", merge_del, split_first, convert_multi, merge_first, true, multiple_spaces_only)\n"
        "DO\n"
        "value = ValueFromTable(table, 2);\n"
        "AddDBLink(\"Sequence Read Archive\", value, \"eReplace\", false);\n"
        "DONE\n"
        "---------------------------------------------------\n";
    BOOST_CHECK(macros.front() == check_macro);
}

const char* sc_Test_DBLinkMulti = " \
User-object ::= { \
  type str \"Column Meta Info\",\
  data { \
    { \
      label str \"Column.1\", \
      data str \"&skipped=false&match=true&name=match BioSample&qualifier=BioSample&qual_type=DBLink Field&blanks=ignore\" \
    }, \
    { \
      label str \"Column.2\", \
      data str \"&skipped=false&match=false&name=Sequence Read Archive&qualifier=Sequence Read Archive&qual_type=DBLink Field&blanks=ignore&text_separator=;&update_rule=replace\" \
    } \
  } \
}";


BOOST_AUTO_TEST_CASE(Test_DBLinkBsrcMatch)
{
    CRef<CUser_object> column_prop = ReadUserObj(sc_Test_DBLinkBsrcMatch);
    CTableReaderMacro tbl_macro;
    CTableReaderMacro::SColumnDelimiters del_options("\t");
    auto macros = tbl_macro.CreateMacros("dblink_tbl.txt", column_prop, del_options, nullptr, true);
    BOOST_CHECK(macros.size() == 1);

    string check_macro =
        "MACRO TableReaderMacro \"Description\"\n"
        "VAR\n"
        "table = \"dblink_tbl.txt\"\n"
        "merge_del = false\n"
        "split_first = false\n"
        "convert_multi = false\n"
        "merge_first = false\n"
        "multiple_spaces_only = false\n"
        "FOR EACH SeqNA\n"
        "WHERE INTABLE(BIOSOURCE_FOR_SEQUENCE(\"subtype\", \"clone\"), table, 1, \"\\t\", merge_del, split_first, convert_multi, merge_first, true, multiple_spaces_only)\n"
        "DO\n"
        "value = ValueFromTable(table, 2);\n"
        "AddDBLink(\"BioSample\", value, \"eReplace\", false);\n"
        "DONE\n"
        "---------------------------------------------------\n";
    BOOST_CHECK(macros.front() == check_macro);
}

const char* sc_Test_DBLinkBsrcMatch = " \
User-object ::= { \
  type str \"Column Meta Info\",\
  data { \
    { \
      label str \"Column.1\", \
      data str \"&skipped=false&match=true&name=match clone&qualifier=clone&qual_type=Source Qualifier&blanks=ignore\" \
    }, \
    { \
      label str \"Column.2\", \
      data str \"&skipped=false&match=false&name=BioSample&qualifier=BioSample&qual_type=DBLink Field&blanks=ignore&text_separator=;&update_rule=replace\" \
    } \
  } \
}";

BOOST_AUTO_TEST_CASE(Test_BsrcMolinfoMatch)
{
    CRef<CUser_object> column_prop = ReadUserObj(sc_Test_BsrcMolinfoMatch);
    CTableReaderMacro tbl_macro;
    CTableReaderMacro::SColumnDelimiters del_options(";");
    auto macros = tbl_macro.CreateMacros("molinfo_clonetbl.txt", column_prop, del_options, nullptr, true);
    BOOST_CHECK(macros.size() == 1);

    string check_macro =
        "MACRO TableReaderMacro \"Description\"\n"
        "VAR\n"
        "table = \"molinfo_clonetbl.txt\"\n"
        "merge_del = false\n"
        "split_first = false\n"
        "convert_multi = false\n"
        "merge_first = false\n"
        "multiple_spaces_only = false\n"
        "FOR EACH BioSource\n"
        "WHERE INTABLE(MOLINFO_FOR_BIOSOURCE(\"tech\"), table, 2, \";\", merge_del, split_first, convert_multi, merge_first, true, multiple_spaces_only)\n"
        "DO\n"
        "value = ValueFromTable(table, 3);\n"
        "obj = Resolve(\"subtype\") WHERE obj.subtype = \"clone\";\n"
        "SetModifier(obj, \"subtype\", \"clone\", value, \"eAppend\", \";\", false);\n"
        "DONE\n"
        "---------------------------------------------------\n";
    BOOST_CHECK(macros.front() == check_macro);
}

const char* sc_Test_BsrcMolinfoMatch = " \
User-object ::= { \
  type str \"Column Meta Info\",\
  data { \
    { \
      label str \"Column.1\", \
      data str \"&skipped=true&match=false&name=accession&qualifier=SeqId&qual_type=SeqId&blanks=ignore&text_separator=;&update_rule=add_new_qual\" \
    }, \
    { \
      label str \"Column.2\", \
      data str \"&skipped=false&match=true&name=match technique&qualifier=technique&qual_type=MolInfo Qualifier&blanks=ignore\" \
    }, \
    { \
      label str \"Column.3\", \
      data str \"&skipped=false&match=false&name=clone&qualifier=clone&qual_type=Source Qualifier&blanks=ignore&text_separator=;&update_rule=append\" \
    } \
  } \
}";

BOOST_AUTO_TEST_CASE(Test_BsrcInstMatch)
{
    CRef<CUser_object> column_prop = ReadUserObj(sc_Test_BsrcInstMatch);
    CTableReaderMacro tbl_macro;
    CTableReaderMacro::SColumnDelimiters del_options("\t");
    auto macros = tbl_macro.CreateMacros("topology_tbl.txt", column_prop, del_options, nullptr, true);
    BOOST_CHECK(macros.size() == 1);

    string check_macro =
        "MACRO TableReaderMacro \"Description\"\n"
        "VAR\n"
        "table = \"topology_tbl.txt\"\n"
        "merge_del = false\n"
        "split_first = false\n"
        "convert_multi = false\n"
        "merge_first = false\n"
        "multiple_spaces_only = false\n"
        "FOR EACH BioSource\n"
        "WHERE INTABLE(SEQUENCE_FOR_SEQDESC(\"inst.topology\"), table, 1, \"\\t\", merge_del, split_first, convert_multi, merge_first, true, multiple_spaces_only)\n"
        "DO\n"
        "value = ValueFromTable(table, 2);\n"
        "SetStringQual(\"org.taxname\", value, \"eReplace\", false);\n"
        "DONE\n"
        "---------------------------------------------------\n";
    BOOST_CHECK(macros.front() == check_macro);
}

const char* sc_Test_BsrcInstMatch = " \
User-object ::= { \
  type str \"Column Meta Info\",\
  data { \
    { \
      label str \"Column.1\", \
      data str \"&skipped=false&match=true&name=match topology&qualifier=topology&qual_type=MolInfo Qualifier&blanks=ignore\" \
    }, \
    { \
      label str \"Column.2\", \
      data str \"&skipped=false&match=false&name=taxname&qualifier=taxname&qual_type=Taxname&blanks=ignore&text_separator=;&update_rule=replace\" \
    } \
  } \
}";

BOOST_AUTO_TEST_CASE(Test_BsrcDBLinkMatch)
{
    CRef<CUser_object> column_prop = ReadUserObj(sc_Test_BsrcDBLinkMatch);
    CTableReaderMacro tbl_macro;
    CTableReaderMacro::SColumnDelimiters del_options("\t");
    auto macros = tbl_macro.CreateMacros("fieldname_dblink.txt", column_prop, del_options, nullptr, true);
    BOOST_CHECK(macros.size() == 1);

    string check_macro =
        "MACRO TableReaderMacro \"Description\"\n"
        "VAR\n"
        "table = \"fieldname_dblink.txt\"\n"
        "merge_del = false\n"
        "split_first = false\n"
        "convert_multi = false\n"
        "merge_first = false\n"
        "multiple_spaces_only = false\n"
        "FOR EACH BioSource\n"
        "WHERE INTABLE(DBLINKTYPE(\"BioSample\"), table, 1, \"\\t\", merge_del, split_first, convert_multi, merge_first, true, multiple_spaces_only)\n"
        "DO\n"
        "value = ValueFromTable(table, 2);\n"
        "obj = Resolve(\"subtype\") WHERE obj.subtype = \"other\";\n"
        "SetModifier(obj, \"subtype\", \"other\", value, \"eReplace\", false);\n"
        "DONE\n"
        "---------------------------------------------------\n";
    BOOST_CHECK(macros.front() == check_macro);
}

const char* sc_Test_BsrcDBLinkMatch = " \
User-object ::= { \
  type str \"Column Meta Info\",\
  data { \
    { \
      label str \"Column.1\", \
      data str \"&skipped=false&match=true&name=match BioSample&qualifier=BioSample&qual_type=DBLink Field&blanks=ignore\" \
    }, \
    { \
      label str \"Column.2\", \
      data str \"&skipped=false&match=false&name=note-subsource&qualifier=note-subsource&qual_type=Source Qualifier&blanks=ignore&text_separator=;&update_rule=replace\" \
    } \
  } \
}";

BOOST_AUTO_TEST_CASE(Test_DBLink_StrCommFieldMatch)
{
    CRef<CUser_object> column_prop = ReadUserObj(sc_Test_DBLinkStrCommFieldMatch);
    CTableReaderMacro tbl_macro;
    CTableReaderMacro::SColumnDelimiters del_options("\t");
    auto macros = tbl_macro.CreateMacros("dblink_strcomm.tbl", column_prop, del_options, nullptr, true);
    BOOST_CHECK(macros.size() == 1);

    string check_macro =
        "MACRO TableReaderMacro \"Description\"\n"
        "VAR\n"
        "table = \"dblink_strcomm.tbl\"\n"
        "merge_del = false\n"
        "split_first = false\n"
        "convert_multi = false\n"
        "merge_first = false\n"
        "multiple_spaces_only = false\n"
        "FOR EACH SeqNA\n"
        "WHERE INTABLE(STRUCTCOMMFIELD(\"Genome Coverage\"), table, 2, \"\\t\", merge_del, split_first, convert_multi, merge_first, true, multiple_spaces_only)\n"
        "DO\n"
        "value = ValueFromTable(table, 1);\n"
        "AddDBLink(\"BioProject\", value, \"eReplace\", false);\n"
        "DONE\n"
        "---------------------------------------------------\n";
    BOOST_CHECK(macros.front() == check_macro);
}

const char* sc_Test_DBLinkStrCommFieldMatch = " \
User-object ::= { \
  type str \"Column Meta Info\",\
  data { \
    { \
      label str \"Column.1\", \
      data str \"&skipped=false&match=false&name=BioProject&qualifier=BioProject&qual_type=DBLink Field&blanks=ignore&text_separator=;&update_rule=replace\" \
    }, \
    { \
      label str \"Column.2\", \
      data str \"&skipped=false&match=true&name=match Structured comment Field Genome Coverage&qualifier=Structured comment Field Genome Coverage&qual_type=Structured Comment&blanks=ignore\" \
    } \
  } \
}";

BOOST_AUTO_TEST_CASE(Test_DBLink_MolinfoMatch)
{
    CRef<CUser_object> column_prop = ReadUserObj(sc_Test_DBLinkMolinfoMatch);
    CTableReaderMacro tbl_macro;
    CTableReaderMacro::SColumnDelimiters del_options(":");
    auto macros = tbl_macro.CreateMacros("tech_dblink.tbl", column_prop, del_options, nullptr, true);
    BOOST_CHECK(macros.size() == 1);

    string check_macro =
        "MACRO TableReaderMacro \"Description\"\n"
        "VAR\n"
        "table = \"tech_dblink.tbl\"\n"
        "merge_del = false\n"
        "split_first = false\n"
        "convert_multi = false\n"
        "merge_first = false\n"
        "multiple_spaces_only = false\n"
        "FOR EACH SeqNA\n"
        "WHERE INTABLE(\"descr..molinfo.tech\", table, 2, \":\", merge_del, split_first, convert_multi, merge_first, true, multiple_spaces_only)\n"
        "DO\n"
        "value = ValueFromTable(table, 1);\n"
        "AddDBLink(\"BioSample\", value, \"eReplace\", false);\n"
        "DONE\n"
        "---------------------------------------------------\n";
    BOOST_CHECK(macros.front() == check_macro);
}

const char* sc_Test_DBLinkMolinfoMatch = " \
User-object ::= { \
  type str \"Column Meta Info\",\
  data { \
    { \
      label str \"Column.1\", \
      data str \"&skipped=false&match=false&name=BioSample&qualifier=BioSample&qual_type=DBLink Field&blanks=ignore&text_separator=;&update_rule=replace\" \
    }, \
    { \
      label str \"Column.2\", \
      data str \"&skipped=false&match=true&name=match technique&qualifier=technique&qual_type=MolInfo Qualifier&blanks=ignore\" \
    } \
  } \
}";

BOOST_AUTO_TEST_CASE(Test_DBLink_MolinfoInstMatch)
{
    CRef<CUser_object> column_prop = ReadUserObj(sc_Test_DBLinkMolinfoInstMatch);
    CTableReaderMacro tbl_macro;
    CTableReaderMacro::SColumnDelimiters del_options("\t");
    auto macros = tbl_macro.CreateMacros("topology_dblink.tbl", column_prop, del_options, nullptr, true);
    BOOST_CHECK(macros.size() == 1);

    string check_macro =
        "MACRO TableReaderMacro \"Description\"\n"
        "VAR\n"
        "table = \"topology_dblink.tbl\"\n"
        "merge_del = false\n"
        "split_first = false\n"
        "convert_multi = false\n"
        "merge_first = false\n"
        "multiple_spaces_only = false\n"
        "FOR EACH SeqNA\n"
        "WHERE INTABLE(\"inst.topology\", table, 2, \"\\t\", merge_del, split_first, convert_multi, merge_first, true, multiple_spaces_only)\n"
        "DO\n"
        "value = ValueFromTable(table, 1);\n"
        "AddDBLink(\"BioProject\", value, \"eReplace\", false);\n"
        "DONE\n"
        "---------------------------------------------------\n";
    BOOST_CHECK(macros.front() == check_macro);
}

const char* sc_Test_DBLinkMolinfoInstMatch = " \
User-object ::= { \
  type str \"Column Meta Info\",\
  data { \
    { \
      label str \"Column.1\", \
      data str \"&skipped=false&match=false&name=BioProject&qualifier=BioProject&qual_type=DBLink Field&blanks=ignore&text_separator=;&update_rule=replace\" \
    }, \
    { \
      label str \"Column.2\", \
      data str \"&skipped=false&match=true&name=match topology&qualifier=topology&qual_type=MolInfo Qualifier&blanks=ignore\" \
    } \
  } \
}";


/* Target = Structured Comment
Match_field:
 SeqID - Test_StructCommDB_SeqIdMatch
 BioSource - Test_StructCommField_BsrcMatch
 Molinfo
 Molinfo (inst)
 FeatQual
 CDS-Gene-Prot
 RNA
 Misc
 Pub
 StrComm
 DBlink - Test_StructCommField_DblinkMatch
*/

BOOST_AUTO_TEST_CASE(Test_StructCommField_BsrcMatch)
{
    CRef<CUser_object> column_prop = ReadUserObj(sc_TestStrCommFieldBsrcMatch);
    CTableReaderMacro tbl_macro;
    CTableReaderMacro::SColumnDelimiters del_options("\t");
    auto macros = tbl_macro.CreateMacros("genome_coverage.txt", column_prop, del_options, nullptr, true);
    BOOST_CHECK(macros.size() == 1);

    string check_macro =
        "MACRO TableReaderMacro \"Description\"\n"
        "VAR\n"
        "table = \"genome_coverage.txt\"\n"
        "merge_del = false\n"
        "split_first = false\n"
        "convert_multi = false\n"
        "merge_first = false\n"
        "multiple_spaces_only = false\n"
        "FOR EACH StructComment\n"
        "WHERE INTABLE(BIOSOURCE_FOR_SEQDESC(\"org.orgname.mod\", \"strain\"), table, 4, \"\\t\", merge_del, split_first, convert_multi, merge_first, true, multiple_spaces_only)\n"
        "DO\n"
        "value = ValueFromTable(table, 5);\n"
        "SetStructCommField(\"Genome Coverage\", value, \"eReplace\", false);\n"
        "DONE\n"
        "---------------------------------------------------\n";
    BOOST_CHECK(macros.front() == check_macro);
}

const char* sc_TestStrCommFieldBsrcMatch = " \
User-object ::= { \
  type str \"Column Meta Info\",\
  data { \
    { \
      label str \"Column.1\", \
      data str \"&skipped=true&match=false&name=Col 1&qualifier=SeqId&qual_type=SeqId&blanks=ignore&text_separator=;&update_rule=replace\" \
    }, \
    { \
      label str \"Column.2\", \
      data str \"&skipped=true&match=false&name=Col 2&qualifier=&qual_type=\" \
    }, \
    { \
      label str \"Column.3\", \
      data str \"&skipped=true&match=false&name=Col 3&qualifier=&qual_type=\" \
    }, \
    { \
      label str \"Column.4\", \
      data str \"&skipped=false&match=true&name=match strain&qualifier=strain&qual_type=Source Qualifier&blanks=ignore\" \
    }, \
    { \
      label str \"Column.5\", \
      data str \"&skipped=false&match=false&name=Structured comment Field Genome Coverage&qualifier=Structured comment Field Genome Coverage&qual_type=Structured Comment&blanks=ignore&text_separator=;&update_rule=replace\" \
    } \
  } \
}";

BOOST_AUTO_TEST_CASE(Test_StructCommField_DblinkMatch)
{
    CRef<CUser_object> column_prop = ReadUserObj(sc_TestStrCommFieldDbLinkMatch);
    CTableReaderMacro tbl_macro;
    CTableReaderMacro::SColumnDelimiters del_options("\t");
    auto macros = tbl_macro.CreateMacros("genome_coverage.txt", column_prop, del_options, nullptr, true);
    BOOST_CHECK(macros.size() == 1);

    string check_macro =
        "MACRO TableReaderMacro \"Description\"\n"
        "VAR\n"
        "table = \"genome_coverage.txt\"\n"
        "merge_del = false\n"
        "split_first = false\n"
        "convert_multi = false\n"
        "merge_first = false\n"
        "multiple_spaces_only = false\n"
        "FOR EACH StructComment\n"
        "WHERE INTABLE(DBLINKTYPE(\"BioSample\"), table, 1, \"\\t\", merge_del, split_first, convert_multi, merge_first, true, multiple_spaces_only)\n"
        "DO\n"
        "value = ValueFromTable(table, 2);\n"
        "SetStructCommField(\"Genome Coverage\", value, \"eReplace\", true);\n"
        "DONE\n"
        "---------------------------------------------------\n";
    BOOST_CHECK(macros.front() == check_macro);
}

const char* sc_TestStrCommFieldDbLinkMatch = " \
User-object ::= { \
  type str \"Column Meta Info\",\
  data { \
    { \
      label str \"Column.1\", \
      data str \"&skipped=false&match=true&name=match BioSample&qualifier=BioSample&qual_type=DBLink Field&blanks=ignore\" \
    }, \
    { \
      label str \"Column.2\", \
      data str \"&skipped=false&match=false&name=Structured comment Field Genome Coverage&qualifier=Structured comment Field Genome Coverage&qual_type=Structured Comment&blanks=erase&text_separator=;&update_rule=replace\" \
    } \
  } \
}";

BOOST_AUTO_TEST_CASE(Test_StructCommDB_SeqIdMatch)
{
    CRef<CUser_object> column_prop = ReadUserObj(sc_TestStrCommDBSeqidMatch);
    CTableReaderMacro tbl_macro;
    CTableReaderMacro::SColumnDelimiters del_options("\t");
    auto macros = tbl_macro.CreateMacros("seqid_dbnames.txt", column_prop, del_options, nullptr, true);
    BOOST_CHECK(macros.size() == 1);

    string check_macro =
        "MACRO TableReaderMacro \"Description\"\n"
        "VAR\n"
        "table = \"seqid_dbnames.txt\"\n"
        "merge_del = false\n"
        "split_first = false\n"
        "convert_multi = false\n"
        "merge_first = false\n"
        "multiple_spaces_only = false\n"
        "FOR EACH StructComment\n"
        "WHERE INTABLE(SEQID(), table, 1, \"\\t\", merge_del, split_first, convert_multi, merge_first, false, multiple_spaces_only)\n"
        "DO\n"
        "value = ValueFromTable(table, 3);\n"
        "SetStructCommDb(value, \"eReplace\", false);\n"
        "DONE\n"
        "---------------------------------------------------\n";
    BOOST_CHECK(macros.front() == check_macro);
}

const char* sc_TestStrCommDBSeqidMatch = " \
User-object ::= { \
  type str \"Column Meta Info\",\
  data { \
    { \
      label str \"Column.1\", \
      data str \"&skipped=false&match=true&name=match Seq-ID&qualifier=SeqId&qual_type=SeqId&blanks=ignore\" \
    }, \
    { \
      label str \"Column.2\", \
      data str \"&skipped=true&match=false&name=Col 2&qualifier=&qual_type=\" \
    }, \
    { \
      label str \"Column.3\", \
      data str \"&skipped=false&match=false&name=Structured comment Database Name&qualifier=Structured comment Database Name&qual_type=Structured Comment&blanks=ignore&text_separator=;&update_rule=replace\" \
    } \
  } \
}";


BOOST_AUTO_TEST_CASE(Test_StructCommFieldname_DBLinkMatch)
{
    CRef<CUser_object> column_prop = ReadUserObj(sc_TestStrCommFieldnameDblinkMatch);
    CTableReaderMacro tbl_macro;
    CTableReaderMacro::SColumnDelimiters del_options("\t");
    auto macros = tbl_macro.CreateMacros("fieldname_dblink.txt", column_prop, del_options, nullptr, true);
    BOOST_CHECK(macros.size() == 1);

    string check_macro =
        "MACRO TableReaderMacro \"Description\"\n"
        "VAR\n"
        "table = \"fieldname_dblink.txt\"\n"
        "merge_del = false\n"
        "split_first = false\n"
        "convert_multi = false\n"
        "merge_first = false\n"
        "multiple_spaces_only = false\n"
        "FOR EACH StructComment\n"
        "WHERE INTABLE(DBLINKTYPE(\"BioSample\"), table, 1, \"\\t\", merge_del, split_first, convert_multi, merge_first, true, multiple_spaces_only)\n"
        "DO\n"
        "value = ValueFromTable(table, 2);\n"
        "field = Resolve(\"data\") "
        "WHERE NOT EQUALS(\"field.label.str\", \"StructuredCommentPrefix\", true) AND "
        "NOT EQUALS(\"field.label.str\", \"StructuredCommentSuffix\", true);\n"
        "SetStringQual(\"field.label.str\", value, \"eReplace\", true);\n"
        "DONE\n"
        "---------------------------------------------------\n";
    BOOST_CHECK(macros.front() == check_macro);
}

const char* sc_TestStrCommFieldnameDblinkMatch = " \
User-object ::= { \
  type str \"Column Meta Info\",\
  data { \
    { \
      label str \"Column.1\", \
      data str \"&skipped=false&match=true&name=match BioSample&qualifier=BioSample&qual_type=DBLink Field&blanks=ignore\" \
    }, \
    { \
      label str \"Column.2\", \
      data str \"&skipped=false&match=false&name=Structured comment Field Name&qualifier=Structured comment Field Name&qual_type=Structured Comment&blanks=erase&text_separator=;&update_rule=replace\" \
    } \
  } \
}";


/* Target = MolInfo
Match_field:
 SeqID - Test_Molinfo_SeqIDMatch
 BioSource - Test_Molinfo_BsrcMatch
 Molinfo - Test_Molinfo_MolinfoMatch
 Molinfo (inst) - Test_Molinfo_MolinfoMatch
 FeatQual
 CDS-Gene-Prot
 RNA
 Misc
 Pub
 StrComm
 DBlink - Test_Molinfo_DBLinkMatch
*/

BOOST_AUTO_TEST_CASE(Test_Molinfo_BsrcMatch)
{
    CRef<CUser_object> column_prop = ReadUserObj(sc_TestMolinfoBsrcMatch);
    CTableReaderMacro tbl_macro;
    CTableReaderMacro::SColumnDelimiters del_options(" ");
    del_options.m_Merge_delimiters = true;
    del_options.m_Multispace_to_tab = true;
    auto macros = tbl_macro.CreateMacros("clonetechnique.txt", column_prop, del_options, nullptr, true);
    BOOST_CHECK(macros.size() == 1);

    string check_macro =
        "MACRO TableReaderMacro \"Description\"\n"
        "VAR\n"
        "table = \"clonetechnique.txt\"\n"
        "merge_del = true\n"
        "split_first = false\n"
        "convert_multi = true\n"
        "merge_first = false\n"
        "multiple_spaces_only = false\n"
        "FOR EACH Seq\n"
        "WHERE INTABLE(BIOSOURCE_FOR_SEQUENCE(\"subtype\", \"clone\"), table, 1, \" \", merge_del, split_first, convert_multi, merge_first, true, multiple_spaces_only)\n"
        "DO\n"
        "value = ValueFromTable(table, 2);\n"
        "SetStringQual(\"descr..molinfo.tech\", value, \"eReplace\", false);\n"
        "DONE\n"
        "---------------------------------------------------\n";
    BOOST_CHECK(macros.front() == check_macro);
}

const char* sc_TestMolinfoBsrcMatch = " \
User-object ::= { \
  type str \"Column Meta Info\",\
  data { \
    { \
      label str \"Column.1\", \
      data str \"&skipped=false&match=true&name=match clone&qualifier=clone&qual_type=Source Qualifier&blanks=ignore\" \
    }, \
    { \
      label str \"Column.2\", \
      data str \"&skipped=false&match=false&name=technique&qualifier=technique&qual_type=MolInfo Qualifier&blanks=ignore&text_separator=;&update_rule=replace\" \
    } \
  } \
}";

BOOST_AUTO_TEST_CASE(Test_Molinfo_SeqIDMatch)
{
    CRef<CUser_object> column_prop = ReadUserObj(sc_TestMolinfoSeqIDMatch);
    CTableReaderMacro tbl_macro;
    CTableReaderMacro::SColumnDelimiters del_options(" ");
    del_options.m_Merge_delimiters = true;
    del_options.m_Multispace_to_tab = true;
    auto macros = tbl_macro.CreateMacros("clonetechnique.txt", column_prop, del_options, nullptr, true);
    BOOST_CHECK(macros.size() == 1);

    string check_macro =
        "MACRO TableReaderMacro \"Description\"\n"
        "VAR\n"
        "table = \"clonetechnique.txt\"\n"
        "merge_del = true\n"
        "split_first = false\n"
        "convert_multi = true\n"
        "merge_first = false\n"
        "multiple_spaces_only = false\n"
        "FOR EACH Seq\n"
        "WHERE INTABLE(SEQID(), table, 1, \" \", merge_del, split_first, convert_multi, merge_first, false, multiple_spaces_only)\n"
        "DO\n"
        "value = ValueFromTable(table, 2);\n"
        "SetStringQual(\"descr..molinfo.tech\", value, \"eReplace\", false);\n"
        "DONE\n"
        "---------------------------------------------------\n";
    BOOST_CHECK(macros.front() == check_macro);
}

const char* sc_TestMolinfoSeqIDMatch = " \
User-object ::= { \
  type str \"Column Meta Info\",\
  data { \
    { \
      label str \"Column.1\", \
      data str \"&skipped=false&match=true&name=match Seq-ID&qualifier=SeqId&qual_type=SeqId&blanks=ignore\" \
    }, \
    { \
      label str \"Column.2\", \
      data str \"&skipped=false&match=false&name=technique&qualifier=technique&qual_type=MolInfo Qualifier&blanks=ignore&text_separator=;&update_rule=replace\" \
    } \
  } \
}";

BOOST_AUTO_TEST_CASE(Test_Molinfo_DBLinkMatch)
{
    CRef<CUser_object> column_prop = ReadUserObj(sc_TestMolinfoDBLinkMatch);
    CTableReaderMacro tbl_macro;
    CTableReaderMacro::SColumnDelimiters del_options("\t");
    auto macros = tbl_macro.CreateMacros("molinfo_dblink.txt", column_prop, del_options, nullptr, true);
    BOOST_CHECK(macros.size() == 1);

    string check_macro =
        "MACRO TableReaderMacro \"Description\"\n"
        "VAR\n"
        "table = \"molinfo_dblink.txt\"\n"
        "merge_del = false\n"
        "split_first = false\n"
        "convert_multi = false\n"
        "merge_first = false\n"
        "multiple_spaces_only = false\n"
        "FOR EACH Seq\n"
        "WHERE INTABLE(DBLINKTYPE(\"BioSample\"), table, 1, \"\\t\", merge_del, split_first, convert_multi, merge_first, true, multiple_spaces_only)\n"
        "DO\n"
        "value = ValueFromTable(table, 2);\n"
        "SetStringQual(\"descr..molinfo.biomol\", value, \"eReplace\", false);\n"
        "value = ValueFromTable(table, 3);\n"
        "SetStringQual(\"inst.mol\", value, \"eReplace\", false);\n"
        "DONE\n"
        "---------------------------------------------------\n";
    BOOST_CHECK(macros.front() == check_macro);
}

const char* sc_TestMolinfoDBLinkMatch = " \
User-object ::= { \
  type str \"Column Meta Info\",\
  data { \
    { \
      label str \"Column.1\", \
      data str \"&skipped=false&match=true&name=match BioSample&qualifier=BioSample&qual_type=DBLink Field&blanks=ignore\" \
    }, \
    { \
      label str \"Column.2\", \
      data str \"&skipped=false&match=false&name=molecule&qualifier=molecule&qual_type=MolInfo Qualifier&blanks=ignore&text_separator=;&update_rule=replace\" \
    }, \
    { \
      label str \"Column.3\", \
      data str \"&skipped=false&match=false&name=class&qualifier=class&qual_type=MolInfo Qualifier&blanks=ignore&text_separator=;&update_rule=replace\" \
    }, \
    { \
      label str \"Column.4\", \
      data str \"&skipped=true&match=false&name=Col 4&qualifier=&qual_type=\" \
    } \
  } \
}";

BOOST_AUTO_TEST_CASE(Test_Molinfo_MolinfoMatch)
{
    CRef<CUser_object> column_prop = ReadUserObj(sc_TestMolinfoMolinfoMatch);
    CTableReaderMacro tbl_macro;
    CTableReaderMacro::SColumnDelimiters del_options(";");
    auto macros = tbl_macro.CreateMacros("molinfo_techmol.txt", column_prop, del_options, nullptr, true);
    BOOST_CHECK(macros.size() == 1);

    string check_macro =
        "MACRO TableReaderMacro \"Description\"\n"
        "VAR\n"
        "table = \"molinfo_techmol.txt\"\n"
        "merge_del = false\n"
        "split_first = false\n"
        "convert_multi = false\n"
        "merge_first = false\n"
        "multiple_spaces_only = false\n"
        "FOR EACH Seq\n"
        "WHERE INTABLE(\"descr..molinfo.tech\", table, 2, \";\", merge_del, split_first, convert_multi, merge_first, true, multiple_spaces_only)\n"
        "DO\n"
        "value = ValueFromTable(table, 3);\n"
        "SetStringQual(\"descr..molinfo.biomol\", value, \"eReplace\", false);\n"
        "value = ValueFromTable(table, 4);\n"
        "SetStringQual(\"inst.topology\", value, \"eReplace\", false);\n"
        "DONE\n"
        "---------------------------------------------------\n";
    BOOST_CHECK(macros.front() == check_macro);
}

const char* sc_TestMolinfoMolinfoMatch = " \
User-object ::= { \
  type str \"Column Meta Info\",\
  data { \
    { \
      label str \"Column.1\", \
      data str \"&skipped=true&match=false&name=Col 1&qualifier=SeqId&qual_type=SeqId&blanks=ignore&text_separator=;&update_rule=add_new_qual\" \
    }, \
    { \
      label str \"Column.2\", \
      data str \"&skipped=false&match=true&name=match technique&qualifier=technique&qual_type=MolInfo Qualifier&blanks=ignore\" \
    }, \
    { \
      label str \"Column.3\", \
      data str \"&skipped=false&match=false&name=molecule&qualifier=molecule&qual_type=MolInfo Qualifier&blanks=ignore&text_separator=;&update_rule=replace\" \
    }, \
    { \
      label str \"Column.4\", \
      data str \"&skipped=false&match=false&name=topology&qualifier=topology&qual_type=MolInfo Qualifier&blanks=ignore&text_separator=;&update_rule=replace\" \
    } \
  } \
}";

/* Target = Misc
Match_field:
 SeqID - Test_Misc_SeqIDMatch
 BioSource - Test_Misc_BsrcMatch
 Molinfo
 Molinfo (inst)
 FeatQual - Test_Misc_GeneMatch
 CDS-Gene-Prot - Test_Misc_CdsGeneProtMatch
 RNA
 Misc
 Pub - Test_Misc_PubMatch
 StrComm
 DBlink - Test_Misc_DBLinkMatch
*/

BOOST_AUTO_TEST_CASE(Test_Misc_BsrcMatch)
{
    CRef<CUser_object> column_prop = ReadUserObj(sc_TestMiscBsrcMatch);
    CTableReaderMacro tbl_macro;
    CTableReaderMacro::SColumnDelimiters del_options("\t");
    auto macros = tbl_macro.CreateMacros("clonemisctable.txt", column_prop, del_options, nullptr, true);
    BOOST_CHECK(macros.size() == 1);

    string check_macro =
        "MACRO TableReaderMacro \"Description\"\n"
        "VAR\n"
        "table = \"clonemisctable.txt\"\n"
        "merge_del = false\n"
        "split_first = false\n"
        "convert_multi = false\n"
        "merge_first = false\n"
        "multiple_spaces_only = false\n"
        "FOR EACH SeqNA\n"
        "WHERE INTABLE(BIOSOURCE_FOR_SEQUENCE(\"subtype\", \"clone\"), table, 1, \"\\t\", merge_del, split_first, convert_multi, merge_first, true, multiple_spaces_only)\n"
        "DO\n"
        "value = ValueFromTable(table, 2);\n"
        "SetStringQual(\"descr..title\", value, \"eReplace\", false);\n"
        "value = ValueFromTable(table, 3);\n"
        "SetStringQual(\"descr..comment\", value, \"eReplace\", false);\n"
        "value = ValueFromTable(table, 4);\n"
        "SetStringQual(\"descr..genbank.keywords\", value, \"eReplace\", false);\n"
        "DONE\n"
        "---------------------------------------------------\n";
    BOOST_CHECK(macros.front() == check_macro);
}

const char* sc_TestMiscBsrcMatch = " \
User-object ::= { \
  type str \"Column Meta Info\",\
  data { \
    { \
      label str \"Column.1\", \
      data str \"&skipped=false&match=true&name=match clone&qualifier=clone&qual_type=Source Qualifier&blanks=ignore\" \
    }, \
    { \
      label str \"Column.2\", \
      data str \"&skipped=false&match=false&name=Definition Line&qualifier=Definition Line&qual_type=Misc Qualifiers&blanks=ignore&text_separator=;&update_rule=replace\" \
    }, \
    { \
      label str \"Column.3\", \
      data str \"&skipped=false&match=false&name=Comment Descriptor&qualifier=Comment Descriptor&qual_type=Misc Qualifiers&blanks=ignore&text_separator=;&update_rule=replace\" \
    }, \
    { \
      label str \"Column.4\", \
      data str \"&skipped=false&match=false&name=Keyword&qualifier=Keyword&qual_type=Misc Qualifiers&blanks=ignore&text_separator=;&update_rule=replace\" \
    } \
  } \
}";

BOOST_AUTO_TEST_CASE(Test_Misc_SeqIDMatch)
{
    CRef<CUser_object> column_prop = ReadUserObj(sc_TestMiscSeqIdMatch);
    CTableReaderMacro tbl_macro;
    CTableReaderMacro::SColumnDelimiters del_options("\t");
    auto macros = tbl_macro.CreateMacros("defline_misctable.txt", column_prop, del_options, nullptr, true);
    BOOST_CHECK(macros.size() == 1);

    string check_macro =
        "MACRO TableReaderMacro \"Description\"\n"
        "VAR\n"
        "table = \"defline_misctable.txt\"\n"
        "merge_del = false\n"
        "split_first = false\n"
        "convert_multi = false\n"
        "merge_first = false\n"
        "multiple_spaces_only = false\n"
        "FOR EACH SeqNA\n"
        "WHERE INTABLE(SEQID(), table, 1, \"\\t\", merge_del, split_first, convert_multi, merge_first, false, multiple_spaces_only)\n"
        "DO\n"
        "value = ValueFromTable(table, 3);\n"
        "SetStringQual(\"descr..title\", value, \"eAppend\", \";\", false);\n"
        "DONE\n"
        "---------------------------------------------------\n";
    BOOST_CHECK(macros.front() == check_macro);
}

const char* sc_TestMiscSeqIdMatch = " \
User-object ::= { \
  type str \"Column Meta Info\",\
  data { \
    { \
      label str \"Column.1\", \
      data str \"&skipped=false&match=true&name=match Seq-ID&qualifier=SeqId&qual_type=SeqId&blanks=ignore\" \
    }, \
    { \
      label str \"Column.2\", \
      data str \"&skipped=true&match=false&name=biosample&qualifier=&qual_type=\" \
    }, \
    { \
      label str \"Column.3\", \
      data str \"&skipped=false&match=false&name=Definition Line&qualifier=Definition Line&qual_type=Misc Qualifiers&blanks=ignore&text_separator=;&update_rule=append\" \
    } \
  } \
}";

BOOST_AUTO_TEST_CASE(Test_Misc_DBLinkMatch)
{
    CRef<CUser_object> column_prop = ReadUserObj(sc_TestMiscDBLinkMatch);
    CTableReaderMacro tbl_macro;
    CTableReaderMacro::SColumnDelimiters del_options("\t");
    auto macros = tbl_macro.CreateMacros("defline_misctable.txt", column_prop, del_options, nullptr, true);
    BOOST_CHECK(macros.size() == 1);

    string check_macro =
        "MACRO TableReaderMacro \"Description\"\n"
        "VAR\n"
        "table = \"defline_misctable.txt\"\n"
        "merge_del = false\n"
        "split_first = false\n"
        "convert_multi = false\n"
        "merge_first = false\n"
        "multiple_spaces_only = false\n"
        "FOR EACH SeqNA\n"
        "WHERE INTABLE(DBLINKTYPE(\"BioSample\"), table, 2, \"\\t\", merge_del, split_first, convert_multi, merge_first, true, multiple_spaces_only)\n"
        "DO\n"
        "value = ValueFromTable(table, 3);\n"
        "SetStringQual(\"descr..title\", value, \"ePrepend\", \";\", false);\n"
        "DONE\n"
        "---------------------------------------------------\n";
    BOOST_CHECK(macros.front() == check_macro);
}

const char* sc_TestMiscDBLinkMatch = " \
User-object ::= { \
  type str \"Column Meta Info\",\
  data { \
    { \
      label str \"Column.1\", \
      data str \"&skipped=true&match=false&name=accession&qualifier=SeqId&qual_type=SeqId&blanks=ignore&text_separator=;&update_rule=replace\" \
    }, \
    { \
      label str \"Column.2\", \
      data str \"&skipped=false&match=true&name=match BioSample&qualifier=BioSample&qual_type=DBLink Field&blanks=ignore\" \
    }, \
    { \
      label str \"Column.3\", \
      data str \"&skipped=false&match=false&name=Definition Line&qualifier=Definition Line&qual_type=Misc Qualifiers&blanks=ignore&text_separator=;&update_rule=prefix\" \
    } \
  } \
}";

BOOST_AUTO_TEST_CASE(Test_Misc_PubMatch)
{
    CRef<CUser_object> column_prop = ReadUserObj(sc_TestMiscPubMatch);
    CTableReaderMacro tbl_macro;
    CTableReaderMacro::SColumnDelimiters del_options("\t");
    auto macros = tbl_macro.CreateMacros("pmid_names.txt", column_prop, del_options, nullptr, true);
    BOOST_CHECK(macros.size() == 1);

    string check_macro =
        "MACRO TableReaderMacro \"Description\"\n"
        "VAR\n"
        "table = \"pmid_names.txt\"\n"
        "merge_del = false\n"
        "split_first = false\n"
        "convert_multi = false\n"
        "merge_first = false\n"
        "multiple_spaces_only = false\n"
        "FOR EACH SeqNA\n"
        "WHERE INTABLE(PUB_PMID(), table, 1, \"\\t\", merge_del, split_first, convert_multi, merge_first, true, multiple_spaces_only)\n"
        "DO\n"
        "value = ValueFromTable(table, 2);\n"
        "SetStringQual(\"descr..comment\", value, \"eReplace\", false);\n"
        "DONE\n"
        "---------------------------------------------------\n";
    BOOST_CHECK(macros.front() == check_macro);
}

const char* sc_TestMiscPubMatch = " \
User-object ::= { \
  type str \"Column Meta Info\",\
  data { \
    { \
      label str \"Column.1\", \
      data str \"&skipped=false&match=true&name=match pmid&qualifier=pmid&qual_type=Pub Qualifier&blanks=ignore\" \
    }, \
    { \
      label str \"Column.2\", \
      data str \"&skipped=false&match=false&name=Comment Descriptor&qualifier=Comment Descriptor&qual_type=Misc Qualifiers&blanks=ignore&text_separator=;&update_rule=replace\" \
    } \
  } \
}";


BOOST_AUTO_TEST_CASE(Test_Misc_GeneMatch)
{
    CRef<CUser_object> column_prop = ReadUserObj(sc_TestMiscGeneMatch);
    CTableReaderMacro tbl_macro;
    CTableReaderMacro::SColumnDelimiters del_options("\t");
    auto macros = tbl_macro.CreateMacros("locustag_title.txt", column_prop, del_options, nullptr, true);
    BOOST_CHECK(macros.size() == 1);

    string check_macro =
        "MACRO TableReaderMacro \"Description\"\n"
        "VAR\n"
        "table = \"locustag_title.txt\"\n"
        "merge_del = false\n"
        "split_first = false\n"
        "convert_multi = false\n"
        "merge_first = false\n"
        "multiple_spaces_only = false\n"
        "FOR EACH SeqNA\n"
        "WHERE INTABLE(FEATURES(\"Gene\",\"qual\", \"old_locus_tag\"), table, 1, \"\\t\", merge_del, split_first, convert_multi, merge_first, true, multiple_spaces_only)\n"
        "DO\n"
        "value = ValueFromTable(table, 2);\n"
        "SetStringQual(\"descr..comment\", value, \"eReplace\", false);\n"
        "DONE\n"
        "---------------------------------------------------\n";
    BOOST_CHECK(macros.front() == check_macro);
}

const char* sc_TestMiscGeneMatch = " \
User-object ::= { \
  type str \"Column Meta Info\",\
  data { \
    { \
      label str \"Column.1\", \
      data str \"&skipped=false&match=true&name=match Gene old_locus_tag&qualifier=Gene old_locus_tag&qual_type=Feature Qualifier&blanks=ignore\" \
    }, \
    { \
      label str \"Column.2\", \
      data str \"&skipped=false&match=false&name=Comment Descriptor&qualifier=Comment Descriptor&qual_type=Misc Qualifiers&blanks=ignore&text_separator=;&update_rule=replace\" \
    } \
  } \
}";

BOOST_AUTO_TEST_CASE(Test_Misc_CdsGeneProtMatch)
{
    CRef<CUser_object> column_prop = ReadUserObj(sc_TestMiscCdsGeneProtMatch);
    CTableReaderMacro tbl_macro;
    CTableReaderMacro::SColumnDelimiters del_options("\t");
    auto macros = tbl_macro.CreateMacros("locustag_title.txt", column_prop, del_options, nullptr, true);
    BOOST_CHECK(macros.size() == 1);

    string check_macro =
        "MACRO TableReaderMacro \"Description\"\n"
        "VAR\n"
        "table = \"locustag_title.txt\"\n"
        "merge_del = false\n"
        "split_first = false\n"
        "convert_multi = false\n"
        "merge_first = false\n"
        "multiple_spaces_only = false\n"
        "FOR EACH SeqNA\n"
        "WHERE INTABLE(FEATURES(\"Gene\",\"qual\", \"old_locus_tag\"), table, 1, \"\\t\", merge_del, split_first, convert_multi, merge_first, true, multiple_spaces_only)\n"
        "DO\n"
        "value = ValueFromTable(table, 2);\n"
        "SetStringQual(\"descr..genbank.keywords\", value, \"eReplace\", false);\n"
        "DONE\n"
        "---------------------------------------------------\n";
    BOOST_CHECK(NStr::EqualNocase(macros.front(), check_macro) == true);
}

const char* sc_TestMiscCdsGeneProtMatch = " \
User-object ::= { \
  type str \"Column Meta Info\",\
  data { \
    { \
      label str \"Column.1\", \
      data str \"&skipped=false&match=true&name=match gene old_locus_tag&qualifier=gene old_locus_tag&qual_type=CDS-Gene-Prot Qualifier&blanks=ignore\" \
    }, \
    { \
      label str \"Column.2\", \
      data str \"&skipped=false&match=false&name=Keyword&qualifier=Keyword&qual_type=Misc Qualifiers&blanks=ignore&text_separator=;&update_rule=replace\" \
    } \
  } \
}";


/* Target = Pub
Match_field:
 SeqID - Test_PubAffil_SeqIDMatch
 BioSource - Test_PubVol_BsrcMatch
 Molinfo
 Molinfo (inst)
 FeatQual
 CDS-Gene-Prot
 RNA
 Misc
 Pub - Test_Pub_PubMatch
 StrComm - Test_Pub_StrCommMatch
 DBlink - Test_Pub_DBLinkMatch
 */

BOOST_AUTO_TEST_CASE(Test_PubAffil_SeqIDMatch)
{
    CRef<CUser_object> column_prop = ReadUserObj(sc_TestPubSeqIDMatch);
    CTableReaderMacro tbl_macro;
    CTableReaderMacro::SColumnDelimiters del_options("\t");
    auto macros = tbl_macro.CreateMacros("pubaffil_tbl.txt", column_prop, del_options, nullptr, true);
    BOOST_CHECK(macros.size() == 1);

    string check_macro =
        "MACRO TableReaderMacro \"Description\"\n"
        "VAR\n"
        "table = \"pubaffil_tbl.txt\"\n"
        "merge_del = false\n"
        "split_first = false\n"
        "convert_multi = false\n"
        "merge_first = false\n"
        "multiple_spaces_only = false\n"
        "FOR EACH Pubdesc\n"
        "WHERE INTABLE(SEQID(), table, 1, \"\\t\", merge_del, split_first, convert_multi, merge_first, false, multiple_spaces_only)\n"
        "DO\n"
        "value = ValueFromTable(table, 2);\n"
        "SetPubAffil(\"affil\", value, \"eReplace\", false);\n"
        "value = ValueFromTable(table, 3);\n"
        "SetPubAffil(\"div\", value, \"eReplace\", false);\n"
        "value = ValueFromTable(table, 4);\n"
        "SetPubAffil(\"street\", value, \"eReplace\", false);\n"
        "value = ValueFromTable(table, 5);\n"
        "SetPubAffil(\"country\", value, \"eReplace\", false);\n"
        "value = ValueFromTable(table, 6);\n"
        "title = PUB_TITLE();\n"
        "SetPubTitle(title, value, \"eReplace\", false);\n"
        "DONE\n"
        "---------------------------------------------------\n";
    BOOST_CHECK(macros.front() == check_macro);
}

const char* sc_TestPubSeqIDMatch = " \
User-object ::= { \
  type str \"Column Meta Info\",\
  data { \
    { \
      label str \"Column.1\", \
      data str \"&skipped=false&match=true&name=match Seq-ID&qualifier=SeqId&qual_type=SeqId&blanks=ignore\" \
    }, \
    { \
      label str \"Column.2\", \
      data str \"&skipped=false&match=false&name=affiliation institution&qualifier=affiliation institution&qual_type=Pub Qualifier&blanks=ignore&text_separator=;&update_rule=replace\" \
    }, \
    { \
      label str \"Column.3\", \
      data str \"&skipped=false&match=false&name=affiliation department&qualifier=affiliation department&qual_type=Pub Qualifier&blanks=ignore&text_separator=;&update_rule=replace\" \
    }, \
    { \
      label str \"Column.4\", \
      data str \"&skipped=false&match=false&name=affiliation street&qualifier=affiliation street&qual_type=Pub Qualifier&blanks=ignore&text_separator=;&update_rule=replace\" \
    }, \
    { \
      label str \"Column.5\", \
      data str \"&skipped=false&match=false&name=affiliation country&qualifier=affiliation country&qual_type=Pub Qualifier&blanks=ignore&text_separator=;&update_rule=replace\" \
    }, \
    { \
      label str \"Column.6\", \
      data str \"&skipped=false&match=false&name=title&qualifier=title&qual_type=Pub Qualifier&blanks=ignore&text_separator=;&update_rule=replace\" \
    }, \
    { \
      label str \"Column.7\", \
      data str \"&skipped=true&match=false&name=Col 7&qualifier=&qual_type=\" \
    } \
  } \
}";

BOOST_AUTO_TEST_CASE(Test_PubVol_BsrcMatch)
{
    CRef<CUser_object> column_prop = ReadUserObj(sc_TestPubBsrcMatch);
    CTableReaderMacro tbl_macro;
    CTableReaderMacro::SColumnDelimiters del_options("\t");
    auto macros = tbl_macro.CreateMacros("pubvol_bsrctable.txt", column_prop, del_options, nullptr, true);
    BOOST_CHECK(macros.size() == 1);

    string check_macro =
        "MACRO TableReaderMacro \"Description\"\n"
        "VAR\n"
        "table = \"pubvol_bsrctable.txt\"\n"
        "merge_del = false\n"
        "split_first = false\n"
        "convert_multi = false\n"
        "merge_first = false\n"
        "multiple_spaces_only = false\n"
        "FOR EACH Pubdesc\n"
        "WHERE INTABLE(BIOSOURCE_FOR_SEQDESC(\"org.taxname\"), table, 1, \"\\t\", merge_del, split_first, convert_multi, merge_first, true, multiple_spaces_only)\n"
        "DO\n"
        "value = ValueFromTable(table, 2);\n"
        "SetPubVolume(value, \"eReplace\", false);\n"
        "value = ValueFromTable(table, 3);\n"
        "SetPubIssue(value, \"eReplace\", false);\n"
        "value = ValueFromTable(table, 4);\n"
        "SetPubPages(value, \"eReplace\", false);\n"
        "value = ValueFromTable(table, 5);\n"
        "SetPubJournal(value, \"eReplace\", false);\n"
        "DONE\n"
        "---------------------------------------------------\n";
    BOOST_CHECK(macros.front() == check_macro);
}

const char* sc_TestPubBsrcMatch = " \
User-object ::= { \
  type str \"Column Meta Info\",\
  data { \
    { \
      label str \"Column.1\", \
      data str \"&skipped=false&match=true&name=match Taxname&qualifier=Taxname&qual_type=Taxname&blanks=ignore\" \
    }, \
    { \
      label str \"Column.2\", \
      data str \"&skipped=false&match=false&name=volume&qualifier=volume&qual_type=Pub Qualifier&blanks=ignore&text_separator=;&update_rule=replace\" \
    }, \
    { \
      label str \"Column.3\", \
      data str \"&skipped=false&match=false&name=issue&qualifier=issue&qual_type=Pub Qualifier&blanks=ignore&text_separator=;&update_rule=replace\" \
    }, \
    { \
      label str \"Column.4\", \
      data str \"&skipped=false&match=false&name=pages&qualifier=pages&qual_type=Pub Qualifier&blanks=ignore&text_separator=;&update_rule=replace\" \
    }, \
    { \
      label str \"Column.5\", \
      data str \"&skipped=false&match=false&name=journal&qualifier=journal&qual_type=Pub Qualifier&blanks=ignore&text_separator=;&update_rule=replace\" \
    }, \
    { \
      label str \"Column.6\", \
      data str \"&skipped=true&match=false&name=Column 6&qualifier=&qual_type=\" \
    } \
  } \
}";


BOOST_AUTO_TEST_CASE(Test_Pub_DBLinkMatch)
{
    CRef<CUser_object> column_prop = ReadUserObj(sc_TestPubDBLinkMatch);
    CTableReaderMacro tbl_macro;
    CTableReaderMacro::SColumnDelimiters del_options("\t");
    auto macros = tbl_macro.CreateMacros("pubauth_dblinktbl.txt", column_prop, del_options, nullptr, true);
    BOOST_CHECK(macros.size() == 1);

    string check_macro =
        "MACRO TableReaderMacro \"Description\"\n"
        "VAR\n"
        "table = \"pubauth_dblinktbl.txt\"\n"
        "merge_del = false\n"
        "split_first = false\n"
        "convert_multi = false\n"
        "merge_first = false\n"
        "multiple_spaces_only = false\n"
        "FOR EACH Pubdesc\n"
        "WHERE INTABLE(DBLINKTYPE(\"BioSample\"), table, 1, \"\\t\", merge_del, split_first, convert_multi, merge_first, true, multiple_spaces_only)\n"
        "DO\n"
        "value = ValueFromTable(table, 2);\n"
        "AddAuthorList(value, false);\n"
        "value = ValueFromTable(table, 3);\n"
        "SetPubJournal(value, \"eReplace\", false);\n"
        "value = ValueFromTable(table, 4);\n"
        "SetPubVolume(value, \"eReplace\", false);\n"
        "value = ValueFromTable(table, 5);\n"
        "SetPubIssue(value, \"eReplace\", false);\n"
        "value = ValueFromTable(table, 6);\n"
        "SetPubPages(value, \"eReplace\", false);\n"
        "value = ValueFromTable(table, 7);\n"
        "SetPubDate(value);\n"
        "value = ValueFromTable(table, 8);\n"
        "title = PUB_TITLE();\n"
        "SetPubTitle(title, value, \"eReplace\", false);\n"
        "DONE\n"
        "---------------------------------------------------\n";
    BOOST_CHECK(macros.front() == check_macro);
}

const char* sc_TestPubDBLinkMatch = " \
User-object ::= { \
  type str \"Column Meta Info\",\
  data { \
    { \
      label str \"Column.1\", \
      data str \"&skipped=false&match=true&name=match BioSample&qualifier=BioSample&qual_type=DBLink Field&blanks=ignore\" \
    }, \
    { \
      label str \"Column.2\", \
      data str \"&skipped=false&match=false&name=author name list&qualifier=author name list&qual_type=Pub Qualifier&blanks=ignore&text_separator=;&update_rule=replace\" \
    }, \
    { \
      label str \"Column.3\", \
      data str \"&skipped=false&match=false&name=journal&qualifier=journal&qual_type=Pub Qualifier&blanks=ignore&text_separator=;&update_rule=replace\" \
    }, \
    { \
      label str \"Column.4\", \
      data str \"&skipped=false&match=false&name=volume&qualifier=volume&qual_type=Pub Qualifier&blanks=ignore&text_separator=;&update_rule=replace\" \
    }, \
    { \
      label str \"Column.5\", \
      data str \"&skipped=false&match=false&name=issue&qualifier=issue&qual_type=Pub Qualifier&blanks=ignore&text_separator=;&update_rule=replace\" \
    }, \
    { \
      label str \"Column.6\", \
      data str \"&skipped=false&match=false&name=pages&qualifier=pages&qual_type=Pub Qualifier&blanks=ignore&text_separator=;&update_rule=replace\" \
    }, \
    { \
      label str \"Column.7\", \
      data str \"&skipped=false&match=false&name=date&qualifier=date&qual_type=Pub Qualifier&blanks=ignore&text_separator=;&update_rule=replace\" \
    }, \
    { \
      label str \"Column.8\", \
      data str \"&skipped=false&match=false&name=title&qualifier=title&qual_type=Pub Qualifier&blanks=ignore&text_separator=;&update_rule=replace\" \
    } \
  } \
}";

BOOST_AUTO_TEST_CASE(Test_Pub_PubMatch)
{
    CRef<CUser_object> column_prop = ReadUserObj(sc_TestPubPubMatch);
    CTableReaderMacro tbl_macro;
    CTableReaderMacro::SColumnDelimiters del_options("\t");
    auto macros = tbl_macro.CreateMacros("pmid_names.txt", column_prop, del_options, nullptr, true);
    BOOST_CHECK(macros.size() == 1);

    string check_macro =
        "MACRO TableReaderMacro \"Description\"\n"
        "VAR\n"
        "table = \"pmid_names.txt\"\n"
        "merge_del = false\n"
        "split_first = false\n"
        "convert_multi = false\n"
        "merge_first = false\n"
        "multiple_spaces_only = false\n"
        "FOR EACH Pubdesc\n"
        "WHERE INTABLE(PUB_PMID(), table, 1, \"\\t\", merge_del, split_first, convert_multi, merge_first, true, multiple_spaces_only)\n"
        "DO\n"
        "value = ValueFromTable(table, 2);\n"
        "AddAuthorList(value, false);\n"
        "DONE\n"
        "---------------------------------------------------\n";
    BOOST_CHECK(macros.front() == check_macro);
}

const char* sc_TestPubPubMatch = " \
User-object ::= { \
  type str \"Column Meta Info\",\
  data { \
    { \
      label str \"Column.1\", \
      data str \"&skipped=false&match=true&name=match pmid&qualifier=pmid&qual_type=Pub Qualifier&blanks=ignore&update_rule=add_new_qual\" \
    }, \
    { \
      label str \"Column.2\", \
      data str \"&skipped=false&match=false&name=author name list&qualifier=author name list&qual_type=Pub Qualifier&blanks=ignore&text_separator=;&update_rule=replace\" \
    } \
  } \
}";

BOOST_AUTO_TEST_CASE(Test_Pub_StrCommMatch)
{
    CRef<CUser_object> column_prop = ReadUserObj(sc_TestPubStrCommMatch);
    CTableReaderMacro tbl_macro;
    CTableReaderMacro::SColumnDelimiters del_options("\t");
    auto macros = tbl_macro.CreateMacros("dbname_pubname.txt", column_prop, del_options, nullptr, true);
    BOOST_CHECK(macros.size() == 1);

    string check_macro =
        "MACRO TableReaderMacro \"Description\"\n"
        "VAR\n"
        "table = \"dbname_pubname.txt\"\n"
        "merge_del = false\n"
        "split_first = false\n"
        "convert_multi = false\n"
        "merge_first = false\n"
        "multiple_spaces_only = false\n"
        "FOR EACH Pubdesc\n"
        "WHERE INTABLE(STRUCTCOMMDB(), table, 1, \"\\t\", merge_del, split_first, convert_multi, merge_first, true, multiple_spaces_only)\n"
        "DO\n"
        "value = ValueFromTable(table, 2);\n"
        "auth_obj = PUB_AUTHORS();\n"
        "SetPubAuthorName(auth_obj, \"first\", value, \"eReplace\", false);\n"
        "DONE\n"
        "---------------------------------------------------\n";
    BOOST_CHECK(macros.front() == check_macro);
}

const char* sc_TestPubStrCommMatch = " \
User-object ::= { \
  type str \"Column Meta Info\",\
  data { \
    { \
      label str \"Column.1\", \
      data str \"&skipped=false&match=true&name=match Structured comment Database Name&qualifier=Structured comment Database Name&qual_type=Structured Comment&blanks=ignore&update_rule=add_new_qual\" \
    }, \
    { \
      label str \"Column.2\", \
      data str \"&skipped=false&match=false&name=author first name&qualifier=author first name&qual_type=Pub Qualifier&blanks=ignore&text_separator=;&update_rule=replace\" \
    } \
  } \
}";

/* Target = Features
Match_field:
 SeqID - Test_FeatGene_IDMatch
 BioSource - Test_FeatProtein_BsrcMatch
 Molinfo
 Molinfo (inst)
 FeatQual - Test_FeatGene_GeneMatch, Test_FeatGene1_GeneMatch, Test_FeatProtein_GeneMatch, Test_FeatCDS_ProteinMatch, Test_GeneSetDbXref
 CDS-Gene-Prot - Test_FeatGene_GeneMatch
 RNA
 Misc
 Pub
 StrComm
 DBlink - Test_FeatProtein_DBLinkMatch
 */

BOOST_AUTO_TEST_CASE(Test_FeatGene_GeneMatch)
{
    CRef<CUser_object> column_prop = ReadUserObj(sc_TestFeatGene_GeneMatch);
    CTableReaderMacro tbl_macro;
    CTableReaderMacro::SColumnDelimiters del_options("\t");
    auto macros = tbl_macro.CreateMacros("applygenetable.txt", column_prop, del_options, nullptr, true);
    BOOST_CHECK(macros.size() == 1);

    string check_macro =
        "MACRO TableReaderMacro \"Description\"\n"
        "VAR\n"
        "table = \"applygenetable.txt\"\n"
        "merge_del = false\n"
        "split_first = false\n"
        "convert_multi = false\n"
        "merge_first = false\n"
        "multiple_spaces_only = false\n"
        "update_mrna = false\n"
        "FOR EACH Gene\n"
        "WHERE INTABLE(\"data.gene.locus-tag\", table, 2, \"\\t\", merge_del, split_first, convert_multi, merge_first, true, multiple_spaces_only)\n"
        "DO\n"
        "value = ValueFromTable(table, 4);\n"
        "SetStringQual(\"data.gene.locus\", value, \"eReplace\", false);\n"
        "DONE\n"
        "---------------------------------------------------\n";
    BOOST_CHECK(macros.front() == check_macro);

    column_prop = ReadUserObj(sc_TestFeatGene_CdsGeneProtMatch);
    macros = tbl_macro.CreateMacros("applygenetable.txt", column_prop, del_options, nullptr, true);
    BOOST_CHECK(macros.size() == 1);
    BOOST_CHECK(macros.front() == check_macro);
}

const char* sc_TestFeatGene_GeneMatch = " \
User-object ::= { \
  type str \"Column Meta Info\",\
  data { \
    { \
      label str \"Column.1\", \
      data str \"&skipped=true&match=false&name=Col 1&qualifier=SeqId&qual_type=SeqId&blanks=ignore&text_separator=;&update_rule=replace\" \
    }, \
    { \
      label str \"Column.2\", \
      data str \"&skipped=false&match=true&name=match Gene locus_tag&qualifier=Gene locus_tag&qual_type=Feature Qualifier&blanks=ignore\" \
    }, \
    { \
      label str \"Column.3\", \
      data str \"&skipped=true&match=false&name=Col 3&qualifier=&qual_type=\" \
    }, \
    { \
      label str \"Column.4\", \
      data str \"&skipped=false&match=false&name=Gene locus&qualifier=Gene locus&qual_type=Feature Qualifier&blanks=ignore&text_separator=;&update_rule=replace\" \
    } \
  } \
}";

const char* sc_TestFeatGene_CdsGeneProtMatch = " \
User-object ::= { \
  type str \"Column Meta Info\",\
  data { \
    { \
      label str \"Column.1\", \
      data str \"&skipped=true&match=false&name=Col 1&qualifier=SeqId&qual_type=SeqId&blanks=ignore&text_separator=;&update_rule=replace\" \
    }, \
    { \
      label str \"Column.2\", \
      data str \"&skipped=false&match=true&name=match gene locus tag&qualifier=gene locus tag&qual_type=CDS-Gene-Prot Qualifier&blanks=ignore\" \
    }, \
    { \
      label str \"Column.3\", \
      data str \"&skipped=true&match=false&name=Col 3&qualifier=&qual_type=\" \
    }, \
    { \
      label str \"Column.4\", \
      data str \"&skipped=false&match=false&name=Gene locus&qualifier=Gene locus&qual_type=Feature Qualifier&blanks=ignore&text_separator=;&update_rule=replace\" \
    } \
  } \
}";

BOOST_AUTO_TEST_CASE(Test_FeatGene1_GeneMatch)
{
    CRef<CUser_object> column_prop = ReadUserObj(sc_TestFeatGene1_GeneMatch);
    CTableReaderMacro tbl_macro;
    CTableReaderMacro::SColumnDelimiters del_options("\t");
    auto macros = tbl_macro.CreateMacros("applygenetable.txt", column_prop, del_options, nullptr, true);
    BOOST_CHECK(macros.size() == 1);

    string check_macro =
        "MACRO TableReaderMacro \"Description\"\n"
        "VAR\n"
        "table = \"applygenetable.txt\"\n"
        "merge_del = false\n"
        "split_first = false\n"
        "convert_multi = false\n"
        "merge_first = false\n"
        "multiple_spaces_only = false\n"
        "update_mrna = false\n"
        "FOR EACH Gene\n"
        "WHERE INTABLE(\"data.gene.locus-tag\", table, 2, \"\\t\", merge_del, split_first, convert_multi, merge_first, true, multiple_spaces_only)\n"
        "DO\n"
        "value = ValueFromTable(table, 4);\n"
        "SetStringQual(\"data.gene.locus\", value, \"eReplace\", false);\n"
        "value = ValueFromTable(table, 6);\n"
        "SetRelatedFeatureQual(\"Protein\", \"data.prot.name\", value, \"eReplace\", false, update_mrna);\n"
        "value = ValueFromTable(table, 8);\n"
        "SetRelatedFeatureQual(\"CDS\", \"comment\", value, \"eReplace\", false, update_mrna);\n"
        "DONE\n"
        "---------------------------------------------------\n";
    BOOST_CHECK(macros.front() == check_macro);
}

const char* sc_TestFeatGene1_GeneMatch = " \
User-object ::= { \
  type str \"Column Meta Info\",\
  data { \
    { \
      label str \"Column.1\", \
      data str \"&skipped=true&match=false&name=Col 1&qualifier=SeqId&qual_type=SeqId&blanks=ignore&text_separator=;&update_rule=replace\" \
    }, \
    { \
      label str \"Column.2\", \
      data str \"&skipped=false&match=true&name=match Gene locus_tag&qualifier=Gene locus_tag&qual_type=Feature Qualifier&blanks=ignore\" \
    }, \
    { \
      label str \"Column.3\", \
      data str \"&skipped=true&match=false&name=Col 3&qualifier=&qual_type=\" \
    }, \
    { \
      label str \"Column.4\", \
      data str \"&skipped=false&match=false&name=Gene locus&qualifier=Gene locus&qual_type=Feature Qualifier&blanks=ignore&text_separator=;&update_rule=replace\" \
    }, \
    { \
      label str \"Column.5\", \
      data str \"&skipped=true&match=false&name=Col 5&qualifier=&qual_type=\" \
    }, \
    { \
      label str \"Column.6\", \
      data str \"&skipped=false&match=false&name=CDS product&qualifier=CDS product&qual_type=Feature Qualifier&blanks=ignore&text_separator=;&update_rule=replace\" \
    }, \
    { \
      label str \"Column.7\", \
      data str \"&skipped=true&match=false&name=Col 7&qualifier=&qual_type=\" \
    }, \
    { \
      label str \"Column.8\", \
      data str \"&skipped=false&match=false&name=CDS note&qualifier=CDS note&qual_type=Feature Qualifier&blanks=ignore&text_separator=;&update_rule=replace\" \
    } \
  } \
}";

void CheckCreateProtFeatMacro(const string& macro)
{
    string prot_feat_macro =
        "MACRO CreateProtFeats \"Create full length protein features\"\n"
        "FOR EACH TSEntry\n"
        "DO\n"
        "CreateProteinFeatures();\n"
        "DONE\n"
        "---------------------------------------------------\n";
    BOOST_CHECK(macro == prot_feat_macro);
}

BOOST_AUTO_TEST_CASE(Test_FeatProtein_GeneMatch)
{
    CRef<CUser_object> column_prop = ReadUserObj(sc_TestFeatProtein_GeneMatch);
    CTableReaderMacro tbl_macro;
    CTableReaderMacro::SColumnDelimiters del_options("\t");
    auto macros = tbl_macro.CreateMacros("applygenetable.txt", column_prop, del_options, nullptr, true);
    BOOST_CHECK(macros.size() == 2);

    CheckCreateProtFeatMacro(macros.front());

    string check_macro =
        "MACRO TableReaderMacro \"Description\"\n"
        "VAR\n"
        "table = \"applygenetable.txt\"\n"
        "merge_del = false\n"
        "split_first = false\n"
        "convert_multi = false\n"
        "merge_first = false\n"
        "multiple_spaces_only = false\n"
        "update_mrna = false\n"
        "FOR EACH Protein\n"
        "WHERE INTABLE(RELATED_FEATURE(\"Gene\", \"data.gene.locus-tag\"), table, 2, \"\\t\", merge_del, split_first, convert_multi, merge_first, true, multiple_spaces_only)\n"
        "DO\n"
        "value = ValueFromTable(table, 4);\n"
        "SetStringQual(\"data.prot.name\", value, \"eReplace\", false);\n"
        "DONE\n"
        "---------------------------------------------------\n";
    BOOST_CHECK(macros[1] == check_macro);

    column_prop = ReadUserObj(sc_TestCGPProtein_GeneMatch);
    macros = tbl_macro.CreateMacros("applygenetable.txt", column_prop, del_options, nullptr, true);
    BOOST_CHECK(macros.size() == 2);

    CheckCreateProtFeatMacro(macros.front());
    BOOST_CHECK(NStr::EqualNocase(macros[1], check_macro) == true);

}

const char* sc_TestFeatProtein_GeneMatch = " \
User-object ::= { \
  type str \"Column Meta Info\",\
  data { \
    { \
      label str \"Column.1\", \
      data str \"&skipped=true&match=false&name=Col 1&qualifier=SeqId&qual_type=SeqId&blanks=ignore&text_separator=;&update_rule=replace\" \
    }, \
    { \
      label str \"Column.2\", \
      data str \"&skipped=false&match=true&name=match Gene locus_tag&qualifier=Gene locus_tag&qual_type=Feature Qualifier&blanks=ignore\" \
    }, \
    { \
      label str \"Column.3\", \
      data str \"&skipped=true&match=false&name=Col 3&qualifier=&qual_type=\" \
    }, \
    { \
      label str \"Column.4\", \
      data str \"&skipped=false&match=false&name=CDS product&qualifier=CDS product&qual_type=Feature Qualifier&blanks=ignore&text_separator=;&update_rule=replace\" \
    } \
  } \
}";


const char* sc_TestCGPProtein_GeneMatch = " \
User-object ::= { \
  type str \"Column Meta Info\",\
  data { \
    { \
      label str \"Column.1\", \
      data str \"&skipped=true&match=false&name=Col 1&qualifier=SeqId&qual_type=SeqId&blanks=ignore&text_separator=;&update_rule=replace\" \
    }, \
    { \
      label str \"Column.2\", \
      data str \"&skipped=false&match=true&name=match gene locus tag&qualifier=gene locus tag&qual_type=CDS-Gene-Prot Qualifier&blanks=ignore\" \
    }, \
    { \
      label str \"Column.3\", \
      data str \"&skipped=true&match=false&name=Col 3&qualifier=&qual_type=\" \
    }, \
    { \
      label str \"Column.4\", \
      data str \"&skipped=false&match=false&name=protein name&qualifier=protein name&qual_type=CDS-Gene-Prot Qualifier&blanks=ignore&text_separator=;&update_rule=replace\" \
    } \
  } \
}";


BOOST_AUTO_TEST_CASE(Test_FeatCDS_ProteinMatch)
{
    CRef<CUser_object> column_prop = ReadUserObj(sc_TestFeatCDS_ProteinMatch);
    CTableReaderMacro tbl_macro;
    CTableReaderMacro::SColumnDelimiters del_options("\t");
    auto macros = tbl_macro.CreateMacros("applygenetable.txt", column_prop, del_options, nullptr, true);
    BOOST_CHECK(macros.size() == 1);

    string check_macro =
        "MACRO TableReaderMacro \"Description\"\n"
        "VAR\n"
        "table = \"applygenetable.txt\"\n"
        "merge_del = false\n"
        "split_first = false\n"
        "convert_multi = false\n"
        "merge_first = false\n"
        "multiple_spaces_only = false\n"
        "update_mrna = false\n"
        "FOR EACH Cdregion\n"
        "WHERE INTABLE(RELATED_FEATURE(\"protein\", \"data.prot.name\"), table, 4, \"\\t\", merge_del, split_first, convert_multi, merge_first, true, multiple_spaces_only)\n"
        "DO\n"
        "value = ValueFromTable(table, 2);\n"
        "SetStringQual(\"comment\", value, \"eReplace\", true);\n"
        "DONE\n"
        "---------------------------------------------------\n";
    BOOST_CHECK(macros.front() == check_macro);

    column_prop = ReadUserObj(sc_TestCGPCDS_ProteinMatch);
    macros = tbl_macro.CreateMacros("applygenetable.txt", column_prop, del_options, nullptr, true);
    BOOST_CHECK(macros.size() == 1);

    BOOST_CHECK(macros[0] == check_macro);
}

const char* sc_TestFeatCDS_ProteinMatch = " \
User-object ::= { \
  type str \"Column Meta Info\",\
  data { \
    { \
      label str \"Column.1\", \
      data str \"&skipped=true&match=false&name=Col 1&qualifier=SeqId&qual_type=SeqId&blanks=ignore&text_separator=;&update_rule=replace\" \
    }, \
    { \
      label str \"Column.2\", \
      data str \"&skipped=false&match=false&name=CDS note&qualifier=CDS note&qual_type=Feature Qualifier&blanks=erase&text_separator=;&update_rule=replace\" \
    }, \
    { \
      label str \"Column.3\", \
      data str \"&skipped=true&match=false&name=Col 7&qualifier=&qual_type=\" \
    }, \
    { \
      label str \"Column.4\", \
      data str \"&skipped=false&match=true&name=match CDS product&qualifier=CDS product&qual_type=Feature Qualifier&blanks=ignore&text_separator=;&update_rule=replace\" \
    } \
  } \
}";

const char* sc_TestCGPCDS_ProteinMatch = " \
User-object ::= { \
  type str \"Column Meta Info\",\
  data { \
    { \
      label str \"Column.1\", \
      data str \"&skipped=true&match=false&name=Col 1&qualifier=&qual_type=\" \
    }, \
    { \
      label str \"Column.2\", \
      data str \"&skipped=false&match=false&name=CDS comment&qualifier=CDS comment&qual_type=CDS-Gene-Prot Qualifier&blanks=erase&text_separator=;&update_rule=replace\" \
    }, \
    { \
      label str \"Column.3\", \
      data str \"&skipped=true&match=false&name=Col 3&qualifier=&qual_type=\" \
    }, \
    { \
      label str \"Column.4\", \
      data str \"&skipped=false&match=true&name=match protein name&qualifier=protein name&qual_type=CDS-Gene-Prot Qualifier&blanks=ignore&text_separator=;&update_rule=replace\" \
    } \
  } \
}";


BOOST_AUTO_TEST_CASE(Test_FeatGene_IDMatch)
{
    CRef<CUser_object> column_prop = ReadUserObj(sc_TestFeatGene_IDMatch);
    CTableReaderMacro tbl_macro;
    CTableReaderMacro::SColumnDelimiters del_options("\t");
    auto macros = tbl_macro.CreateMacros("featqual_table.txt", column_prop, del_options, nullptr, true);
    BOOST_CHECK(macros.size() == 1);

    string check_macro =
        "MACRO TableReaderMacro \"Description\"\n"
        "VAR\n"
        "table = \"featqual_table.txt\"\n"
        "merge_del = false\n"
        "split_first = false\n"
        "convert_multi = false\n"
        "merge_first = false\n"
        "multiple_spaces_only = false\n"
        "update_mrna = false\n"
        "FOR EACH Gene\n"
        "WHERE INTABLE(SEQID(), table, 1, \"\\t\", merge_del, split_first, convert_multi, merge_first, false, multiple_spaces_only)\n"
        "DO\n"
        "value = ValueFromTable(table, 4);\n"
        "SetStringQual(\"data.gene.syn\", value, \"eAddQual\", false);\n"
        "DONE\n"
        "---------------------------------------------------\n";
    BOOST_CHECK(macros.front() == check_macro);

    column_prop = ReadUserObj(sc_TestCGPGene_IDMatch);
    macros = tbl_macro.CreateMacros("featqual_table.txt", column_prop, del_options, nullptr, true);
    BOOST_CHECK(macros.size() == 1);

    BOOST_CHECK(macros[0] == check_macro);
}

const char* sc_TestFeatGene_IDMatch = " \
User-object ::= { \
  type str \"Column Meta Info\",\
  data { \
    { \
      label str \"Column.1\", \
      data str \"&skipped=false&match=true&name=match accession&qualifier=SeqId&qual_type=SeqId&blanks=ignore\" \
    }, \
    { \
      label str \"Column.2\", \
      data str \"&skipped=true&match=false&name=strain&qualifier=strain&qual_type=Source Qualifier&blanks=ignore&text_separator=;&update_rule=replace\" \
    }, \
    { \
      label str \"Column.3\", \
      data str \"&skipped=true&match=false&name=ProbeDB&qualifier=&qual_type=\" \
    }, \
    { \
      label str \"Column.4\", \
      data str \"&skipped=false&match=false&name=Gene gene_synonym&qualifier=Gene gene_synonym&qual_type=Feature Qualifier&blanks=ignore&text_separator=;&update_rule=add_new_qual\" \
    } \
  } \
}";


const char* sc_TestCGPGene_IDMatch = " \
User-object ::= { \
  type str \"Column Meta Info\",\
  data { \
    { \
      label str \"Column.1\", \
      data str \"&skipped=false&match=true&name=match accession&qualifier=SeqId&qual_type=SeqId&blanks=ignore\" \
    }, \
    { \
      label str \"Column.2\", \
      data str \"&skipped=true&match=false&name=strain&qualifier=strain&qual_type=Source Qualifier&blanks=ignore&text_separator=;&update_rule=replace\" \
    }, \
    { \
      label str \"Column.3\", \
      data str \"&skipped=true&match=false&name=ProbeDB&qualifier=&qual_type=\" \
    }, \
    { \
      label str \"Column.4\", \
      data str \"&skipped=false&match=false&name=gene synonym&qualifier=gene synonym&qual_type=CDS-Gene-Prot Qualifier&blanks=ignore&text_separator=;&update_rule=add_new_qual\" \
    } \
  } \
}";


BOOST_AUTO_TEST_CASE(Test_FeatProtein_BsrcMatch)
{
    CRef<CUser_object> column_prop = ReadUserObj(sc_TestFeatProtein_BsrcMatch);
    CTableReaderMacro tbl_macro;
    CTableReaderMacro::SColumnDelimiters del_options("\t");
    auto macros = tbl_macro.CreateMacros("featqual_table.txt", column_prop, del_options, nullptr, true);
    BOOST_CHECK(macros.size() == 2);

    CheckCreateProtFeatMacro(macros.front());

    string check_macro =
        "MACRO TableReaderMacro \"Description\"\n"
        "VAR\n"
        "table = \"featqual_table.txt\"\n"
        "merge_del = false\n"
        "split_first = false\n"
        "convert_multi = false\n"
        "merge_first = false\n"
        "multiple_spaces_only = false\n"
        "update_mrna = false\n"
        "FOR EACH Protein\n"
        "WHERE INTABLE(BIOSOURCE_FOR_SEQFEAT(\"org.orgname.mod\", \"strain\"), table, 1, \"\\t\", merge_del, split_first, convert_multi, merge_first, true, multiple_spaces_only)\n"
        "DO\n"
        "value = ValueFromTable(table, 2);\n"
        "SetStringQual(\"data.prot.name\", value, \"eReplace\", false);\n"
        "value = ValueFromTable(table, 3);\n"
        "SetRelatedFeatureQual(\"mRNA\", \"comment\", value, \"eReplace\", false, update_mrna);\n"
        "DONE\n"
        "---------------------------------------------------\n";
    BOOST_CHECK(macros[1] == check_macro);

    column_prop = ReadUserObj(sc_TestCGPProtein_BsrcMatch);
    macros = tbl_macro.CreateMacros("featqual_table.txt", column_prop, del_options, nullptr, true);
    BOOST_CHECK(macros.size() == 2);

    CheckCreateProtFeatMacro(macros.front());
    BOOST_CHECK(macros[1] == check_macro);
}

const char* sc_TestFeatProtein_BsrcMatch = " \
User-object ::= { \
  type str \"Column Meta Info\",\
  data { \
    { \
      label str \"Column.1\", \
      data str \"&skipped=false&match=true&name=match strain&qualifier=strain&qual_type=Source Qualifier&blanks=ignore\" \
    }, \
    { \
      label str \"Column.2\", \
      data str \"&skipped=false&match=false&name=CDS product&qualifier=CDS product&qual_type=Feature Qualifier&blanks=ignore&text_separator=;&update_rule=replace\" \
    }, \
    { \
      label str \"Column.3\", \
      data str \"&skipped=false&match=false&name=mRNA note&qualifier=mRNA note&qual_type=Feature Qualifier&blanks=ignore&text_separator=;&update_rule=replace\" \
    } \
  } \
}";

const char* sc_TestCGPProtein_BsrcMatch = " \
User-object ::= { \
  type str \"Column Meta Info\",\
  data { \
    { \
      label str \"Column.1\", \
      data str \"&skipped=false&match=true&name=match strain&qualifier=strain&qual_type=Source Qualifier&blanks=ignore\" \
    }, \
    { \
      label str \"Column.2\", \
      data str \"&skipped=false&match=false&name=protein name&qualifier=protein name&qual_type=CDS-Gene-Prot Qualifier&blanks=ignore&text_separator=;&update_rule=replace\" \
    }, \
    { \
      label str \"Column.3\", \
      data str \"&skipped=false&match=false&name=mRNA comment&qualifier=mRNA comment&qual_type=CDS-Gene-Prot Qualifier&blanks=ignore&text_separator=;&update_rule=replace\" \
    } \
  } \
}";


BOOST_AUTO_TEST_CASE(Test_FeatProtein_DBLinkMatch)
{
    CRef<CUser_object> column_prop = ReadUserObj(sc_TestFeatProtein_DBLinkMatch);
    CTableReaderMacro tbl_macro;
    CTableReaderMacro::SColumnDelimiters del_options("\t");
    auto macros = tbl_macro.CreateMacros("featqual_table.txt", column_prop, del_options, nullptr, true);
    BOOST_CHECK(macros.size() == 2);

    CheckCreateProtFeatMacro(macros.front());

    string check_macro =
        "MACRO TableReaderMacro \"Description\"\n"
        "VAR\n"
        "table = \"featqual_table.txt\"\n"
        "merge_del = false\n"
        "split_first = false\n"
        "convert_multi = false\n"
        "merge_first = false\n"
        "multiple_spaces_only = false\n"
        "update_mrna = false\n"
        "FOR EACH Protein\n"
        "WHERE INTABLE(DBLINKTYPE(\"ProbeDB\"), table, 1, \"\\t\", merge_del, split_first, convert_multi, merge_first, true, multiple_spaces_only)\n"
        "DO\n"
        "value = ValueFromTable(table, 3);\n"
        "SetStringQual(\"data.prot.ec\", value, \"eReplace\", false);\n"
        "DONE\n"
        "---------------------------------------------------\n";
    BOOST_CHECK(macros[1] == check_macro);

    column_prop = ReadUserObj(sc_TestCGPProtein_DBLinkMatch);
    macros = tbl_macro.CreateMacros("featqual_table.txt", column_prop, del_options, nullptr, true);
    BOOST_CHECK(macros.size() == 2);

    CheckCreateProtFeatMacro(macros.front());
    BOOST_CHECK(macros[1] == check_macro);
}

const char* sc_TestFeatProtein_DBLinkMatch = " \
User-object ::= { \
  type str \"Column Meta Info\",\
  data { \
    { \
      label str \"Column.1\", \
      data str \"&skipped=false&match=true&name=match ProbeDB&qualifier=ProbeDB&qual_type=DBLink Field&blanks=ignore\" \
    }, \
    { \
      label str \"Column.2\", \
      data str \"&skipped=true&match=false&name=synonym&qualifier=synonym&qual_type=Source Qualifier&blanks=ignore&text_separator=;&update_rule=replace\" \
    }, \
    { \
      label str \"Column.3\", \
      data str \"&skipped=false&match=false&name=CDS EC_number&qualifier=CDS EC_number&qual_type=Feature Qualifier&blanks=ignore&text_separator=;&update_rule=replace\" \
    } \
  } \
}";

const char* sc_TestCGPProtein_DBLinkMatch = " \
User-object ::= { \
  type str \"Column Meta Info\",\
  data { \
    { \
      label str \"Column.1\", \
      data str \"&skipped=false&match=true&name=match ProbeDB&qualifier=ProbeDB&qual_type=DBLink Field&blanks=ignore\" \
    }, \
    { \
      label str \"Column.2\", \
      data str \"&skipped=true&match=false&name=synonym&qualifier=synonym&qual_type=Source Qualifier&blanks=ignore&text_separator=;&update_rule=replace\" \
    }, \
    { \
      label str \"Column.3\", \
      data str \"&skipped=false&match=false&name=protein EC number&qualifier=protein EC number&qual_type=CDS-Gene-Prot Qualifier&blanks=ignore&text_separator=;&update_rule=replace\" \
    } \
  } \
}";

BOOST_AUTO_TEST_CASE(Test_FeatGB_8268)
{
    CRef<CUser_object> column_prop = ReadUserObj(sc_TestFeat_GB8268);
    CTableReaderMacro tbl_macro;
    CTableReaderMacro::SColumnDelimiters del_options("\t");
    auto macros = tbl_macro.CreateMacros("compound_qual_table.txt", column_prop, del_options, nullptr, true);
    BOOST_CHECK(macros.size() == 1);

    string check_macro =
        "MACRO TableReaderMacro \"Description\"\n"
        "VAR\n"
        "table = \"compound_qual_table.txt\"\n"
        "merge_del = false\n"
        "split_first = false\n"
        "convert_multi = false\n"
        "merge_first = false\n"
        "multiple_spaces_only = false\n"
        "update_mrna = false\n"
        "FOR EACH ImpFeat\n"
        "WHERE data.imp.key = \"mobile_element\" AND INTABLE(SEQID(), table, 1, \"\\t\", merge_del, split_first, convert_multi, merge_first, false, multiple_spaces_only)\n"
        "DO\n"
        "value = ValueFromTable(table, 2);\n"
        "obj = Resolve(\"qual\") WHERE obj.qual = \"mobile_element_type\";\n"
        "SetGBQualifier(obj, \"qual\", \"mobile-element-type-name\", value, \"eReplace\", false);\n"
        "DONE\n"
        "---------------------------------------------------\n";
    BOOST_CHECK(macros.front() == check_macro);
}

const char* sc_TestFeat_GB8268 = " \
User-object ::= { \
  type str \"Column Meta Info\",\
  data { \
    { \
      label str \"Column.1\", \
      data str \"&skipped=false&match=true&name=match Seq-ID&qualifier=SeqId&qual_type=SeqId&blanks=ignore\" \
    }, \
    { \
      label str \"Column.2\", \
      data str \"&skipped=false&match=false&name=mobile_element mobile_element_type_name&qualifier=mobile_element mobile_element_type_name&qual_type=Feature Qualifier&blanks=ignore&text_separator=;&update_rule=replace\" \
    } \
  } \
}";

/* Target = CDS-Gene-Prot
Match_field:
 SeqID - Test_FeatGene_IDMatch
 BioSource - Test_FeatProtein_BsrcMatch
 Molinfo
 Molinfo (inst)
 FeatQual - Test_CDSGeneProtmRNA_CDSMatch, Test_GeneDbXrefMatch
 CDS-Gene-Prot - Test_CdsGeneProt_GeneMatch, Test_CDSGeneProt1_GeneMatch, Test_FeatProtein_GeneMatch, Test_FeatCDS_ProteinMatch
 Test_CDSGeneProtMatPeptide_GeneMatch
 RNA
 Misc
 Pub
 StrComm
 DBlink - Test_FeatProtein_DBLinkMatch
 */


BOOST_AUTO_TEST_CASE(Test_CdsGeneProt_GeneMatch)
{
    CRef<CUser_object> column_prop = ReadUserObj(sc_TestCGProt_GeneMatch);
    CTableReaderMacro tbl_macro;
    CTableReaderMacro::SColumnDelimiters del_options("\t");
    auto macros = tbl_macro.CreateMacros("applygenetable.txt", column_prop, del_options, nullptr, true);
    BOOST_CHECK(macros.size() == 1);

    string check_macro =
        "MACRO TableReaderMacro \"Description\"\n"
        "VAR\n"
        "table = \"applygenetable.txt\"\n"
        "merge_del = false\n"
        "split_first = false\n"
        "convert_multi = false\n"
        "merge_first = false\n"
        "multiple_spaces_only = false\n"
        "update_mrna = false\n"
        "FOR EACH Gene\n"
        "WHERE INTABLE(\"data.gene.locus-tag\", table, 2, \"\\t\", merge_del, split_first, convert_multi, merge_first, true, multiple_spaces_only)\n"
        "DO\n"
        "value = ValueFromTable(table, 3);\n"
        "SetStringQual(\"data.gene.desc\", value, \"eReplace\", false);\n"
        "DONE\n"
        "---------------------------------------------------\n";
    BOOST_CHECK(macros.front() == check_macro);
}

const char* sc_TestCGProt_GeneMatch = " \
User-object ::= { \
  type str \"Column Meta Info\",\
  data { \
    { \
      label str \"Column.1\", \
      data str \"&skipped=true&match=false&name=Col 1&qualifier=SeqId&qual_type=SeqId&blanks=ignore&text_separator=;&update_rule=replace\" \
    }, \
    { \
      label str \"Column.2\", \
      data str \"&skipped=false&match=true&name=match gene locus tag&qualifier=gene locus tag&qual_type=CDS-Gene-Prot Qualifier&blanks=ignore\" \
    }, \
    { \
      label str \"Column.3\", \
      data str \"&skipped=false&match=false&name=gene description&qualifier=gene description&qual_type=CDS-Gene-Prot Qualifier&blanks=ignore&text_separator=;&update_rule=replace\" \
    } \
  } \
}";


BOOST_AUTO_TEST_CASE(Test_CDSGeneProt1_GeneMatch)
{
    CRef<CUser_object> column_prop = ReadUserObj(sc_TestCGProt1_GeneMatch);
    CTableReaderMacro tbl_macro;
    CTableReaderMacro::SColumnDelimiters del_options("\t");
    auto macros = tbl_macro.CreateMacros("applygenetable.txt", column_prop, del_options, nullptr, true);
    BOOST_CHECK(macros.size() == 1);

    string check_macro =
        "MACRO TableReaderMacro \"Description\"\n"
        "VAR\n"
        "table = \"applygenetable.txt\"\n"
        "merge_del = false\n"
        "split_first = false\n"
        "convert_multi = false\n"
        "merge_first = false\n"
        "multiple_spaces_only = false\n"
        "update_mrna = false\n"
        "FOR EACH Gene\n"
        "WHERE INTABLE(\"data.gene.locus-tag\", table, 2, \"\\t\", merge_del, split_first, convert_multi, merge_first, true, multiple_spaces_only)\n"
        "DO\n"
        "value = ValueFromTable(table, 4);\n"
        "SetStringQual(\"data.gene.locus\", value, \"eReplace\", false);\n"
        "value = ValueFromTable(table, 6);\n"
        "SetRelatedFeatureQual(\"protein\", \"data.prot.name\", value, \"eReplace\", false, update_mrna);\n"
        "value = ValueFromTable(table, 8);\n"
        "SetRelatedFeatureQual(\"CDS\", \"comment\", value, \"eReplace\", false, update_mrna);\n"
        "DONE\n"
        "---------------------------------------------------\n";
    BOOST_CHECK(macros.front() == check_macro);
}

const char* sc_TestCGProt1_GeneMatch = " \
User-object ::= { \
  type str \"Column Meta Info\",\
  data { \
    { \
      label str \"Column.1\", \
      data str \"&skipped=true&match=false&name=Col 1&qualifier=SeqId&qual_type=SeqId&blanks=ignore&text_separator=;&update_rule=replace\" \
    }, \
    { \
      label str \"Column.2\", \
      data str \"&skipped=false&match=true&name=match gene locus tag&qualifier=gene locus tag&qual_type=CDS-Gene-Prot Qualifier&blanks=ignore\" \
    }, \
    { \
      label str \"Column.3\", \
      data str \"&skipped=true&match=false&name=Col 3&qualifier=&qual_type=\" \
    }, \
    { \
      label str \"Column.4\", \
      data str \"&skipped=false&match=false&name=gene locus&qualifier=gene locus&qual_type=CDS-Gene-Prot Qualifier&blanks=ignore&text_separator=;&update_rule=replace\" \
    }, \
    { \
      label str \"Column.5\", \
      data str \"&skipped=true&match=false&name=Col 5&qualifier=&qual_type=\" \
    }, \
    { \
      label str \"Column.6\", \
      data str \"&skipped=false&match=false&name=protein name&qualifier=protein name&qual_type=CDS-Gene-Prot Qualifier&blanks=ignore&text_separator=;&update_rule=replace\" \
    }, \
    { \
      label str \"Column.7\", \
      data str \"&skipped=true&match=false&name=Col 7&qualifier=&qual_type=\" \
    }, \
    { \
      label str \"Column.8\", \
      data str \"&skipped=false&match=false&name=CDS comment&qualifier=CDS comment&qual_type=CDS-Gene-Prot Qualifier&blanks=ignore&text_separator=;&update_rule=replace\" \
    } \
  } \
}";


BOOST_AUTO_TEST_CASE(Test_CDSGeneProtmRNA_CDSMatch)
{
    CRef<CUser_object> column_prop = ReadUserObj(sc_TestCGPmRNA_CDSMatch);
    CTableReaderMacro tbl_macro;
    CTableReaderMacro::SColumnDelimiters del_options("\t");
    auto macros = tbl_macro.CreateMacros("applygenetable.txt", column_prop, del_options, nullptr, true);
    BOOST_CHECK(macros.size() == 1);

    string check_macro =
        "MACRO TableReaderMacro \"Description\"\n"
        "VAR\n"
        "table = \"applygenetable.txt\"\n"
        "merge_del = false\n"
        "split_first = false\n"
        "convert_multi = false\n"
        "merge_first = false\n"
        "multiple_spaces_only = false\n"
        "update_mrna = false\n"
        "FOR EACH mRNA\n"
        "WHERE INTABLE(RELATED_FEATURE(\"cds\", \"qual\", \"inference\"), table, 1, \"\\t\", merge_del, split_first, convert_multi, merge_first, true, multiple_spaces_only)\n"
        "DO\n"
        "value = ValueFromTable(table, 2);\n"
        "SetStringQual(\"data.rna.ext.name\", value, \"eReplace\", false);\n"
        "DONE\n"
        "---------------------------------------------------\n";
    BOOST_CHECK(macros.front() == check_macro);
}

const char* sc_TestCGPmRNA_CDSMatch = " \
User-object ::= { \
  type str \"Column Meta Info\",\
  data { \
    { \
      label str \"Column.1\", \
      data str \"&skipped=false&match=true&name=match CDS inference&qualifier=CDS inference&qual_type=Feature Qualifier&blanks=ignore\" \
    }, \
    { \
      label str \"Column.2\", \
      data str \"&skipped=false&match=false&name=mRNA product&qualifier=mRNA product&qual_type=CDS-Gene-Prot Qualifier&blanks=ignore&text_separator=;&update_rule=replace\" \
    } \
  } \
}";

BOOST_AUTO_TEST_CASE(Test_CDSGeneProtMatPeptide_GeneMatch)
{
    CRef<CUser_object> column_prop = ReadUserObj(sc_TestCGPMatPeptide_GeneMatch);
    CTableReaderMacro tbl_macro;
    CTableReaderMacro::SColumnDelimiters del_options("\t");
    auto macros = tbl_macro.CreateMacros("featqual_table.txt", column_prop, del_options, nullptr, true);
    BOOST_CHECK(macros.size() == 1);

    string check_macro =
        "MACRO TableReaderMacro \"Description\"\n"
        "VAR\n"
        "table = \"featqual_table.txt\"\n"
        "merge_del = false\n"
        "split_first = false\n"
        "convert_multi = false\n"
        "merge_first = false\n"
        "multiple_spaces_only = false\n"
        "update_mrna = false\n"
        "FOR EACH SeqFeat\n"
        "WHERE CHOICETYPE(\"data\") = \"prot\" AND data.prot.processed = \"mature\""
        " AND INTABLE(RELATED_FEATURE(\"gene\", \"data.gene.syn\"), table, 1, \"\\t\", merge_del, split_first, convert_multi, merge_first, true, multiple_spaces_only)\n"
        "DO\n"
        "value = ValueFromTable(table, 2);\n"
        "SetStringQual(\"data.prot.name\", value, \"ePrepend\", \",\", false);\n"
        "DONE\n"
        "---------------------------------------------------\n";
    BOOST_CHECK(macros.front() == check_macro);
}

const char* sc_TestCGPMatPeptide_GeneMatch = " \
User-object ::= { \
  type str \"Column Meta Info\",\
  data { \
    { \
      label str \"Column.1\", \
      data str \"&skipped=false&match=true&name=match gene synonym&qualifier=gene synonym&qual_type=CDS-Gene-Prot Qualifier&blanks=ignore\" \
    }, \
    { \
      label str \"Column.2\", \
      data str \"&skipped=false&match=false&name=mat_peptide name&qualifier=mat_peptide name&qual_type=CDS-Gene-Prot Qualifier&blanks=ignore&text_separator=,&update_rule=prefix\" \
    } \
  } \
}";

/* Target = RNA 
Match_field:
 SeqID - Test_RNA_SeqIdMatch, Test_ncRNA_SeqIDMatch
 BioSource - Test_rRNA_BsrcMatch
 Molinfo
 Molinfo (inst)
 FeatQual
 CDS-Gene-Prot - Test_rRNA_CdsGeneProtMatch
 RNA - Test_mRNA_GeneMatch
 Misc
 Pub
 StrComm
 DBlink
 */

BOOST_AUTO_TEST_CASE(Test_RNA_SeqIdMatch)
{
    CRef<CUser_object> column_prop = ReadUserObj(sc_TestRNA_SeqIDMatch);
    CTableReaderMacro tbl_macro;
    CTableReaderMacro::SColumnDelimiters del_options("\t");
    auto macros = tbl_macro.CreateMacros("featqual_table.txt", column_prop, del_options, nullptr, true);
    BOOST_CHECK(macros.size() == 1);

    string check_macro =
        "MACRO TableReaderMacro \"Description\"\n"
        "VAR\n"
        "table = \"featqual_table.txt\"\n"
        "merge_del = false\n"
        "split_first = false\n"
        "convert_multi = false\n"
        "merge_first = false\n"
        "multiple_spaces_only = false\n"
        "FOR EACH RNA\n"
        "WHERE INTABLE(SEQID(), table, 1, \"\\t\", merge_del, split_first, convert_multi, merge_first, false, multiple_spaces_only)\n"
        "DO\n"
        "value = ValueFromTable(table, 2);\n"
        "SetRnaProduct(value, \"eReplace\", true);\n"
        "DONE\n"
        "---------------------------------------------------\n";
    BOOST_CHECK(macros.front() == check_macro);
}

const char* sc_TestRNA_SeqIDMatch = " \
User-object ::= { \
  type str \"Column Meta Info\",\
  data { \
    { \
      label str \"Column.1\", \
      data str \"&skipped=false&match=true&name=match Seq-ID&qualifier=SeqId&qual_type=SeqId&blanks=ignore\" \
    }, \
    { \
      label str \"Column.2\", \
      data str \"&skipped=false&match=false&name=RNA product&qualifier=RNA product&qual_type=RNA Qualifier&blanks=erase&text_separator=;&update_rule=replace\" \
    } \
  } \
}";

BOOST_AUTO_TEST_CASE(Test_mRNA_GeneMatch)
{
    CRef<CUser_object> column_prop = ReadUserObj(sc_TestmRNA_GeneMatch);
    CTableReaderMacro tbl_macro;
    CTableReaderMacro::SColumnDelimiters del_options("\t");
    auto macros = tbl_macro.CreateMacros("applyrnatable.txt", column_prop, del_options, nullptr, true);
    BOOST_CHECK(macros.size() == 1);

    string check_macro =
        "MACRO TableReaderMacro \"Description\"\n"
        "VAR\n"
        "table = \"applyrnatable.txt\"\n"
        "merge_del = false\n"
        "split_first = false\n"
        "convert_multi = false\n"
        "merge_first = false\n"
        "multiple_spaces_only = false\n"
        "FOR EACH mRNA\n"
        "WHERE INTABLE(RELATED_FEATURE(\"gene\", \"data.gene.locus-tag\"), table, 2, \"\\t\", merge_del, split_first, convert_multi, merge_first, true, multiple_spaces_only)\n"
        "DO\n"
        "value = ValueFromTable(table, 3);\n"
        "SetRelatedFeatureQual(\"gene\", \"data.gene.locus\", value, \"eReplace\", false, false);\n"
        "value = ValueFromTable(table, 4);\n"
        "SetStringQual(\"data.rna.ext.name\", value, \"eReplace\", false);\n"
        "DONE\n"
        "---------------------------------------------------\n";
    BOOST_CHECK(macros.front() == check_macro);
}

const char* sc_TestmRNA_GeneMatch = " \
User-object ::= { \
  type str \"Column Meta Info\",\
  data { \
    { \
      label str \"Column.1\", \
      data str \"&skipped=true&match=false&name=accession&qualifier=SeqId&qual_type=SeqId&blanks=ignore&text_separator=;&update_rule=replace\" \
    }, \
    { \
      label str \"Column.2\", \
      data str \"&skipped=false&match=true&name=match mRNA gene locus tag&qualifier=mRNA gene locus tag&qual_type=RNA Qualifier&blanks=ignore\" \
    }, \
    { \
      label str \"Column.3\", \
      data str \"&skipped=false&match=false&name=mRNA gene locus&qualifier=mRNA gene locus&qual_type=RNA Qualifier&blanks=ignore&text_separator=;&update_rule=replace\" \
    }, \
    { \
      label str \"Column.4\", \
      data str \"&skipped=false&match=false&name=mRNA product&qualifier=mRNA product&qual_type=RNA Qualifier&blanks=ignore&text_separator=;&update_rule=replace\" \
    } \
  } \
}";

BOOST_AUTO_TEST_CASE(Test_ncRNA_SeqIDMatch)
{
    CRef<CUser_object> column_prop = ReadUserObj(sc_TestncRNA_SeqIDMatch);
    CTableReaderMacro tbl_macro;
    CTableReaderMacro::SColumnDelimiters del_options("\t");
    auto macros = tbl_macro.CreateMacros("applyrnatable.txt", column_prop, del_options, nullptr, true);
    BOOST_CHECK(macros.size() == 1);

    string check_macro =
        "MACRO TableReaderMacro \"Description\"\n"
        "VAR\n"
        "table = \"applyrnatable.txt\"\n"
        "merge_del = false\n"
        "split_first = false\n"
        "convert_multi = false\n"
        "merge_first = false\n"
        "multiple_spaces_only = false\n"
        "FOR EACH RNA\n"
        "WHERE data.rna.type = \"ncRNA\" AND data.rna.ext.gen.class = \"miRNA\" AND INTABLE(SEQID(), table, 1, \"\\t\", merge_del, split_first, convert_multi, merge_first, false, multiple_spaces_only)\n"
        "DO\n"
        "value = ValueFromTable(table, 2);\n"
        "SetStringQual(\"comment\", value, \"eReplace\", false);\n"
        "DONE\n"
        "---------------------------------------------------\n";
    BOOST_CHECK(macros.front() == check_macro);
}

const char* sc_TestncRNA_SeqIDMatch = " \
User-object ::= { \
  type str \"Column Meta Info\",\
  data { \
    { \
      label str \"Column.1\", \
      data str \"&skipped=false&match=true&name=match Seq-ID&qualifier=SeqId&qual_type=SeqId&blanks=ignore\" \
    }, \
    { \
      label str \"Column.2\", \
      data str \"&skipped=false&match=false&name=ncRNA miRNA comment&qualifier=ncRNA miRNA comment&qual_type=RNA Qualifier&blanks=ignore&text_separator=;&update_rule=replace\" \
    } \
  } \
}";

BOOST_AUTO_TEST_CASE(Test_rRNA_BsrcMatch)
{
    CRef<CUser_object> column_prop = ReadUserObj(sc_TestrRNA_BsrcMatch);
    CTableReaderMacro tbl_macro;
    CTableReaderMacro::SColumnDelimiters del_options("\t");
    auto macros = tbl_macro.CreateMacros("featqual_table.txt", column_prop, del_options, nullptr, true);
    BOOST_CHECK(macros.size() == 1);

    string check_macro =
        "MACRO TableReaderMacro \"Description\"\n"
        "VAR\n"
        "table = \"featqual_table.txt\"\n"
        "merge_del = false\n"
        "split_first = false\n"
        "convert_multi = false\n"
        "merge_first = false\n"
        "multiple_spaces_only = false\n"
        "FOR EACH rRNA\n"
        "WHERE INTABLE(BIOSOURCE_FOR_SEQFEAT(\"org.orgname.mod\", \"strain\"), table, 2, \"\\t\", merge_del, split_first, convert_multi, merge_first, true, multiple_spaces_only)\n"
        "DO\n"
        "value = ValueFromTable(table, 3);\n"
        "obj = Resolve(\"qual\") WHERE obj.qual = \"tag_peptide\";\n"
        "SetGBQualifier(obj, \"qual\", \"tag_peptide\", value, \"eReplace\", false);\n"
        "DONE\n"
        "---------------------------------------------------\n";
    BOOST_CHECK(macros.front() == check_macro);
}

const char* sc_TestrRNA_BsrcMatch = " \
User-object ::= { \
  type str \"Column Meta Info\",\
  data { \
    { \
      label str \"Column.1\", \
      data str \"&skipped=true&match=false&name=accession&qualifier=SeqId&qual_type=SeqId&blanks=ignore&text_separator=;&update_rule=replace\" \
    }, \
    { \
      label str \"Column.2\", \
      data str \"&skipped=false&match=true&name=match strain&qualifier=strain&qual_type=Source Qualifier&blanks=ignore\" \
    }, \
    { \
      label str \"Column.3\", \
      data str \"&skipped=false&match=false&name=rRNA tag-peptide&qualifier=rRNA tag-peptide&qual_type=RNA Qualifier&blanks=ignore&text_separator=;&update_rule=replace\" \
    } \
  } \
}";

BOOST_AUTO_TEST_CASE(Test_rRNA_CdsGeneProtMatch)
{
    CRef<CUser_object> column_prop = ReadUserObj(sc_TestrRNA_GeneLocusTagMatch);
    CTableReaderMacro tbl_macro;
    CTableReaderMacro::SColumnDelimiters del_options("\t");
    auto macros = tbl_macro.CreateMacros("applyrnatable.txt", column_prop, del_options, nullptr, true);
    BOOST_CHECK(macros.size() == 1);

    string check_macro =
        "MACRO TableReaderMacro \"Description\"\n"
        "VAR\n"
        "table = \"applyrnatable.txt\"\n"
        "merge_del = false\n"
        "split_first = false\n"
        "convert_multi = false\n"
        "merge_first = false\n"
        "multiple_spaces_only = false\n"
        "FOR EACH rRNA\n"
        "WHERE INTABLE(RELATED_FEATURE(\"gene\", \"data.gene.locus-tag\"), table, 2, \"\\t\", merge_del, split_first, convert_multi, merge_first, true, multiple_spaces_only)\n"
        "DO\n"
        "value = ValueFromTable(table, 3);\n"
        "SetStringQual(\"data.rna.ext.name\", value, \"eAppend\", \";\", false);\n"
        "DONE\n"
        "---------------------------------------------------\n";
    BOOST_CHECK(macros.front() == check_macro);
}

const char* sc_TestrRNA_GeneLocusTagMatch = " \
User-object ::= { \
  type str \"Column Meta Info\",\
  data { \
    { \
      label str \"Column.1\", \
      data str \"&skipped=true&match=false&name=accession&qualifier=SeqId&qual_type=SeqId&blanks=ignore&text_separator=;&update_rule=replace\" \
    }, \
    { \
      label str \"Column.2\", \
      data str \"&skipped=false&match=true&name=match gene locus tag&qualifier=gene locus tag&qual_type=CDS-Gene-Prot Qualifier&blanks=ignore\" \
    }, \
    { \
      label str \"Column.3\", \
      data str \"&skipped=false&match=false&name=rRNA product&qualifier=rRNA product&qual_type=RNA Qualifier&blanks=ignore&text_separator=;&update_rule=append\" \
    } \
  } \
}";

/////////////////////////////
// Mixed tables
// Test_BsrcAndStrucComment
// Test_BsrcAndrRNA
// Test_PmidMatch_BsrcDBlinkPubFeat

BOOST_AUTO_TEST_CASE(Test_BsrcAndStrucComment)
{
    CRef<CUser_object> column_prop = ReadUserObj(sc_TestBsrc_StructuredComment);
    CTableReaderMacro tbl_macro;
    CTableReaderMacro::SColumnDelimiters del_options("\t");
    auto macros = tbl_macro.CreateMacros("CloneIsosrcExtendedTbl.txt", column_prop, del_options, nullptr, true);
    BOOST_CHECK(macros.size() == 2);

    string first_macro =
        "MACRO TableReaderMacro \"Description\"\n"
        "VAR\n"
        "table = \"CloneIsosrcExtendedTbl.txt\"\n"
        "merge_del = false\n"
        "split_first = false\n"
        "convert_multi = false\n"
        "merge_first = false\n"
        "multiple_spaces_only = false\n"
        "FOR EACH BioSource\n"
        "WHERE INTABLE(\"subtype\", \"clone\", table, 1, \"\\t\", merge_del, split_first, convert_multi, merge_first, true, multiple_spaces_only)\n"
        "DO\n"
        "value = ValueFromTable(table, 2);\n"
        "obj = Resolve(\"subtype\") WHERE obj.subtype = \"isolation-source\";\n"
        "SetModifier(obj, \"subtype\", \"isolation-source\", value, \"eReplace\", false);\n"
        "DONE\n"
        "---------------------------------------------------\n";
    BOOST_CHECK(macros[0] == first_macro);

    string second_macro = "MACRO TableReaderMacro \"Description\"\n"
        "VAR\n"
        "table = \"CloneIsosrcExtendedTbl.txt\"\n"
        "merge_del = false\n"
        "split_first = false\n"
        "convert_multi = false\n"
        "merge_first = false\n"
        "multiple_spaces_only = false\n"
        "FOR EACH StructComment\n"
        "WHERE INTABLE(BIOSOURCE_FOR_SEQDESC(\"subtype\", \"clone\"), table, 1, \"\\t\", merge_del, split_first, convert_multi, merge_first, true, multiple_spaces_only)\n"
        "DO\n"
        "value = ValueFromTable(table, 3);\n"
        "SetStructCommField(\"Sequencing Technology\", value, \"eReplace\", false);\n"
        "DONE\n"
        "---------------------------------------------------\n";
    BOOST_CHECK(macros[1] == second_macro);
}

const char* sc_TestBsrc_StructuredComment = " \
User-object ::= { \
  type str \"Column Meta Info\",\
  data { \
    { \
      label str \"Column.1\", \
      data str \"&skipped=false&match=true&name=match clone&qualifier=clone&qual_type=Source Qualifier&blanks=ignore&update_rule=add_new_qual\" \
    }, \
    { \
      label str \"Column.2\", \
      data str \"&skipped=false&match=false&name=isolation-source&qualifier=isolation-source&qual_type=Source Qualifier&blanks=ignore&text_separator=;&update_rule=replace\" \
    }, \
    { \
      label str \"Column.3\", \
      data str \"&skipped=false&match=false&name=Structured comment Field Sequencing Technology&qualifier=Structured comment Field Sequencing Technology&qual_type=Structured Comment&blanks=ignore&text_separator=;&update_rule=replace\" \
    } \
  } \
}";

BOOST_AUTO_TEST_CASE(Test_BsrcAndrRNA)
{
    CRef<CUser_object> column_prop = ReadUserObj(sc_TestBsrc_rRNAComment);
    CTableReaderMacro tbl_macro;
    CTableReaderMacro::SColumnDelimiters del_options("\t");
    del_options.m_Merge_first_two_cols = true;
    auto macros = tbl_macro.CreateMacros("withrRNAcomment.txt", column_prop, del_options, nullptr, true);
    BOOST_CHECK(macros.size() == 2);

    string first_macro =
        "MACRO TableReaderMacro \"Description\"\n"
        "VAR\n"
        "table = \"withrRNAcomment.txt\"\n"
        "merge_del = false\n"
        "split_first = false\n"
        "convert_multi = false\n"
        "merge_first = true\n"
        "multiple_spaces_only = false\n"
        "FOR EACH BioSource\n"
        "WHERE INTABLE(SEQID(), table, 1, \"\\t\", merge_del, split_first, convert_multi, merge_first, false, multiple_spaces_only)\n"
        "DO\n"
        "value = ValueFromTable(table, 2);\n"
        "obj = Resolve(\"org.orgname.mod\") WHERE obj.subtype = \"strain\";\n"
        "SetModifier(obj, \"org.orgname.mod\", \"strain\", value, \"eReplace\", false);\n"
        "DONE\n"
        "---------------------------------------------------\n";
    BOOST_CHECK(macros[0] == first_macro);

    string second_macro = "MACRO TableReaderMacro \"Description\"\n"
        "VAR\n"
        "table = \"withrRNAcomment.txt\"\n"
        "merge_del = false\n"
        "split_first = false\n"
        "convert_multi = false\n"
        "merge_first = true\n"
        "multiple_spaces_only = false\n"
        "FOR EACH rRNA\n"
        "WHERE INTABLE(SEQID(), table, 1, \"\\t\", merge_del, split_first, convert_multi, merge_first, false, multiple_spaces_only)\n"
        "DO\n"
        "value = ValueFromTable(table, 3);\n"
        "SetStringQual(\"comment\", value, \"eReplace\", false);\n"
        "DONE\n"
        "---------------------------------------------------\n";
    BOOST_CHECK(macros[1] == second_macro);
}

const char* sc_TestBsrc_rRNAComment = " \
User-object ::= { \
  type str \"Column Meta Info\",\
  data { \
    { \
      label str \"Column.1\", \
      data str \"&skipped=false&match=true&name=match Seq-ID&qualifier=SeqId&qual_type=SeqId&blanks=ignore\" \
    }, \
    { \
      label str \"Column.2\", \
      data str \"&skipped=false&match=false&name=strain&qualifier=strain&qual_type=Source Qualifier&blanks=ignore&text_separator=;&update_rule=replace\" \
    }, \
    { \
      label str \"Column.3\", \
      data str \"&skipped=false&match=false&name=rRNA comment&qualifier=rRNA comment&qual_type=RNA Qualifier&blanks=ignore&text_separator=;&update_rule=replace\" \
    } \
  } \
}";


BOOST_AUTO_TEST_CASE(Test_GeneDbXrefMatch)
{
    CRef<CUser_object> column_prop = ReadUserObj(sc_TestCGP_GeneDbxrefMatch);
    CTableReaderMacro tbl_macro;
    CTableReaderMacro::SColumnDelimiters del_options("\t");
    auto macros = tbl_macro.CreateMacros("genedbxref.txt", column_prop, del_options, nullptr, true);
    BOOST_CHECK(macros.size() == 1);

    string first_macro =
        "MACRO TableReaderMacro \"Description\"\n"
        "VAR\n"
        "table = \"genedbxref.txt\"\n"
        "merge_del = false\n"
        "split_first = false\n"
        "convert_multi = false\n"
        "merge_first = false\n"
        "multiple_spaces_only = false\n"
        "update_mrna = false\n"
        "FOR EACH Gene\n"
        "WHERE INTABLE(GETDBXREF(), table, 1, \"\\t\", merge_del, split_first, convert_multi, merge_first, true, multiple_spaces_only)\n"
        "DO\n"
        "value = ValueFromTable(table, 2);\n"
        "SetStringQual(\"data.gene.locus\", value, \"eReplace\", false);\n"
        "value = ValueFromTable(table, 3);\n"
        "SetRelatedFeatureQual(\"protein\", \"data.prot.name\", value, \"eReplace\", false, update_mrna);\n"
        "DONE\n"
        "---------------------------------------------------\n";
    BOOST_CHECK(macros[0] == first_macro);
}

const char* sc_TestCGP_GeneDbxrefMatch = " \
User-object ::= { \
  type str \"Column Meta Info\",\
  data { \
    { \
      label str \"Column.1\", \
      data str \"&skipped=false&match=true&name=match Gene db_xref&qualifier=Gene db_xref&qual_type=Feature Qualifier&blanks=ignore\" \
    }, \
    { \
      label str \"Column.2\", \
      data str \"&skipped=false&match=false&name=gene locus&qualifier=gene locus&qual_type=CDS-Gene-Prot Qualifier&blanks=ignore&text_separator=;&update_rule=replace\" \
    }, \
    { \
      label str \"Column.3\", \
      data str \"&skipped=false&match=false&name=protein name&qualifier=protein name&qual_type=CDS-Gene-Prot Qualifier&blanks=ignore&text_separator=;&update_rule=replace\" \
    } \
  } \
}";


BOOST_AUTO_TEST_CASE(Test_GeneSetDbXref)
{
    CRef<CUser_object> column_prop = ReadUserObj(sc_TestGeneDbxref_GeneMatch);
    CTableReaderMacro tbl_macro;
    CTableReaderMacro::SColumnDelimiters del_options("\t");
    auto macros = tbl_macro.CreateMacros("apply_dbxref.txt", column_prop, del_options, nullptr, true);
    BOOST_CHECK(macros.size() == 1);

    string first_macro =
        "MACRO TableReaderMacro \"Description\"\n"
        "VAR\n"
        "table = \"apply_dbxref.txt\"\n"
        "merge_del = false\n"
        "split_first = false\n"
        "convert_multi = false\n"
        "merge_first = false\n"
        "multiple_spaces_only = false\n"
        "update_mrna = false\n"
        "FOR EACH Gene\n"
        "WHERE INTABLE(\"comment\", table, 1, \"\\t\", merge_del, split_first, convert_multi, merge_first, true, multiple_spaces_only)\n"
        "DO\n"
        "value = ValueFromTable(table, 2);\n"
        "SetStringQual(\"dbxref\", value, \"eReplace\", false);\n"
        "DONE\n"
        "---------------------------------------------------\n";
    BOOST_CHECK(macros[0] == first_macro);
}

const char* sc_TestGeneDbxref_GeneMatch = " \
User-object ::= { \
  type str \"Column Meta Info\",\
  data { \
    { \
      label str \"Column.1\", \
      data str \"&skipped=false&match=true&name=match gene comment&qualifier=gene comment&qual_type=CDS-Gene-Prot Qualifier&blanks=ignore\" \
    }, \
    { \
      label str \"Column.2\", \
      data str \"&skipped=false&match=false&name=Gene db_xref&qualifier=Gene db_xref&qual_type=Feature Qualifier&blanks=ignore&text_separator=;&update_rule=replace\" \
    } \
  } \
}";


BOOST_AUTO_TEST_CASE(Test_PmidMatch_BsrcDBlinkPubFeat)
{
    CRef<CUser_object> column_prop = ReadUserObj(sc_TestMany_PmidMatch);
    CTableReaderMacro tbl_macro;
    CTableReaderMacro::SColumnDelimiters del_options("\t");
    auto macros = tbl_macro.CreateMacros("pmid_names.txt", column_prop, del_options, nullptr, true);
    BOOST_CHECK(macros.size() == 4);

    string first_macro =
        "MACRO TableReaderMacro \"Description\"\n"
        "VAR\n"
        "table = \"pmid_names.txt\"\n"
        "merge_del = false\n"
        "split_first = false\n"
        "convert_multi = false\n"
        "merge_first = false\n"
        "multiple_spaces_only = false\n"
        "FOR EACH Pubdesc\n"
        "WHERE INTABLE(PUB_PMID(), table, 1, \"\\t\", merge_del, split_first, convert_multi, merge_first, true, multiple_spaces_only)\n"
        "DO\n"
        "value = ValueFromTable(table, 2);\n"
        "AddAuthorList(value, false);\n"
        "DONE\n"
        "---------------------------------------------------\n";
    BOOST_CHECK(macros[0] == first_macro);

    string second_macro =
        "MACRO TableReaderMacro \"Description\"\n"
        "VAR\n"
        "table = \"pmid_names.txt\"\n"
        "merge_del = false\n"
        "split_first = false\n"
        "convert_multi = false\n"
        "merge_first = false\n"
        "multiple_spaces_only = false\n"
        "FOR EACH BioSource\n"
        "WHERE INTABLE(PUB_PMID(), table, 1, \"\\t\", merge_del, split_first, convert_multi, merge_first, true, multiple_spaces_only)\n"
        "DO\n"
        "value = ValueFromTable(table, 3);\n"
        "obj = Resolve(\"subtype\") WHERE obj.subtype = \"country\";\n"
        "SetModifier(obj, \"subtype\", \"country\", value, \"eReplace\", false);\n"
        "DONE\n"
        "---------------------------------------------------\n";
    BOOST_CHECK(macros[1] == second_macro);

    string third_macro =
        "MACRO TableReaderMacro \"Description\"\n"
        "VAR\n"
        "table = \"pmid_names.txt\"\n"
        "merge_del = false\n"
        "split_first = false\n"
        "convert_multi = false\n"
        "merge_first = false\n"
        "multiple_spaces_only = false\n"
        "FOR EACH SeqNA\n"
        "WHERE INTABLE(PUB_PMID(), table, 1, \"\\t\", merge_del, split_first, convert_multi, merge_first, true, multiple_spaces_only)\n"
        "DO\n"
        "value = ValueFromTable(table, 4);\n"
        "AddDBLink(\"BioSample\", value, \"eReplace\", false);\n"
        "DONE\n"
        "---------------------------------------------------\n";
    
    BOOST_CHECK(macros[2] == third_macro);

    string fourth_macro =
        "MACRO TableReaderMacro \"Description\"\n"
        "VAR\n"
        "table = \"pmid_names.txt\"\n"
        "merge_del = false\n"
        "split_first = false\n"
        "convert_multi = false\n"
        "merge_first = false\n"
        "multiple_spaces_only = false\n"
        "FOR EACH RNA\n"
        "WHERE data.rna.type = \"tRNA\" AND INTABLE(PUB_PMID(), table, 1, \"\\t\", merge_del, split_first, convert_multi, merge_first, true, multiple_spaces_only)\n"
        "DO\n"
        "value = ValueFromTable(table, 5);\n"
        "SetStringQual(\"comment\", value, \"eReplace\", false);\n"
        "DONE\n"
        "---------------------------------------------------\n";

    BOOST_CHECK(macros[3] == fourth_macro);
}

const char* sc_TestMany_PmidMatch = " \
User-object ::= { \
  type str \"Column Meta Info\",\
  data { \
    { \
      label str \"Column.1\", \
      data str \"&skipped=false&match=true&name=match pmid&qualifier=pmid&qual_type=Pub Qualifier&blanks=ignore\" \
    }, \
    { \
      label str \"Column.2\", \
      data str \"&skipped=false&match=false&name=author name list&qualifier=author name list&qual_type=Pub Qualifier&blanks=ignore&text_separator=;&update_rule=replace\" \
    }, \
    { \
      label str \"Column.3\", \
      data str \"&skipped=false&match=false&name=country&qualifier=country&qual_type=Source Qualifier&blanks=ignore&text_separator=;&update_rule=replace\" \
    }, \
    { \
      label str \"Column.4\", \
      data str \"&skipped=false&match=false&name=BioSample&qualifier=BioSample&qual_type=DBLink Field&blanks=ignore&text_separator=;&update_rule=replace\" \
    }, \
    { \
      label str \"Column.5\", \
      data str \"&skipped=false&match=false&name=tRNA comment&qualifier=tRNA comment&qual_type=RNA Qualifier&blanks=ignore&text_separator=;&update_rule=replace\" \
    } \
  } \
}";

