/*  
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data, the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties, express or implied, including
 *  warranties of performance, merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 * Author: Andrei Shkeda
 *
 * File Description:
 *
 * ===========================================================================
 */

#include <ncbi_pch.hpp>
#include <corelib/test_boost.hpp>
#include <corelib/ncbifile.hpp>
#include <gui/utils/data_frame.hpp>

BOOST_AUTO_TEST_SUITE(TestSuiteDataFrame)

BOOST_AUTO_TEST_CASE(Test_Comparison)
{
    CDataFrame<255> df({eDF_String, eDF_Uint, eDF_Bit});
    df.String(0).set(0, "1");
    df.Uint(1).set(0, 1);
    df.Bit(2).set(0);
    
    {
        // different str
        CDataFrame<255> df1({eDF_String, eDF_Uint, eDF_Bit});
        df1.String(0).set(0, "2");
        df1.Uint(1).set(0, 1);
        df1.Bit(2).set(0);
        BOOST_CHECK(df.Equal(df1) == false);
    }
    {
        // different uint
        CDataFrame<255> df1({eDF_String, eDF_Uint, eDF_Bit});
        df1.String(0).set(0, "1");
        df1.Uint(1).set(0, 2);
        df1.Bit(2).set(0);
        BOOST_CHECK(df.Equal(df1) == false);
    }

    {
        // different bit
        CDataFrame<255> df1({eDF_String, eDF_Uint, eDF_Bit});
        df1.String(0).set(0, "1");
        df1.Uint(1).set(0, 1);
        df1.Bit(2).set(1);
        BOOST_CHECK(df.Equal(df1) == false);
    }
    {
        // same
        CDataFrame<255> df1({eDF_String, eDF_Uint, eDF_Bit});
        df1.String(0).set(0, "1");
        df1.Uint(1).set(0, 1);
        df1.Bit(2).set(0);
        BOOST_CHECK(df.Equal(df1));
    }
}

BOOST_AUTO_TEST_CASE(Test_Serialization)
{
    CDataFrame<125> df({eDF_String, eDF_Uint, eDF_Bit});
    
    auto str_it = df.String(0).get_back_inserter();
    str_it.add("1"); str_it.add("2"); str_it.add("3"); str_it.flush();


    auto uint_it = df.Uint(1).get_back_inserter();
    uint_it.add(1); uint_it.add(2); uint_it.add(3); uint_it.flush();
    
    df.Bit(2).set(0);
    df.Bit(2).set(1);
    df.Bit(2).set(2);

    df.Optimize();

    ncbi::CTmpFile tmpfile;
    
    {
        ofstream out(tmpfile.GetFileName().c_str());
        df.Serialize(out);
    }

    ifstream t(tmpfile.GetFileName().c_str());
    string data((istreambuf_iterator<char>(t)), istreambuf_iterator<char>());
    {
        BOOST_TEST_CHECKPOINT( "Deserialize from start");
        CDataFrame<125> df1;
        df1.Deserialize(data.c_str(), data.size());
        BOOST_CHECK( df.Equal(df1));
    }

}


BOOST_AUTO_TEST_SUITE_END();

