/*  $Id: seqid_field_name_panel.cpp 45227 2020-06-18 18:26:59Z asztalos $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data,  the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties,  express or implied,  including
 *  warranties of performance,  merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 * Authors:  Igor Filippov
 */


#include <ncbi_pch.hpp>
#include <gui/objutils/macro_fn_where.hpp>
#include <gui/widgets/edit/field_type_constants.hpp>
#include <gui/widgets/edit/seqid_field_name_panel.hpp>
#include <wx/sizer.h>

BEGIN_NCBI_SCOPE

/*!
 * CSeqIdPanel type definition
 */

IMPLEMENT_DYNAMIC_CLASS(CSeqIdPanel, CFieldNamePanel)

/*!
 * CSeqIdPanel constructors
 */

CSeqIdPanel::CSeqIdPanel()
{
    Init();
}

CSeqIdPanel::CSeqIdPanel( wxWindow* parent)
{
    Init();
    Create(parent);
}

/*!
 * CSeqIdPanel creator
 */

bool CSeqIdPanel::Create(wxWindow* parent)
{
    CFieldNamePanel::Create(parent);
    if (GetSizer())
    {
        GetSizer()->SetSizeHints(this);
    }
    Centre();
    return true;
}

string CSeqIdPanel::GetFieldName(const bool subfield)
{
    return kFieldTypeSeqId;
}

bool CSeqIdPanel::SetFieldName(const string& field)
{
    return true;
}

string CSeqIdPanel::GetMacroFieldName(const string &target, const string& selected_field)
{ 
    return macro::CMacroFunction_SeqID::GetFuncName() + "()";
};


// CFileIdPanel
/*!
 * CFileIdPanel type definition
 */

IMPLEMENT_DYNAMIC_CLASS(CFileIdPanel, CFieldNamePanel)

/*!
 * CFileIdPanel constructors
 */

CFileIdPanel::CFileIdPanel()
{
    Init();
}

CFileIdPanel::CFileIdPanel(wxWindow* parent)
{
    Init();
    Create(parent);
}

/*!
 * CSeqIdPanel creator
 */

bool CFileIdPanel::Create(wxWindow* parent)
{
    CFieldNamePanel::Create(parent);
    if (GetSizer())
    {
        GetSizer()->SetSizeHints(this);
    }
    Centre();
    return true;
}

string CFileIdPanel::GetFieldName(const bool subfield)
{
    return kFileSeqId;
}

bool CFileIdPanel::SetFieldName(const string& field)
{
    return true;
}

string CFileIdPanel::GetMacroFieldName(const string &target, const string& selected_field)
{
    return macro::CMacroFunction_SeqID::GetFuncName() + "()";
};


END_NCBI_SCOPE
