/*  $Id: submitter_panel_depr.cpp 42777 2019-04-11 15:54:35Z asztalos $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data,  the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties,  express or implied,  including
 *  warranties of performance,  merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 * Authors:  Andrea Asztalos
 */


#include <ncbi_pch.hpp>
#include <objects/submit/Contact_info.hpp>
#include <objects/biblio/Author.hpp>
#include <objects/general/Person_id.hpp>
#include <objects/general/Name_std.hpp>

#include <gui/objutils/macro_fn_pubfields.hpp>
#include <gui/widgets/wx/wx_utils.hpp>
#include <gui/widgets/edit/singleauthor_panel.hpp>
#include <gui/widgets/edit/submitter_panel_depr.hpp>

#include <wx/sizer.h>
#include <wx/textctrl.h>
#include <wx/choice.h>
#include <wx/button.h>


BEGIN_NCBI_SCOPE
USING_SCOPE(objects);


/*
 * CSubmitterPanel_Depr type definition
 */

IMPLEMENT_DYNAMIC_CLASS(CSubmitterPanel_Depr, wxPanel )


/*
 * CSubmitterPanel_Depr event table definition
 */

BEGIN_EVENT_TABLE(CSubmitterPanel_Depr, wxPanel )

END_EVENT_TABLE()


/*
 * CSubmitterPanel_Depr constructors
 */

CSubmitterPanel_Depr::CSubmitterPanel_Depr()
{
    Init();
}

CSubmitterPanel_Depr::CSubmitterPanel_Depr( wxWindow* parent, CContact_info& object, wxWindowID id, const wxPoint& pos, const wxSize& size, long style )
    : m_Contact(&object)
{
    Init();
    Create(parent, id, pos, size, style);
}


/*
 * CSubmitterPanel_Depr creator
 */

bool CSubmitterPanel_Depr::Create( wxWindow* parent, wxWindowID id, const wxPoint& pos, const wxSize& size, long style )
{
////@begin CSubmitterPanel_Depr creation
    wxPanel::Create( parent, id, pos, size, style );

    CreateControls();
    if (GetSizer())
    {
        GetSizer()->SetSizeHints(this);
    }
    Centre();
////@end CSubmitterPanel_Depr creation
    return true;
}


/*
 * CSubmitterPanel_Depr destructor
 */

CSubmitterPanel_Depr::~CSubmitterPanel_Depr()
{
////@begin CSubmitterPanel_Depr destruction
////@end CSubmitterPanel_Depr destruction
}


/*
 * Member initialisation
 */

void CSubmitterPanel_Depr::Init()
{
////@begin CSubmitterPanel_Depr member initialisation
    m_First = NULL;
    m_MI = NULL;
    m_Last = NULL;
    m_Suffix = NULL;
    m_EmailPrimary = NULL;
    m_EmailSecondary = NULL;
////@end CSubmitterPanel_Depr member initialisation
}


/*
 * Control creation for CSubmitterPanel_Depr
 */

void CSubmitterPanel_Depr::CreateControls()
{    
////@begin CSubmitterPanel_Depr content construction
    // Generated by DialogBlocks, 19/12/2018 11:00:35 (unregistered)

    CSubmitterPanel_Depr* itemPanel1 = this;

    wxBoxSizer* itemBoxSizer2 = new wxBoxSizer(wxVERTICAL);
    itemPanel1->SetSizer(itemBoxSizer2);

    wxFlexGridSizer* itemFlexGridSizer1 = new wxFlexGridSizer(0, 2, 0, 0);
    itemBoxSizer2->Add(itemFlexGridSizer1, 0, wxALIGN_CENTER_HORIZONTAL | wxALL, 0);

    itemFlexGridSizer1->Add(5, 5, 0, wxALIGN_CENTER_HORIZONTAL | wxALIGN_CENTER_VERTICAL | wxALL, 5);

    wxFlexGridSizer* itemFlexGridSizer2 = new wxFlexGridSizer(2, 4, 0, 0);
    itemFlexGridSizer1->Add(itemFlexGridSizer2, 0, wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxStaticText* itemStaticText4 = new wxStaticText( itemPanel1, wxID_STATIC, _("First Name"), wxDefaultPosition, wxDefaultSize, 0 );
    itemFlexGridSizer2->Add(itemStaticText4, 0, wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxStaticText* itemStaticText5 = new wxStaticText( itemPanel1, wxID_STATIC, _("M.I."), wxDefaultPosition, wxDefaultSize, 0 );
    itemFlexGridSizer2->Add(itemStaticText5, 0, wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxStaticText* itemStaticText6 = new wxStaticText( itemPanel1, wxID_STATIC, _("Last Name"), wxDefaultPosition, wxDefaultSize, 0 );
    itemFlexGridSizer2->Add(itemStaticText6, 0, wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxStaticText* itemStaticText7 = new wxStaticText( itemPanel1, wxID_STATIC, _("Suffix"), wxDefaultPosition, wxDefaultSize, 0 );
    itemFlexGridSizer2->Add(itemStaticText7, 0, wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL|wxALL, 5);

    m_First = new wxTextCtrl( itemPanel1, ID_TXT_SUBMITTER1, wxEmptyString, wxDefaultPosition, wxDefaultSize, 0 );
    itemFlexGridSizer2->Add(m_First, 0, wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL|wxALL, 0);

    m_MI = new wxTextCtrl( itemPanel1, ID_TXT_SUBMITTER2, wxEmptyString, wxDefaultPosition, wxSize(50, -1), 0 );
    itemFlexGridSizer2->Add(m_MI, 0, wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL|wxALL, 0);

    m_Last = new wxTextCtrl( itemPanel1, ID_TXT_SUBMITTER3, wxEmptyString, wxDefaultPosition, wxDefaultSize, 0 );
    itemFlexGridSizer2->Add(m_Last, 0, wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL|wxALL, 0);

    wxArrayString m_SuffixStrings;
    m_SuffixStrings.Add(wxEmptyString);
    m_SuffixStrings.Add(_("Jr."));
    m_SuffixStrings.Add(_("Sr."));
    m_SuffixStrings.Add(_("II"));
    m_SuffixStrings.Add(_("III"));
    m_SuffixStrings.Add(_("IV"));
    m_SuffixStrings.Add(_("V"));
    m_SuffixStrings.Add(_("VI"));
    m_Suffix = new wxChoice( itemPanel1, ID_SUBMITTER_CHOICE, wxDefaultPosition, wxDefaultSize, m_SuffixStrings, 0 );
    itemFlexGridSizer2->Add(m_Suffix, 0, wxALIGN_CENTER_HORIZONTAL|wxALIGN_CENTER_VERTICAL|wxALL, 0);

    wxStaticText* itemStaticText8 = new wxStaticText( itemPanel1, wxID_STATIC, _("Email (primary)"), wxDefaultPosition, wxDefaultSize, 0 );
    itemFlexGridSizer1->Add(itemStaticText8, 0, wxALIGN_RIGHT|wxALIGN_CENTER_VERTICAL|wxALL, 5);

    m_EmailPrimary = new wxTextCtrl( itemPanel1, ID_TXT_SUBMITTER4, wxEmptyString, wxDefaultPosition, wxDefaultSize, 0 );
    itemFlexGridSizer1->Add(m_EmailPrimary, 0, wxGROW|wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxStaticText* itemStaticText9 = new wxStaticText(itemPanel1, wxID_STATIC, _("Email (secondary)"), wxDefaultPosition, wxDefaultSize, 0);
    itemFlexGridSizer1->Add(itemStaticText9, 0, wxALIGN_RIGHT | wxALIGN_CENTER_VERTICAL | wxALL, 5);

    m_EmailSecondary = new wxTextCtrl(itemPanel1, ID_TXT_SUBMITTER5, wxEmptyString, wxDefaultPosition, wxDefaultSize, 0);
    itemFlexGridSizer1->Add(m_EmailSecondary, 0, wxGROW | wxALIGN_CENTER_VERTICAL | wxALL, 5);

////@end CSubmitterPanel_Depr content construction
}


bool CSubmitterPanel_Depr::TransferDataToWindow()
{
    if (!wxPanel::TransferDataToWindow()) {
        return false;
    }

    if (m_Contact->IsSetContact() 
        && m_Contact->GetContact().IsSetName()
        && m_Contact->GetContact().GetName().IsName()) {
        const CName_std& person = m_Contact->GetContact().GetName().GetName();
        if (person.IsSetFirst()) {
            m_First->SetValue(person.GetFirst());
        }
        if (person.IsSetLast()) {
            m_Last->SetValue(person.GetLast());
        }
        if (person.IsSetSuffix()) {
            m_Suffix->SetStringSelection (person.GetSuffix());
        }
        if (person.IsSetInitials()) {
            string middle_init = CSingleAuthorPanel::s_ParseInitialsFromName(person);
            m_MI->SetValue(middle_init);
        }
    } 

    if (m_Contact->IsSetEmail()) {
        vector<string> emails;
        NStr::Split(m_Contact->GetEmail(), ",;", emails);
        if (!emails.empty()) {
            m_EmailPrimary->SetValue(ToWxString(emails[0]));
        }
        if (emails.size() > 1) {
            m_EmailSecondary->SetValue(ToWxString(emails[1]));
        }
    }
    
    return true;
}

bool CSubmitterPanel_Depr::TransferDataFromWindow()
{
    if (!wxPanel::TransferDataFromWindow()) {
        return false;
    }

    CName_std& person = m_Contact->SetContact().SetName().SetName();
    macro::CMacroFunction_AuthorFix::s_BuildName(
        ToAsciiStdString(m_First->GetValue()),
        ToAsciiStdString(m_MI->GetValue()),
        ToAsciiStdString(m_Last->GetValue()),
        ToAsciiStdString(m_Suffix->GetStringSelection()),
        person);

    string email;
    if (!m_EmailPrimary->IsEmpty()) {
        email += ToStdString(m_EmailPrimary->GetValue());
    }
    if (!m_EmailSecondary->IsEmpty()) {
        if (!email.empty()) {
            email += "; ";
        }
        email += ToStdString(m_EmailSecondary->GetValue());
    }
    if (!email.empty()) {
        m_Contact->SetEmail(email);
    }
    return true;
}

void CSubmitterPanel_Depr::SetData(const CContact_info& contact)
{
    m_Contact->Assign(contact);
    TransferDataToWindow();
}

/*
bool CSubmitterPanel_Depr::IsContactNameSet() const
{
    return (!m_First->IsEmpty() && !m_Last->IsEmpty());
}

bool CSubmitterPanel_Depr::IsOneEmailSet() const
{
    return (!m_EmailPrimary->IsEmpty() || !m_EmailSecondary->IsEmpty());
}
*/

void CSubmitterPanel_Depr::SetFocusOnFirstName()
{
    if (m_First) 
        m_First->SetFocus();
}

void CSubmitterPanel_Depr::SetFocusOnLastName()
{
    if (m_Last) 
        m_Last->SetFocus();
}

void CSubmitterPanel_Depr::SetFocusOnFirstEmail()
{
    if (m_EmailPrimary)
        m_EmailPrimary->SetFocus();
}

/*
 * Should we show tooltips?
 */

bool CSubmitterPanel_Depr::ShowToolTips()
{
    return true;
}

/*
 * Get bitmap resources
 */

wxBitmap CSubmitterPanel_Depr::GetBitmapResource( const wxString& name )
{
    // Bitmap retrieval
////@begin CSubmitterPanel_Depr bitmap retrieval
    wxUnusedVar(name);
    return wxNullBitmap;
////@end CSubmitterPanel_Depr bitmap retrieval
}

/*
 * Get icon resources
 */

wxIcon CSubmitterPanel_Depr::GetIconResource( const wxString& name )
{
    // Icon retrieval
////@begin CSubmitterPanel_Depr icon retrieval
    wxUnusedVar(name);
    return wxNullIcon;
////@end CSubmitterPanel_Depr icon retrieval
}

END_NCBI_SCOPE
