/*  $Id: columnar_vcf_cache.cpp 46153 2021-01-26 00:00:32Z evgeniev $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data, the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties, express or implied, including
 *  warranties of performance, merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 * Authors:  Vladislav Evgeniev
 *
 * File Description:
 *   
 *
 */

#include <ncbi_pch.hpp>
#include <gui/widgets/loaders/columnar_vcf_cache.hpp>
#include <gui/widgets/loaders/columnar_vcf_variants_base.hpp>

BEGIN_NCBI_SCOPE

static const string kVcfCache = "ColumnarVcfCache";

CColumnarVcfCache::CColumnarVcfCache(const string &url, const string &seq_id)
    : m_Url(url)
    , m_SeqId(seq_id)
{
    x_Initialize();
}

CNetICacheClient CColumnarVcfCache::x_InitBlobCache()
{
    return x_InitICache(kVcfCache, "NC_SV_ObjCache_DEV", "vcf_blobs");
}

CNetICacheClient CColumnarVcfCache::x_InitSyncCache()
{
    return x_InitICache(kVcfCache, "NC_SV_ObjCache_DEV", "sync");
}

string CColumnarVcfCache::x_GetCacheKey()
{
    string key { m_SeqId };
    key += '_';
    key += m_Url;
    key += '_';
    key += CVCFVariantsBase::s_GetCurrentVersion(true);
    return key;
}

END_NCBI_SCOPE
