/*  $Id: vcf_sample_sel_panel.cpp 45881 2021-01-05 03:10:02Z rudnev $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data,  the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties,  express or implied,  including
 *  warranties of performance,  merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 * Authors: Andrea Asztalos
 */


#include <ncbi_pch.hpp>
#include <gui/widgets/wx/wx_utils.hpp>
#include <gui/widgets/loaders/vcf_sample_sel_panel.hpp>
#include <wx/sizer.h>
#include <wx/checklst.h>
#include <wx/button.h>
#include <wx/wupdlock.h>
#include <wx/stattext.h>


BEGIN_NCBI_SCOPE

/*!
 * CVcfSampleSelectionPanel type definition
 */

IMPLEMENT_DYNAMIC_CLASS( CVcfSampleSelectionPanel, wxPanel )


/*!
 * CVcfSampleSelectionPanel event table definition
 */

BEGIN_EVENT_TABLE( CVcfSampleSelectionPanel, wxPanel )

////@begin CVcfSampleSelectionPanel event table entries
    EVT_BUTTON( ID_SELECTALLSAMPLES, CVcfSampleSelectionPanel::OnSelectallSamplesClick )
    EVT_BUTTON( ID_DESELECTALLSAMPLES, CVcfSampleSelectionPanel::OnDeselectallSamplesClick )
////@end CVcfSampleSelectionPanel event table entries

END_EVENT_TABLE()


/*!
 * CVcfSampleSelectionPanel constructors
 */

CVcfSampleSelectionPanel::CVcfSampleSelectionPanel()
{
    Init();
}

CVcfSampleSelectionPanel::CVcfSampleSelectionPanel( wxWindow* parent, wxWindowID id, const wxPoint& pos, const wxSize& size, long style )
{
    Init();
    Create(parent, id, pos, size, style);
}


/*!
 * CVcfSampleSelectionPanel creator
 */

bool CVcfSampleSelectionPanel::Create( wxWindow* parent, wxWindowID id, const wxPoint& pos, const wxSize& size, long style )
{
////@begin CVcfSampleSelectionPanel creation
    SetExtraStyle(wxWS_EX_VALIDATE_RECURSIVELY);
    wxPanel::Create( parent, id, pos, size, style );

    CreateControls();
    if (GetSizer())
    {
        GetSizer()->SetSizeHints(this);
    }
    Centre();
////@end CVcfSampleSelectionPanel creation
    return true;
}


/*!
 * CVcfSampleSelectionPanel destructor
 */

CVcfSampleSelectionPanel::~CVcfSampleSelectionPanel()
{
////@begin CVcfSampleSelectionPanel destruction
////@end CVcfSampleSelectionPanel destruction
}


/*!
 * Member initialisation
 */

void CVcfSampleSelectionPanel::Init()
{
////@begin CVcfSampleSelectionPanel member initialisation
    m_SampleList = NULL;
////@end CVcfSampleSelectionPanel member initialisation
}


/*!
 * Control creation for CVcfSampleSelectionPanel
 */

void CVcfSampleSelectionPanel::CreateControls()
{    
////@begin CVcfSampleSelectionPanel content construction
    CVcfSampleSelectionPanel* itemPanel1 = this;

    wxBoxSizer* itemBoxSizer2 = new wxBoxSizer(wxVERTICAL);
    itemPanel1->SetSizer(itemBoxSizer2);

    wxStaticText* itemStaticText1 = new wxStaticText( itemPanel1, wxID_STATIC, _("Samples Columns"), wxDefaultPosition, wxDefaultSize, 0 );
    itemBoxSizer2->Add(itemStaticText1, 0, wxALIGN_LEFT|wxALL, 5);

    wxArrayString m_SampleListStrings;
    m_SampleList = new wxCheckListBox( itemPanel1, ID_SAMPLECHKLIST, wxDefaultPosition, wxDefaultSize, m_SampleListStrings, wxLB_SINGLE );
    itemBoxSizer2->Add(m_SampleList, 1, wxGROW|wxALL, 5);

    wxBoxSizer* itemBoxSizer1 = new wxBoxSizer(wxHORIZONTAL);
    itemBoxSizer2->Add(itemBoxSizer1, 0, wxALIGN_LEFT|wxTOP, 5);

    wxButton* itemButton2 = new wxButton( itemPanel1, ID_SELECTALLSAMPLES, _("Select All"), wxDefaultPosition, wxDefaultSize, 0 );
    itemBoxSizer1->Add(itemButton2, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxButton* itemButton3 = new wxButton( itemPanel1, ID_DESELECTALLSAMPLES, _("Deselect All"), wxDefaultPosition, wxDefaultSize, 0 );
    itemBoxSizer1->Add(itemButton3, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);

////@end CVcfSampleSelectionPanel content construction
}

void CVcfSampleSelectionPanel::SetSamples(const map<unsigned, string>& samples)
{
    wxWindowUpdateLocker upd(m_SampleList);
    m_SampleList->Clear();

    for (const auto& it : samples) {
        m_SampleList->Append(it.second);
    }
}

void CVcfSampleSelectionPanel::GetSelectedSamples(map<unsigned, string>& selected)
{
    selected.clear();
    for (unsigned i = 0; i < m_SampleList->GetCount(); ++i) {
        if (m_SampleList->IsChecked(i)) {
            selected.emplace(i, ToStdString(m_SampleList->GetString(i)));
        }
    }
}

bool CVcfSampleSelectionPanel::AllAreChecked() const
{
    wxArrayInt checkedItems;
    m_SampleList->GetCheckedItems(checkedItems);
    return (checkedItems.size() == m_SampleList->GetCount());
}

/*!
 * Should we show tooltips?
 */

bool CVcfSampleSelectionPanel::ShowToolTips()
{
    return true;
}

/*!
 * Get bitmap resources
 */

wxBitmap CVcfSampleSelectionPanel::GetBitmapResource( const wxString& name )
{
    // Bitmap retrieval
////@begin CVcfSampleSelectionPanel bitmap retrieval
    wxUnusedVar(name);
    return wxNullBitmap;
////@end CVcfSampleSelectionPanel bitmap retrieval
}

/*!
 * Get icon resources
 */

wxIcon CVcfSampleSelectionPanel::GetIconResource( const wxString& name )
{
    // Icon retrieval
////@begin CVcfSampleSelectionPanel icon retrieval
    wxUnusedVar(name);
    return wxNullIcon;
////@end CVcfSampleSelectionPanel icon retrieval
}


/*!
 * wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_SELECTALLSAMPLES
 */

void CVcfSampleSelectionPanel::OnSelectallSamplesClick(wxCommandEvent& event)
{
    for (unsigned i = 0; i < m_SampleList->GetCount(); ++i)
        m_SampleList->Check(i);
}


/*!
 * wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_DESELECTALLSAMPLES
 */

void CVcfSampleSelectionPanel::OnDeselectallSamplesClick(wxCommandEvent& event)
{
    for (unsigned i = 0; i < m_SampleList->GetCount(); ++i)
        m_SampleList->Check(i, false);
}

END_NCBI_SCOPE

