/*  $Id: wx_label_edit_dlg.cpp 29104 2013-10-22 18:01:10Z falkrb $
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data, the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties, express or implied, including
 *  warranties of performance, merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 * Authors: Vladimir Tereshkov
 *
 * File Description:
 *
 */
#include <ncbi_pch.hpp>////@begin includes
////@end includes

#include <gui/widgets/phylo_tree/wx_label_edit_dlg.hpp>
#include <gui/widgets/phylo_tree/phylo_tree_label.hpp>
#include <gui/widgets/phylo_tree/phylo_tree.hpp>
#include <gui/widgets/wx/dialog_utils.hpp>

#include <wx/panel.h>
#include <wx/stattext.h>
#include <wx/statline.h>
#include <wx/statbox.h>
#include <wx/combobox.h>
#include <wx/sizer.h>
#include <wx/button.h>
#include <wx/bitmap.h>
#include <wx/icon.h>

#define PHYLO_FEAT_LABEL "label"

////@begin XPM images
////@end XPM images

BEGIN_NCBI_SCOPE

IMPLEMENT_DYNAMIC_CLASS( CwxLabelEditDlg, wxDialog )

BEGIN_EVENT_TABLE( CwxLabelEditDlg, wxDialog )

////@begin CwxLabelEditDlg event table entries
    EVT_BUTTON( wxID_OK, CwxLabelEditDlg::OnOkClick )

////@end CwxLabelEditDlg event table entries

END_EVENT_TABLE()

CwxLabelEditDlg::CwxLabelEditDlg()
: m_Tree(NULL)
, m_NodeIdx(CPhyloTree::Null())
{
    Init();
}

CwxLabelEditDlg::CwxLabelEditDlg( wxWindow* parent, wxWindowID id, const wxString& caption, const wxPoint& pos, const wxSize& size, long style )
: m_Tree(NULL)
, m_NodeIdx(CPhyloTree::Null())
{
    Init();
    Create(parent, id, caption, pos, size, style);
}

bool CwxLabelEditDlg::Create( wxWindow* parent, wxWindowID id, const wxString& caption, const wxPoint& pos, const wxSize& size, long style )
{
////@begin CwxLabelEditDlg creation
    SetExtraStyle(wxWS_EX_BLOCK_EVENTS);
    wxDialog::Create( parent, id, caption, pos, size, style );

    CreateControls();
    if (GetSizer())
    {
        GetSizer()->SetSizeHints(this);
    }
    Centre();
////@end CwxLabelEditDlg creation
    return true;
}

CwxLabelEditDlg::~CwxLabelEditDlg()
{
////@begin CwxLabelEditDlg destruction
////@end CwxLabelEditDlg destruction
}

void CwxLabelEditDlg::Init()
{
////@begin CwxLabelEditDlg member initialisation
////@end CwxLabelEditDlg member initialisation
}

void CwxLabelEditDlg::CreateControls()
{
////@begin CwxLabelEditDlg content construction
    // Generated by DialogBlocks, Sat 16 Feb 2008 11:12:46 AM EST (unregistered)

    CwxLabelEditDlg* itemDialog1 = this;

    wxBoxSizer* itemBoxSizer2 = new wxBoxSizer(wxHORIZONTAL);
    itemDialog1->SetSizer(itemBoxSizer2);

    wxTextCtrl* itemTextCtrl3 = new wxTextCtrl( itemDialog1, ID_TEXTCTRL16, wxT(""), wxDefaultPosition, wxDefaultSize, 0 );
    itemBoxSizer2->Add(itemTextCtrl3, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxStdDialogButtonSizer* itemStdDialogButtonSizer4 = new wxStdDialogButtonSizer;

    itemBoxSizer2->Add(itemStdDialogButtonSizer4, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);
    wxButton* itemButton5 = new wxButton( itemDialog1, wxID_OK, wxT("&OK"), wxDefaultPosition, wxDefaultSize, 0 );
    itemButton5->SetDefault();
    itemStdDialogButtonSizer4->AddButton(itemButton5);

    itemStdDialogButtonSizer4->Realize();

    // Set validators
    itemTextCtrl3->SetValidator( wxTextValidator(wxFILTER_NONE, & m_Label) );
////@end CwxLabelEditDlg content construction
}

bool CwxLabelEditDlg::ShowToolTips()
{
    return true;
}
wxBitmap CwxLabelEditDlg::GetBitmapResource( const wxString& name )
{
    // Bitmap retrieval
////@begin CwxLabelEditDlg bitmap retrieval
    wxUnusedVar(name);
    return wxNullBitmap;
////@end CwxLabelEditDlg bitmap retrieval
}
wxIcon CwxLabelEditDlg::GetIconResource( const wxString& name )
{
    // Icon retrieval
////@begin CwxLabelEditDlg icon retrieval
    wxUnusedVar(name);
    return wxNullIcon;
////@end CwxLabelEditDlg icon retrieval
}

void CwxLabelEditDlg::SetParams(CPhyloTree* tree,
                                CPhyloTree::TTreeIdx node_idx)
{
    m_Tree = tree;
    m_NodeIdx = node_idx; 
    m_Label.erase();

    m_Label = ToWxString((*tree)[node_idx]->GetLabel());
}

void CwxLabelEditDlg::OnOkClick( wxCommandEvent& event )
{
    TransferDataFromWindow();   
    event.Skip();
}

END_NCBI_SCOPE
