/*  
 * ===========================================================================
 *
 *                            PUBLIC DOMAIN NOTICE
 *               National Center for Biotechnology Information
 *
 *  This software/database is a "United States Government Work" under the
 *  terms of the United States Copyright Act.  It was written as part of
 *  the author's official duties as a United States Government employee and
 *  thus cannot be copyrighted.  This software/database is freely available
 *  to the public for use. The National Library of Medicine and the U.S.
 *  Government have not placed any restriction on its use or reproduction.
 *
 *  Although all reasonable efforts have been taken to ensure the accuracy
 *  and reliability of the software and data, the NLM and the U.S.
 *  Government do not and cannot warrant the performance or results that
 *  may be obtained by using this software or data. The NLM and the U.S.
 *  Government disclaim all warranties, express or implied, including
 *  warranties of performance, merchantability or fitness for any particular
 *  purpose.
 *
 *  Please cite the author in any work or product based on this material.
 *
 * ===========================================================================
 *
 * Authors:  Vladislav Evgeniev
 *
 * File Description:
 *
 */

#include <ncbi_pch.hpp>
#include "columnar_vcf_project_item_extension.hpp"
#include <gui/widgets/seq_graphic/vcf_histogram.hpp>
#include <gui/objutils/project_item_extra.hpp>
#include <objects/gbproj/ProjectItem.hpp>
#include <objects/gbproj/AbstractProjectItem.hpp>
#include <objects/seq/Annotdesc.hpp>

BEGIN_NCBI_SCOPE
USING_SCOPE(objects);

///////////////////////////////////////////////////////////////////////////////
/// CColumnarVcfProjectItemExtension

string CColumnarVcfProjectItemExtension::GetExtensionIdentifier() const
{
    static string sid("columnar_vcf_project_item_extension");
    return sid;
}

string CColumnarVcfProjectItemExtension::GetExtensionLabel() const
{
    static string slabel("Columnar VCF Project item extension");
    return slabel;
}

void CColumnarVcfProjectItemExtension::ProjectItemDetached(CProjectItem& prj_item, CGBDocument& /*doc*/, IServiceLocator* /*service_locator*/)
{
    do{
        if (!prj_item.IsSetDescr())
            break;

        if (!prj_item.GetItem().IsOther())
            break;

        if (prj_item.GetItem().GetOther().GetType() != "ColumnarVCF")
            break;

        string annot_name;
        if (!CProjectItemExtra::GetStr(prj_item, "AnnotName", annot_name))
            break;

        CVcfHistogram::EraseCachePerAnnotation(annot_name);

    } while (false);
}

END_NCBI_SCOPE
